@@ -33,7 +33,9 @@
 #include <gtkmm/accelmap.h>
 
 #include <pbd/error.h>
+#include <pbd/basename.h>
 #include <pbd/compose.h>
+#include <pbd/misc.h>
 #include <pbd/pathscanner.h>
 #include <pbd/failed_constructor.h>
 #include <pbd/enumwriter.h>
@@ -75,7 +77,8 @@
 #include "about.h"
 #include "utils.h"
 #include "gui_thread.h"
-#include "color_manager.h"
+#include "theme_manager.h"
+#include "engine_dialog.h"
 
 #include "i18n.h"
 
@@ -86,16 +89,17 @@ using namespace Gtk;
 using namespace sigc;
 
 ARDOUR_UI *ARDOUR_UI::theArdourUI = 0;
+UIConfiguration *ARDOUR_UI::ui_config = 0;
 
 sigc::signal<void,bool> ARDOUR_UI::Blink;
 sigc::signal<void>      ARDOUR_UI::RapidScreenUpdate;
 sigc::signal<void>      ARDOUR_UI::SuperRapidScreenUpdate;
-sigc::signal<void,nframes_t> ARDOUR_UI::Clock;
+sigc::signal<void,nframes_t, bool, nframes_t> ARDOUR_UI::Clock;
 
-ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], string rcfile)
+ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
+
+	: Gtkmm2ext::UI (X_("Ardour"), argcp, argvp),
 
-	: Gtkmm2ext::UI (X_("Ardour"), argcp, argvp, rcfile),
-	  
 	  primary_clock (X_("primary"), false, X_("TransportClockDisplay"), true, false, true),
 	  secondary_clock (X_("secondary"), false, X_("SecondaryClockDisplay"), true, false, true),
 	  preroll_clock (X_("preroll"), false, X_("PreRollClock"), true, true),
@@ -112,7 +116,7 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], string rcfile)
 
 	  /* big clock */
 
-	  big_clock (X_("bigclock"), false, "BigClockNonRecording", false, false, true),
+	  big_clock (X_("bigclock"), false, "BigClockNonRecording", true, false, true),
 
 	  /* transport */
 
@@ -146,11 +150,20 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], string rcfile)
 
 	  auditioning_alert_button (_("AUDITION")),
 	  solo_alert_button (_("SOLO")),
-	  shown_flag (false)
+	  shown_flag (false),
+	  error_log_button (_("Errors"))
 {
 	using namespace Gtk::Menu_Helpers;
 
 	Gtkmm2ext::init();
+
+#ifdef TOP_MENUBAR
+	_auto_display_errors = false;
+#endif
+
+	if (getenv ("ARDOUR_DEBUG_UPDATES")) {
+		gdk_window_set_debug_updates (true);
+	}
 	
 	about = 0;
 
@@ -158,33 +171,37 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], string rcfile)
 		theArdourUI = this;
 	}
 
-	/* load colors */
-
-	color_manager = new ColorManager();
-
-	std::string color_file = ARDOUR::find_config_file("ardour.colors");
-
-	color_manager->load (color_file);
+	ui_config = new UIConfiguration();
+	theme_manager = new ThemeManager();
 
+	engine = 0;
 	editor = 0;
 	mixer = 0;
 	session = 0;
 	_session_is_new = false;
 	big_clock_window = 0;
 	session_selector_window = 0;
+	new_session_dialog = 0;
 	last_key_press_time = 0;
 	connection_editor = 0;
 	add_route_dialog = 0;
 	route_params = 0;
 	option_editor = 0;
 	location_ui = 0;
+	key_editor = 0;
 	open_session_selector = 0;
 	have_configure_timeout = false;
 	have_disk_speed_dialog_displayed = false;
 	_will_create_new_session_automatically = false;
 	session_loaded = false;
+	loading_dialog = 0;
 	last_speed_displayed = -1.0f;
+
 	keybindings_path = ARDOUR::find_config_file ("ardour.bindings");
+	/* all changes go to the user directory */
+	user_keybindings_path = get_user_ardour_path ();
+	user_keybindings_path += '/';
+	user_keybindings_path += "ardour.bindings";
 
 	can_save_keybindings = false;
 
@@ -208,31 +225,78 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], string rcfile)
 
 	ARDOUR::Session::AskAboutPendingState.connect (mem_fun(*this, &ARDOUR_UI::pending_state_dialog));
 
-	/* have to wait for AudioEngine and Configuration before proceeding */
+	/* lets get this party started */
+
+    	try {
+		if (ARDOUR::init (ARDOUR_COMMAND_LINE::use_vst, ARDOUR_COMMAND_LINE::try_hw_optimization)) {
+			throw failed_constructor ();
+		}
+
+		setup_gtk_ardour_enums ();
+		Config->set_current_owner (ConfigVariableBase::Interface);
+		setup_profile ();
+
+	} catch (failed_constructor& err) {
+		error << _("could not initialize Ardour.") << endmsg;
+		// pass it on up
+		throw;
+	} 
+
+	/* we like keyboards */
+
+	keyboard = new Keyboard;
+
+	starting.connect (mem_fun(*this, &ARDOUR_UI::startup));
+	stopping.connect (mem_fun(*this, &ARDOUR_UI::shutdown));
+
+	platform_setup ();
 }
 
-void
-ARDOUR_UI::set_engine (AudioEngine& e)
+int
+ARDOUR_UI::create_engine ()
 {
-	engine = &e;
+	// this gets called every time by new_session()
+
+	if (engine) {
+		return 0;
+	}
+
+	try { 
+		engine = new ARDOUR::AudioEngine (ARDOUR_COMMAND_LINE::jack_client_name);
+
+	} catch (...) {
+
+		return -1;
+	}
 
 	engine->Stopped.connect (mem_fun(*this, &ARDOUR_UI::engine_stopped));
 	engine->Running.connect (mem_fun(*this, &ARDOUR_UI::engine_running));
 	engine->Halted.connect (mem_fun(*this, &ARDOUR_UI::engine_halted));
 	engine->SampleRateChanged.connect (mem_fun(*this, &ARDOUR_UI::update_sample_rate));
 
+	post_engine ();
+
+	return 0;
+}
+
+void
+ARDOUR_UI::post_engine ()
+{
+	/* Things to be done once we create the AudioEngine
+	 */
+
+	check_memory_locking();
+
        	ActionManager::init ();
-	new_session_dialog = new NewSessionDialog();
-
 	_tooltips.enable();
 
-	keyboard = new Keyboard;
-
 	if (setup_windows ()) {
 		throw failed_constructor ();
 	}
 
-	if (GTK_ARDOUR::show_key_actions) {
+	/* this is the first point at which all the keybindings are available */
+
+	if (ARDOUR_COMMAND_LINE::show_key_actions) {
 		vector<string> names;
 		vector<string> paths;
 		vector<string> keys;
@@ -249,9 +313,6 @@ ARDOUR_UI::set_engine (AudioEngine& e)
 		exit (0);
 	}
 
-	/* start with timecode, metering enabled
-	*/
-	
 	blink_timeout_tag = -1;
 
 	/* the global configuration object is now valid */
@@ -270,15 +331,28 @@ ARDOUR_UI::set_engine (AudioEngine& e)
 
 	/* start the time-of-day-clock */
 	
+#ifndef GTKOSX
+	/* OS X provides an always visible wallclock, so don't be stupid */
 	update_wall_clock ();
 	Glib::signal_timeout().connect (mem_fun(*this, &ARDOUR_UI::update_wall_clock), 60000);
+#endif
 
 	update_disk_space ();
 	update_cpu_load ();
 	update_sample_rate (engine->frame_rate());
 
-	starting.connect (mem_fun(*this, &ARDOUR_UI::startup));
-	stopping.connect (mem_fun(*this, &ARDOUR_UI::shutdown));
+	platform_specific ();
+
+	/* now start and maybe save state */
+
+	if (do_engine_start () == 0) {
+		if (session && _session_is_new) {
+			/* we need to retain initial visual 
+			   settings for a new session 
+			*/
+			session->save_state ("");
+		}
+	}
 }
 
 ARDOUR_UI::~ARDOUR_UI ()
@@ -300,6 +374,10 @@ ARDOUR_UI::~ARDOUR_UI ()
 	if (add_route_dialog) {
 		delete add_route_dialog;
 	}
+
+	if (new_session_dialog) {
+		delete new_session_dialog;
+	}
 }
 
 gint
@@ -337,7 +415,7 @@ ARDOUR_UI::configure_handler (GdkEventConfigure* conf)
 		Glib::signal_timeout().connect (mem_fun(*this, &ARDOUR_UI::configure_timeout), 100);
 		have_configure_timeout = true;
 	}
-		
+	
 	return FALSE;
 }
 
@@ -412,7 +490,13 @@ ARDOUR_UI::save_ardour_state ()
 	XMLNode* node = new XMLNode (keyboard->get_state());
 	Config->add_extra_xml (*node);
 	Config->add_extra_xml (get_transport_controllable_state());
+	if (new_session_dialog) {
+		if (new_session_dialog->engine_control.was_used()) {
+			Config->add_extra_xml (new_session_dialog->engine_control.get_state());
+		}
+	}
 	Config->save_state();
+	ui_config->save_state ();
 
 	XMLNode enode(static_cast<Stateful*>(editor)->get_state());
 	XMLNode mnode(mixer->get_state());
@@ -461,10 +545,125 @@ ARDOUR_UI::update_autosave ()
         }
 }
 
+void
+ARDOUR_UI::backend_audio_error (bool we_set_params, Gtk::Window* toplevel)
+{
+	string title;
+	if (we_set_params) {
+		title = _("Ardour could not start JACK");
+	} else {
+		title = _("Ardour could not connect to JACK.");
+	}
+
+	MessageDialog win (title,
+			   false,
+			   Gtk::MESSAGE_INFO,
+			   Gtk::BUTTONS_NONE);
+	
+	if (we_set_params) {
+		win.set_secondary_text(_("There are several possible reasons:\n\
+\n\
+1) You requested audio parameters that are not supported..\n\
+2) JACK is running as another user.\n\
+\n\
+Please consider the possibilities, and perhaps try different parameters."));
+	} else {
+		win.set_secondary_text(_("There are several possible reasons:\n\
+\n\
+1) JACK is not running.\n\
+2) JACK is running as another user, perhaps root.\n\
+3) There is already another client called \"ardour\".\n\
+\n\
+Please consider the possibilities, and perhaps (re)start JACK."));
+	}
+
+	if (toplevel) {
+		win.set_transient_for (*toplevel);
+	}
+
+	if (we_set_params) {
+		win.add_button (Stock::OK, RESPONSE_CLOSE);
+	} else {
+		win.add_button (Stock::QUIT, RESPONSE_CLOSE);
+	}
+
+	win.set_default_response (RESPONSE_CLOSE);
+	
+	win.show_all ();
+	win.set_position (Gtk::WIN_POS_CENTER);
+
+	if (!ARDOUR_COMMAND_LINE::no_splash) {
+		hide_splash ();
+	}
+
+	/* we just don't care about the result, but we want to block */
+
+	win.run ();
+}
+
 void
 ARDOUR_UI::startup ()
 {
-	check_memory_locking();
+	string name, path;
+	bool isnew;
+
+	new_session_dialog = new NewSessionDialog();
+	
+	/* If no session name is given: we're not loading a session yet, nor creating a new one */
+	
+	if (ARDOUR_COMMAND_LINE::session_name.length()) {
+	
+		/* Load session or start the new session dialog */
+		
+		if (Session::find_session (ARDOUR_COMMAND_LINE::session_name, path, name, isnew)) {
+			
+			MessageDialog msg (string_compose(_("Could not find command line session \"%1\""), 
+							  ARDOUR_COMMAND_LINE::session_name),
+					   true,
+					   Gtk::MESSAGE_ERROR,
+					   Gtk::BUTTONS_OK);
+			
+			msg.set_position (Gtk::WIN_POS_MOUSE);
+			msg.present ();
+			msg.run ();
+			
+			exit (1);
+		}
+
+		if (!ARDOUR_COMMAND_LINE::new_session) {
+			
+			/* Supposed to be loading an existing session, but the session doesn't exist */
+			
+			if (isnew) {
+				MessageDialog msg (string_compose (_("\n\nNo session named \"%1\" exists.\n"
+								     "To create it from the command line, start ardour as:\n   ardour --new %1"), path),
+						   true,
+						   Gtk::MESSAGE_ERROR,
+						   Gtk::BUTTONS_OK);
+
+				msg.set_position (Gtk::WIN_POS_MOUSE);
+				msg.present ();
+				msg.run ();
+
+				exit (1);
+			}
+		}
+	}
+
+	hide_splash ();
+
+	bool have_backend = EngineControl::engine_running();
+	XMLNode* audio_setup = Config->extra_xml ("AudioSetup");
+	
+	if (audio_setup) {
+		new_session_dialog->engine_control.set_state (*audio_setup);
+	}
+	
+	if (!get_session_parameters (ARDOUR_COMMAND_LINE::session_name, have_backend, ARDOUR_COMMAND_LINE::new_session)) {
+		return;
+	}
+	
+	show ();
 }
 
 void
@@ -521,7 +720,6 @@ ARDOUR_UI::check_memory_locking ()
 				vbox->pack_start (hbox);
 				hbox.show_all ();
 				
-				editor->ensure_float (msg);
 				msg.run ();
 			}
 		}
@@ -533,34 +731,41 @@ ARDOUR_UI::check_memory_locking ()
 void
 ARDOUR_UI::finish()
 {
-	if (session && session->dirty()) {
-		switch (ask_about_saving_session(_("quit"))) {
-		case -1:
-			return;
-			break;
-		case 1:
-			/* use the default name */
-			if (save_state_canfail ("")) {
-				/* failed - don't quit */
-				MessageDialog msg (*editor, 
-					       _("\
+	if (session) {
+
+		if (session->transport_rolling()) {
+			session->request_stop ();
+			usleep (2500000);
+		}
+
+		if (session->dirty()) {
+			switch (ask_about_saving_session(_("quit"))) {
+			case -1:
+				return;
+				break;
+			case 1:
+				/* use the default name */
+				if (save_state_canfail ("")) {
+					/* failed - don't quit */
+					MessageDialog msg (*editor, 
+							   _("\
 Ardour was unable to save your session.\n\n\
 If you still wish to quit, please use the\n\n\
 \"Just quit\" option."));
-				msg.run ();
-				return;
+					msg.run ();
+					return;
+				}
+				break;
+			case 0:
+				break;
 			}
-			break;
-		case 0:
-			break;
 		}
-	}
-
-	if (session) {
+		
 		session->set_deletion_in_progress ();
 	}
+
 	engine->stop (true);
-	Config->save_state();
+	save_ardour_state ();
 	quit ();
 }
 
@@ -635,7 +840,7 @@ ARDOUR_UI::ask_about_saving_session (const string & what)
 	return -1;
 }
 	
-gint
+int
 ARDOUR_UI::every_second ()
 {
 	update_cpu_load ();
@@ -655,6 +860,8 @@ ARDOUR_UI::every_point_one_seconds ()
 gint
 ARDOUR_UI::every_point_zero_one_seconds ()
 {
+	// august 2007: actual update frequency: 40Hz, not 100Hz
+
 	SuperRapidScreenUpdate(); /* EMIT_SIGNAL */
 	return TRUE;
 }
@@ -865,8 +1072,7 @@ ARDOUR_UI::build_session_selector ()
 	recent_session_display.set_model (recent_session_model);
 	recent_session_display.append_column (_("Recent Sessions"), recent_session_columns.visible_name);
 	recent_session_display.set_headers_visible (false);
-	recent_session_display.get_selection()->set_mode (SELECTION_SINGLE);
-
+	recent_session_display.get_selection()->set_mode (SELECTION_BROWSE);
 	recent_session_display.signal_row_activated().connect (mem_fun (*this, &ARDOUR_UI::recent_session_row_activated));
 
 	scroller->add (recent_session_display);
@@ -887,37 +1093,55 @@ ARDOUR_UI::recent_session_row_activated (const TreePath& path, TreeViewColumn* c
 void
 ARDOUR_UI::open_recent_session ()
 {
-	/* popup selector window */
+	bool can_return = (session != 0);
 
 	if (session_selector_window == 0) {
 		build_session_selector ();
 	}
-
+	
 	redisplay_recent_sessions ();
 
-	ResponseType r = (ResponseType) session_selector_window->run ();
+	while (true) {
+		
+		session_selector_window->set_position (WIN_POS_MOUSE);
 
-	session_selector_window->hide();
+		ResponseType r = (ResponseType) session_selector_window->run ();
+		
+		switch (r) {
+		case RESPONSE_ACCEPT:
+			break;
+		default:
+			if (can_return) {
+				session_selector_window->hide();
+				return;
+			} else {
+				exit (1);
+			}
+		}
 
-	switch (r) {
-	case RESPONSE_ACCEPT:
-		break;
-	default:
-		return;
+		if (recent_session_display.get_selection()->count_selected_rows() == 0) {
+			continue;
+		}
+		
+		session_selector_window->hide();
+
+		Gtk::TreeModel::iterator i = recent_session_display.get_selection()->get_selected();
+		
+		if (i == recent_session_model->children().end()) {
+			return;
+		}
+		
+		Glib::ustring path = (*i)[recent_session_columns.fullpath];
+		Glib::ustring state = (*i)[recent_session_columns.visible_name];
+		
+		_session_is_new = false;
+		
+		if (load_session (path, state) == 0) {
+			break;
+		}
+
+		can_return = false;
 	}
-
-	Gtk::TreeModel::iterator i = recent_session_display.get_selection()->get_selected();
-
-	if (i == recent_session_model->children().end()) {
-		return;
-	}
-	
-	Glib::ustring path = (*i)[recent_session_columns.fullpath];
-	Glib::ustring state = (*i)[recent_session_columns.visible_name];
-
-	_session_is_new = false;
-
-	load_session (path, state);
 }
 
 bool
@@ -968,7 +1192,6 @@ ARDOUR_UI::open_session ()
 {
 	if (!check_audioengine()) {
 		return;
-		
 	}
 
 	/* popup selector window */
@@ -980,6 +1203,7 @@ ARDOUR_UI::open_session ()
 		open_session_selector = new Gtk::FileChooserDialog (_("open session"), FILE_CHOOSER_ACTION_OPEN);
 		open_session_selector->add_button (Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
 		open_session_selector->add_button (Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
+		open_session_selector->set_default_response(Gtk::RESPONSE_ACCEPT);
 
 		FileFilter session_filter;
 		session_filter.add_pattern ("*.ardour");
@@ -1070,7 +1294,6 @@ ARDOUR_UI::session_add_audio_route (bool track, int32_t input_channels, int32_t
 	}
 
 	catch (...) {
-		cerr << "About to complain about JACK\n";
 		MessageDialog msg (*editor, 
 				   _("There are insufficient JACK ports available\n\
 to create a new track or bus.\n\
@@ -1186,7 +1409,7 @@ ARDOUR_UI::remove_last_capture()
 }
 
 void
-ARDOUR_UI::transport_record ()
+ARDOUR_UI::transport_record (bool roll)
 {
 	if (session) {
 		switch (session->record_status()) {
@@ -1197,10 +1420,20 @@ ARDOUR_UI::transport_record ()
 				return;
 			}
 			session->maybe_enable_record ();
+			if (roll) {
+				transport_roll ();
+			}
 			break;
 		case Session::Recording:
+			if (roll) {
+				session->request_stop();
+			} else {
+				session->disable_record (false, true);
+			}
+			break;
+
 		case Session::Enabled:
-			session->disable_record (true);
+			session->disable_record (false, true);
 		}
 	}
 }
@@ -1463,26 +1696,17 @@ ARDOUR_UI::do_engine_start ()
 	return 0;
 }
 
-gint
-ARDOUR_UI::start_engine ()
+void
+ARDOUR_UI::setup_theme ()
 {
-	if (do_engine_start () == 0) {
-		if (session && _session_is_new) {
-			/* we need to retain initial visual 
-			   settings for a new session 
-			*/
-			session->save_state ("");
-		}
-	}
-
-	return FALSE;
+	theme_manager->setup_theme();
 }
 
 void
 ARDOUR_UI::update_clocks ()
 {
 	if (!editor || !editor->dragging_playhead()) {
-		Clock (session->audible_frame()); /* EMIT_SIGNAL */
+		Clock (session->audible_frame(), false, editor->get_preferred_edit_position()); /* EMIT_SIGNAL */
 	}
 }
 
@@ -1667,6 +1891,14 @@ ARDOUR_UI::primary_clock_value_changed ()
 	}
 }
 
+void
+ARDOUR_UI::big_clock_value_changed ()
+{
+	if (session) {
+		session->request_locate (big_clock.current_time ());
+	}
+}
+
 void
 ARDOUR_UI::secondary_clock_value_changed ()
 {
@@ -1747,6 +1979,10 @@ ARDOUR_UI::save_template ()
 	ArdourPrompter prompter (true);
 	string name;
 
+	if (!check_audioengine()) {
+		return;
+	}
+
 	prompter.set_name (X_("Prompter"));
 	prompter.set_prompt (_("Name for mix template:"));
 	prompter.set_initial_text(session->name() + _("-template"));
@@ -1766,99 +2002,254 @@ ARDOUR_UI::save_template ()
 	}
 }
 
-bool
-ARDOUR_UI::new_session (std::string predetermined_path)
+void
+ARDOUR_UI::fontconfig_dialog ()
 {
-	string session_name;
-	string session_path;
-
-	if (!check_audioengine()) {
-		return false;
+#ifdef GTKOSX
+	/* X11 users will always have fontconfig info around, but new GTK-OSX users 
+	   may not and it can take a while to build it. Warn them.
+	*/
+	
+	Glib::ustring fontconfig = Glib::build_filename (Glib::get_home_dir(), ".fontconfig");
+	
+	if (!Glib::file_test (fontconfig, Glib::FILE_TEST_EXISTS|Glib::FILE_TEST_IS_DIR)) {
+		MessageDialog msg (*new_session_dialog,
+				   _("Welcome to Ardour.\n\n"
+				     "The program will take a bit longer to start up\n"
+				     "while the system fonts are checked.\n\n"
+				     "This will only be done once, and you will\n"
+				     "not see this message again\n"),
+				   true,
+				   Gtk::MESSAGE_INFO,
+				   Gtk::BUTTONS_OK);
+		msg.show_all ();
+		msg.present ();
+		msg.run ();
 	}
+#endif
+}
+
+bool
+ARDOUR_UI::get_session_parameters (Glib::ustring predetermined_path, bool have_engine, bool should_be_new)
+{
+	bool existing_session = false;
+	Glib::ustring session_name;
+	Glib::ustring session_path;
+	Glib::ustring template_name;
+
+	if (!loading_dialog) {
+		loading_dialog = new MessageDialog (*new_session_dialog, 
+						    "",
+						    false,
+						    Gtk::MESSAGE_INFO,
+						    Gtk::BUTTONS_NONE);
+	}
+
 
 	int response = Gtk::RESPONSE_NONE;
 
-	new_session_dialog->set_modal(true);
-	new_session_dialog->set_name (predetermined_path);
-	new_session_dialog->reset_recent();
-	new_session_dialog->show();
+	if (predetermined_path.length()) {
+		
+		/* before we start, lets see if the given path looks like
+		   an existing ardour session. if it does, skip the
+		   tabs that we don't need
+		*/
+
+		if (Glib::file_test (predetermined_path, Glib::FILE_TEST_IS_DIR)) {
+			session_path = predetermined_path;
+			existing_session = true;
+		} else if (Glib::file_test (predetermined_path, Glib::FILE_TEST_IS_REGULAR)) {
+			session_path = Glib::path_get_dirname (string (predetermined_path));
+			existing_session = true;
+		} else {
+			/* it doesn't exist, assume the best */
+			session_path = Glib::path_get_dirname (string (predetermined_path));
+		}
+		       
+		session_name = basename_nosuffix (string (predetermined_path));
+
+		new_session_dialog->set_session_name (session_name);
+		new_session_dialog->set_session_folder (session_path);
+		new_session_dialog->set_modal (true);
+
+		if (existing_session) {
+
+			if (session_name.length() == 0 || session_path.length() == 0) {
+				error << string_compose (_("Ardour cannot understand \"%1\" as a session name"), predetermined_path) << endmsg;
+				return false;
+			}
+			
+			if (Glib::file_test (predetermined_path, Glib::FILE_TEST_IS_DIR)) {
+				Glib::ustring predicted_session_file;
+				
+				predicted_session_file = predetermined_path;
+				predicted_session_file += '/';
+				predicted_session_file += session_name;
+				predicted_session_file += Session::statefile_suffix();
+				
+				if (Glib::file_test (predicted_session_file, Glib::FILE_TEST_EXISTS)) {
+					existing_session = true;
+				}
+				
+			} else if (Glib::file_test (predetermined_path, Glib::FILE_TEST_EXISTS)) {
+				
+				if (predetermined_path.find (Session::statefile_suffix()) == predetermined_path.length() - 7) {
+					/* existing .ardour file */
+					existing_session = true;
+				}
+			} else {
+				existing_session = false;
+			}
+			
+			if (existing_session && have_engine) {
+				/* lets just try to load it */
+				
+				loading_dialog->set_message (_("Starting audio engine"));
+				loading_dialog->show_all ();
+				flush_pending ();
+				
+				if (create_engine ()) {
+					backend_audio_error (!have_engine, new_session_dialog);
+					loading_dialog->hide ();
+					return false;
+				}
+				
+				if (load_session (session_path, session_name) == 0) {
+					goto done;
+				}
+			}
+		}
+	}
+
+	/* loading failed, or we need the NSD for something */
+
+	new_session_dialog->set_position (WIN_POS_CENTER);
 	new_session_dialog->set_current_page (0);
+	new_session_dialog->set_existing_session (existing_session);
+	new_session_dialog->reset_recent();
 
 	do {
-	        response = new_session_dialog->run ();
-
-		if (!check_audioengine()) {
-			new_session_dialog->hide ();
-			return false;
-		}
+		new_session_dialog->set_have_engine (have_engine);
+		new_session_dialog->present ();
+		response = new_session_dialog->run ();
+		loading_dialog->hide ();
 		
 		_session_is_new = false;
+		
+		/* handle possible negative responses */
 
 		if (response == Gtk::RESPONSE_CANCEL || response == Gtk::RESPONSE_DELETE_EVENT) {
-
+			
 			if (!session) {
 				quit();
 			}
 			new_session_dialog->hide ();
 			return false;
-
+			
 		} else if (response == Gtk::RESPONSE_NONE) {
+			/* "Clear" was pressed */
+			
+			goto try_again;
+		}
 
-		        /* Clear was pressed */
-		        new_session_dialog->reset();
+		fontconfig_dialog();
 
-		} else if (response == Gtk::RESPONSE_YES) {
+		/* if we're here to help set up audio parameters this is where want to do that.
+		*/
 
-		        /* YES  == OPEN, but there's no enum for that */
+		if (!have_engine) {
+			if (new_session_dialog->engine_control.setup_engine ()) {
+				new_session_dialog->hide ();
+				return false;
+			} 
+
+			loading_dialog->set_message (_("Starting audio engine"));
+			loading_dialog->show_all ();
+			flush_pending ();
+		}
+
+		if (create_engine ()) {
+			backend_audio_error (!have_engine, new_session_dialog);
+			loading_dialog->hide ();
+			flush_pending ();
+			/* audio setup page */
+			new_session_dialog->set_existing_session (false);
+			new_session_dialog->set_current_page (2);
+			response = Gtk::RESPONSE_NONE;
+			goto try_again;
+		}
+
+		loading_dialog->hide ();
+		have_engine = true;		
+			
+		/* now handle possible affirmative responses */
+
+		if (response == Gtk::RESPONSE_YES) {
+
+		        /* YES == OPEN from the session selector */
 
 		        session_name = new_session_dialog->session_name();
 			
 			if (session_name.empty()) {
 				response = Gtk::RESPONSE_NONE;
-				continue;
+				goto try_again;
 			} 
 
 			if (session_name[0] == '/' || 
 			    (session_name.length() > 2 && session_name[0] == '.' && session_name[1] == '/') ||
 			    (session_name.length() > 3 && session_name[0] == '.' && session_name[1] == '.' && session_name[2] == '/')) {
-				load_session (Glib::path_get_dirname (session_name), session_name);
+				if (load_session (Glib::path_get_dirname (session_name), session_name)) {
+					response = Gtk::RESPONSE_NONE;
+					goto try_again;
+				}
 			} else {
 				session_path = new_session_dialog->session_folder();
-				load_session (session_path, session_name);
+				if (load_session (session_path, session_name)) {
+					response = Gtk::RESPONSE_NONE;
+					goto try_again;
+				}
 			}
 			
 		} else if (response == Gtk::RESPONSE_OK) {
 
+			/* OK == OPEN button */
+
 			session_name = new_session_dialog->session_name();
-			
-		        if (new_session_dialog->get_current_page() == 1) {
-		  
-			        /* XXX this is a bit of a hack.. 
-				   i really want the new sesion dialog to return RESPONSE_YES
-				   if we're on page 1 (the load page)
-				   Unfortunately i can't see how atm.. 
-				*/
-				
-				if (session_name.empty()) {
-					response = Gtk::RESPONSE_NONE;
-					continue;
-				} 
-				
+		
+			if (session_name.empty()) {
+				response = Gtk::RESPONSE_NONE;
+				goto try_again;
+			} 
+
+			switch (new_session_dialog->which_page()) {
+			case NewSessionDialog::OpenPage: 
+			case NewSessionDialog::EnginePage:
+
 				if (session_name[0] == '/' || 
 				    (session_name.length() > 2 && session_name[0] == '.' && session_name[1] == '/') ||
 				    (session_name.length() > 3 && session_name[0] == '.' && session_name[1] == '.' && session_name[2] == '/')) {
-					load_session (Glib::path_get_dirname (session_name), session_name);
+
+					if (load_session (Glib::path_get_dirname (session_name), session_name)) {
+						response = Gtk::RESPONSE_NONE;
+						goto try_again;
+					}
+
 				} else {
 					session_path = new_session_dialog->session_folder();
-					load_session (session_path, session_name);
+					if (load_session (session_path, session_name)) {
+						response = Gtk::RESPONSE_NONE;
+						goto try_again;
+					}
 				}
-			
-			} else {
+				break;
 
-				if (session_name.empty()) {
-					response = Gtk::RESPONSE_NONE;
-					continue;
-				} 
+			case NewSessionDialog::NewPage: /* nominally the "new" session creator, but could be in use for an old session */
+
+				if (new_session_dialog->get_current_page() == 0 && ARDOUR_COMMAND_LINE::session_name.empty()) {
+					should_be_new = true;
+				}
+
+				/* handle what appear to be paths rather than just a name */
 
 				if (session_name[0] == '/' || 
 				    (session_name.length() > 2 && session_name[0] == '.' && session_name[1] == '/') ||
@@ -1872,13 +2263,22 @@ ARDOUR_UI::new_session (std::string predetermined_path)
 					session_path = new_session_dialog->session_folder();
 
 				}
-				
+
 				//XXX This is needed because session constructor wants a 
 				//non-existant path. hopefully this will be fixed at some point.
 				
 				session_path = Glib::build_filename (session_path, session_name);
-						
-				if (g_file_test (session_path.c_str(), GFileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
+
+				if (!should_be_new) {
+
+					if (load_session (session_path, session_name)) {
+						response = Gtk::RESPONSE_NONE;
+						goto try_again;
+					}
+
+					continue; /* leaves while() loop because response != NONE */
+
+				} else if (Glib::file_test (session_path, Glib::FileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
 
 					Glib::ustring str = string_compose (_("This session\n%1\nalready exists. Do you want to open it?"), session_path);
 
@@ -1895,62 +2295,94 @@ ARDOUR_UI::new_session (std::string predetermined_path)
 					
 					switch (msg.run()) {
 					case RESPONSE_YES:
-						load_session (session_path, session_name);
+						new_session_dialog->hide ();
+						goto_editor_window ();
+						flush_pending ();
+						if (load_session (session_path, session_name)) {
+							response = Gtk::RESPONSE_NONE;
+							goto try_again;
+						}
 						goto done;
 						break;
 					default:
 						response = RESPONSE_NONE;
 						new_session_dialog->reset ();
+						new_session_dialog->set_existing_session (false);
+						loading_dialog->hide ();
 						continue;
 					}
-				}
+				} 
 
 			        _session_is_new = true;
-
-				std::string template_name = new_session_dialog->session_template_name();
 						
 				if (new_session_dialog->use_session_template()) {
-							
-					load_session (session_path, session_name, &template_name);
+
+					template_name = new_session_dialog->session_template_name();
+
+					new_session_dialog->hide ();
+					goto_editor_window ();
+					flush_pending ();
+
+					if (load_session (session_path, session_name, template_name)) {
+						response = Gtk::RESPONSE_NONE;
+						goto try_again;
+					}
+
 			  
 				} else {
-							
+
 					uint32_t cchns;
 					uint32_t mchns;
 					AutoConnectOption iconnect;
 					AutoConnectOption oconnect;
-							
-					if (new_session_dialog->create_control_bus()) {
-						cchns = (uint32_t) new_session_dialog->control_channel_count();
-					} else {
+					uint32_t nphysin;
+					uint32_t nphysout;
+					
+					if (Profile->get_sae()) {
+
 						cchns = 0;
-					}
-							
-					if (new_session_dialog->create_master_bus()) {
-						mchns = (uint32_t) new_session_dialog->master_channel_count();
-					} else {
-						mchns = 0;
-					}
-							
-					if (new_session_dialog->connect_inputs()) {
+						mchns = 2;
 						iconnect = AutoConnectPhysical;
-					} else {
-						iconnect = AutoConnectOption (0);
-					}
-							
-					/// @todo some minor tweaks.
-							
-					if (new_session_dialog->connect_outs_to_master()) {
 						oconnect = AutoConnectMaster;
-					} else if (new_session_dialog->connect_outs_to_physical()) {
-						oconnect = AutoConnectPhysical;
+						nphysin = 0; // use all available
+						nphysout = 0; // use all available
+
 					} else {
-						oconnect = AutoConnectOption (0);
-					} 
-							
-					uint32_t nphysin = (uint32_t) new_session_dialog->input_limit_count();
-					uint32_t nphysout = (uint32_t) new_session_dialog->output_limit_count();
-							
+
+						/* get settings from advanced section of NSD */
+						
+						if (new_session_dialog->create_control_bus()) {
+							cchns = (uint32_t) new_session_dialog->control_channel_count();
+						} else {
+							cchns = 0;
+						}
+						
+						if (new_session_dialog->create_master_bus()) {
+							mchns = (uint32_t) new_session_dialog->master_channel_count();
+						} else {
+							mchns = 0;
+						}
+						
+						if (new_session_dialog->connect_inputs()) {
+							iconnect = AutoConnectPhysical;
+						} else {
+							iconnect = AutoConnectOption (0);
+						}
+						
+						/// @todo some minor tweaks.
+						
+						if (new_session_dialog->connect_outs_to_master()) {
+							oconnect = AutoConnectMaster;
+						} else if (new_session_dialog->connect_outs_to_physical()) {
+							oconnect = AutoConnectPhysical;
+						} else {
+							oconnect = AutoConnectOption (0);
+						} 
+						
+						nphysin = (uint32_t) new_session_dialog->input_limit_count();
+						nphysout = (uint32_t) new_session_dialog->output_limit_count();
+					}
+
 					if (build_session (session_path,
 							   session_name,
 							   cchns,
@@ -1960,71 +2392,142 @@ ARDOUR_UI::new_session (std::string predetermined_path)
 							   nphysin,
 							   nphysout, 
 							   engine->frame_rate() * 60 * 5)) {
-
+						
 						response = Gtk::RESPONSE_NONE;
-						new_session_dialog->reset ();
-						continue;
+						goto try_again;
 					}
+
+					new_session_dialog->hide ();
+					goto_editor_window ();
+					flush_pending ();
 				}
+				break;
+
+			default:
+				break;
 			}
 		}
+
+	  try_again:
+		if (response == Gtk::RESPONSE_NONE) {
+			loading_dialog->hide ();
+			new_session_dialog->set_existing_session (false);
+			new_session_dialog->reset ();
+		}
 		
 	} while (response == Gtk::RESPONSE_NONE);
 
   done:
 	show();
-	new_session_dialog->get_window()->set_cursor();
+	loading_dialog->hide ();
 	new_session_dialog->hide();
 	return true;
 }
 
 void
-ARDOUR_UI::close_session()
+ARDOUR_UI::close_session ()
 {
 	if (!check_audioengine()) {
 		return;
 	}
 
-	unload_session();
-	new_session ();
+	unload_session (true);
+	
+	get_session_parameters ("", true, false);
 }
 
 int
-ARDOUR_UI::load_session (const string & path, const string & snap_name, string* mix_template)
+ARDOUR_UI::load_session (const Glib::ustring& path, const Glib::ustring& snap_name, Glib::ustring mix_template)
 {
 	Session *new_session;
-	int x;
+	int unload_status;
+	int retval = -1;
+
 	session_loaded = false;
-	
+
 	if (!check_audioengine()) {
 		return -1;
 	}
 
-	x = unload_session ();
+	unload_status = unload_session ();
 
-	if (x < 0) {
-		return -1;
-	} else if (x > 0) {
-		return 0;
+	if (unload_status < 0) {
+		goto out;
+	} else if (unload_status > 0) {
+		retval = 0;
+		goto out;
 	}
 
 	/* if it already exists, we must have write access */
 
-	if (::access (path.c_str(), F_OK) == 0 && ::access (path.c_str(), W_OK)) {
+	if (Glib::file_test (path.c_str(), Glib::FILE_TEST_EXISTS) && ::access (path.c_str(), W_OK)) {
 		MessageDialog msg (*editor, _("You do not have write access to this session.\n"
 					      "This prevents the session from being loaded."));
 		msg.run ();
-		return -1;
+		goto out;
 	}
 
+	if (loading_dialog) {
+		loading_dialog->set_markup (_("Please wait while Ardour loads your session"));
+		flush_pending ();
+	}
+
+	disable_screen_updates ();
+
 	try {
 		new_session = new Session (*engine, path, snap_name, mix_template);
 	}
 
+	/* this one is special */
+
+	catch (AudioEngine::PortRegistrationFailure& err) {
+
+		MessageDialog msg (err.what(),
+				   true,
+				   Gtk::MESSAGE_INFO,
+				   Gtk::BUTTONS_OK_CANCEL);
+		
+		msg.set_title (_("Loading Error"));
+		msg.set_secondary_text (_("Click the OK button to try again."));
+		msg.set_position (Gtk::WIN_POS_CENTER);
+		msg.present ();
+
+		int response = msg.run ();
+
+		msg.hide ();
+
+		switch (response) {
+		case RESPONSE_CANCEL:
+			exit (1);
+		default:
+			break;
+		}
+		goto out;
+	}
+
 	catch (...) {
 
-		error << string_compose(_("Session \"%1 (snapshot %2)\" did not load successfully"), path, snap_name) << endmsg;
-		return -1;
+		MessageDialog msg (string_compose(_("Session \"%1 (snapshot %2)\" did not load successfully"), path, snap_name),
+				   true,
+				   Gtk::MESSAGE_INFO,
+				   Gtk::BUTTONS_OK_CANCEL);
+		
+		msg.set_title (_("Loading Error"));
+		msg.set_secondary_text (_("Click the OK button to try again."));
+		msg.set_position (Gtk::WIN_POS_CENTER);
+		msg.present ();
+
+		int response = msg.run ();
+
+		msg.hide ();
+
+		switch (response) {
+		case RESPONSE_CANCEL:
+			exit (1);
+		default:
+			break;
+		}
+		goto out;
 	}
 
 	connect_to_session (new_session);
@@ -2039,12 +2542,16 @@ ARDOUR_UI::load_session (const string & path, const string & snap_name, string*
 		session->set_clean ();
 	}
 
-	editor->edit_cursor_position (true);
-	return 0;
+	enable_screen_updates ();
+	flush_pending ();
+	retval = 0;
+
+  out:
+	return retval;
 }
 
 int
-ARDOUR_UI::build_session (const string & path, const string & snap_name, 
+ARDOUR_UI::build_session (const Glib::ustring& path, const Glib::ustring& snap_name, 
 			  uint32_t control_channels,
 			  uint32_t master_channels, 
 			  AutoConnectOption input_connect,
@@ -2477,7 +2984,9 @@ ARDOUR_UI::disk_speed_dialog_gone (int ignored_response, MessageDialog* msg)
 int
 ARDOUR_UI::pending_state_dialog ()
 {
-	ArdourDialog dialog ("pending state dialog");
+ 	HBox* hbox = new HBox();
+	Image* image = new Image (Stock::DIALOG_QUESTION, ICON_SIZE_DIALOG);
+	ArdourDialog dialog (_("Crash Recovery"), true);
 	Label  message (_("\
 This session appears to have been in\n\
 middle of recording when ardour or\n\
@@ -2486,14 +2995,18 @@ the computer was shutdown.\n\
 Ardour can recover any captured audio for\n\
 you, or it can ignore it. Please decide\n\
 what you would like to do.\n"));
-
-	dialog.get_vbox()->pack_start (message);
-	dialog.add_button (_("Recover from crash"), RESPONSE_ACCEPT);
+	image->set_alignment(ALIGN_CENTER, ALIGN_TOP);
+	hbox->pack_start (*image, PACK_EXPAND_WIDGET, 12);
+	hbox->pack_end (message, PACK_EXPAND_PADDING, 12);
+	dialog.get_vbox()->pack_start(*hbox, PACK_EXPAND_PADDING, 6);
 	dialog.add_button (_("Ignore crash data"), RESPONSE_REJECT);
-
+	dialog.add_button (_("Recover from crash"), RESPONSE_ACCEPT);
+	dialog.set_default_response (RESPONSE_ACCEPT);
 	dialog.set_position (WIN_POS_CENTER);
-	dialog.show_all ();
-	
+	message.show();
+	image->show();
+	hbox->show();
+
 	switch (dialog.run ()) {
 	case RESPONSE_ACCEPT:
 		return 1;
@@ -2542,7 +3055,7 @@ ARDOUR_UI::cmdline_new_session (string path)
 		path = str;
 	}
 
-	new_session (path);
+	get_session_parameters (path, false, true);
 
 	_will_create_new_session_automatically = false; /* done it */
 	return FALSE; /* don't call it again */
@@ -2560,6 +3073,9 @@ ARDOUR_UI::use_config ()
 	case FormatInt24:
 		act = ActionManager::get_action (X_("options"), X_("FileDataFormat24bit"));
 		break;
+	case FormatInt16:
+		act = ActionManager::get_action (X_("options"), X_("FileDataFormat16bit"));
+		break;
 	}
 
 	if (act) {
@@ -2605,8 +3121,17 @@ ARDOUR_UI::use_config ()
 void
 ARDOUR_UI::update_transport_clocks (nframes_t pos)
 {
-	primary_clock.set (pos);
-	secondary_clock.set (pos);
+	if (Config->get_primary_clock_delta_edit_cursor()) {
+		primary_clock.set (pos, false, editor->get_preferred_edit_position(), 1);
+	} else {
+		primary_clock.set (pos, 0, true);
+	}
+
+	if (Config->get_secondary_clock_delta_edit_cursor()) {
+		secondary_clock.set (pos, false, editor->get_preferred_edit_position(), 2);
+	} else {
+		secondary_clock.set (pos);
+	}
 
 	if (big_clock_window) {
 		big_clock.set (pos);
@@ -2643,7 +3168,7 @@ void
 ARDOUR_UI::save_keybindings ()
 {
 	if (can_save_keybindings) {
-		AccelMap::save (keybindings_path);
+		AccelMap::save (user_keybindings_path);
 	} 
 }
 
@@ -2783,4 +3308,10 @@ ARDOUR_UI::setup_profile ()
 	if (gdk_screen_width() < 1200) {
 		Profile->set_small_screen ();
 	}
+
+	if (getenv ("ARDOUR_SAE")) {
+		Profile->set_sae ();
+		Profile->set_single_package ();
+	}
 }
+
