@@ -68,6 +68,7 @@ class AudioClock;
 class PublicEditor;
 class Keyboard;
 class OptionEditor;
+class KeyEditor;
 class Mixer_UI;
 class ConnectionEditor;
 class RouteParams_UI;
@@ -109,9 +110,9 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void show_splash ();
 	void hide_splash ();
 	
-	int load_session (const string & path, const string & snapshot, string* mix_template = 0);
+	int load_session (const Glib::ustring & path, const Glib::ustring& snapshot, Glib::ustring mix_template = Glib::ustring());
 	bool session_loaded;
-	int build_session (const string & path, const string & snapshot, 
+	int build_session (const Glib::ustring& path, const Glib::ustring& snapshot, 
 			   uint32_t ctl_chns, 
 			   uint32_t master_chns,
 			   ARDOUR::AutoConnectOption input_connect,
@@ -148,7 +149,10 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	PublicEditor&	  the_editor(){return *editor;}
 	Mixer_UI* the_mixer() { return mixer; }
-	
+
+	ARDOUR::AudioEngine& the_engine() const { return *engine; }
+
+	void toggle_key_editor ();
 	void toggle_location_window ();
 	void toggle_theme_manager ();
 	void toggle_big_clock_window ();
@@ -203,9 +207,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	void session_add_midi_track ();
 
-	void create_engine ();
+	int  create_engine ();
 	void post_engine ();
-	gint start_backend_audio ();
 
 	gint exit_on_main_window_close (GdkEventAny *);
 
@@ -266,8 +269,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	ARDOUR::AudioEngine                 *engine;
 	ARDOUR::Session                     *session;
 
-	bool check_audioengine();
-
 	Gtk::Tooltips          _tooltips;
 
 	void                     goto_editor_window ();
@@ -295,7 +296,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	static ARDOUR_UI *theArdourUI;
 
-	void backend_audio_error ();
+	void backend_audio_error (bool we_set_params, Gtk::Window* toplevel = 0);
 	void startup ();
 	void shutdown ();
 
@@ -469,6 +470,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void solo_alert_toggle ();
 	void audition_alert_toggle ();
 
+	void big_clock_value_changed ();
 	void primary_clock_value_changed ();
 	void secondary_clock_value_changed ();
 
@@ -565,7 +567,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void transport_goto_end ();
 	void transport_stop ();
 	void transport_stop_and_forget_capture ();
-	void transport_record ();
+	void transport_record (bool roll);
 	void transport_roll ();
 	void transport_play_selection(); 
 	void transport_forward (int option);
@@ -611,6 +613,10 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	static UIConfiguration *ui_config;
 	ThemeManager *theme_manager;
 
+	/* Key bindings editor */
+
+	KeyEditor *key_editor;
+
 	/* Options window */
 	
 	OptionEditor *option_editor;
@@ -704,6 +710,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_PrimaryClockDeltaEditCursor ();
 	void toggle_SecondaryClockDeltaEditCursor ();
 	void toggle_only_copy_imported_files ();
+	void toggle_ShowTrackMeters ();
+	void toggle_use_narrow_ms();
 
 	void mtc_port_changed ();
 	void map_solo_model ();
@@ -732,6 +740,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void no_memory_warning ();
 	void check_memory_locking ();
 
+	bool check_audioengine();
 	void audioengine_setup ();
 
 	void display_message (const char *prefix, gint prefix_len, 
@@ -739,6 +748,10 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	Gtk::Label status_bar_label;
 	Gtk::ToggleButton error_log_button;
 	Gtk::MessageDialog* loading_dialog;
+
+	void platform_specific ();
+	void platform_setup ();
+	void fontconfig_dialog ();
 };
 
 #endif /* __ardour_gui_h__ */
