@@ -1,7 +1,7 @@
 /*
     Copyright (C) 2001-2006 Paul Davis 
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software; you can r>edistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
@@ -29,6 +29,7 @@
 #include <ardour/audioregion.h>
 #include <ardour/audiosource.h>
 #include <ardour/audio_diskstream.h>
+#include <ardour/profile.h>
 #include <pbd/memento_command.h>
 #include <pbd/stacktrace.h>
 
@@ -46,9 +47,12 @@
 #include "utils.h"
 #include "rgb_macros.h"
 #include "gui_thread.h"
+#include "ardour_ui.h"
 
 #include "i18n.h"
 
+#define MUTED_ALPHA 10
+
 using namespace sigc;
 using namespace ARDOUR;
 using namespace PBD;
@@ -109,7 +113,7 @@ AudioRegionView::AudioRegionView (const AudioRegionView& other)
 	UINT_TO_RGBA (other.fill_color, &r, &g, &b, &a);
 	c.set_rgb_p (r/255.0, g/255.0, b/255.0);
 	
-	init (c, false);
+	init (c, true);
 }
 
 void
@@ -118,7 +122,7 @@ AudioRegionView::init (Gdk::Color& basic_color, bool wfd)
 	// FIXME: Some redundancy here with RegionView::init.  Need to figure out
 	// where order is important and where it isn't...
 	
-	RegionView::init(basic_color, false);
+	RegionView::init (basic_color, true);
 	
 	XMLNode *node;
 
@@ -176,16 +180,18 @@ AudioRegionView::init (Gdk::Color& basic_color, bool wfd)
 	foo += ':';
 	foo += "gain";
 
-	gain_line = new AudioRegionGainLine (foo, trackview.session(), *this, *group, audio_region()->envelope());
+	if (!Profile->get_sae()) {
+		gain_line = new AudioRegionGainLine (foo, trackview.session(), *this, *group, audio_region()->envelope());
 
-	if (!(_flags & EnvelopeVisible)) {
-		gain_line->hide ();
-	} else {
-		gain_line->show ();
+		if (!(_flags & EnvelopeVisible)) {
+			gain_line->hide ();
+		} else {
+			gain_line->show ();
+		}
+
+		gain_line->reset ();
 	}
 
-	gain_line->reset ();
-
 	set_height (trackview.height);
 
 	region_muted ();
@@ -378,9 +384,9 @@ AudioRegionView::region_muted ()
 
 	for (uint32_t n=0; n < waves.size(); ++n) {
 		if (_region->muted()) {
-			waves[n]->property_wave_color() = color_map[cMutedWaveForm];
+			waves[n]->property_wave_color() = UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA);
 		} else {
-			waves[n]->property_wave_color() = color_map[cWaveForm];
+			waves[n]->property_wave_color() = ARDOUR_UI::config()->canvasvar_WaveForm.get();
 		}
 	}
 }
@@ -389,6 +395,8 @@ AudioRegionView::region_muted ()
 void
 AudioRegionView::set_height (gdouble height)
 {
+	RegionView::set_height (height);
+
 	uint32_t wcnt = waves.size();
 
 	// FIXME: ick
@@ -490,7 +498,7 @@ AudioRegionView::reset_fade_in_shape_width (nframes_t width)
 	} else {
 		handle_center = 3.0;
 	}
-	
+
 	fade_in_handle->property_x1() =  handle_center - 3.0;
 	fade_in_handle->property_x2() =  handle_center + 3.0;
 	
@@ -669,18 +677,18 @@ AudioRegionView::set_colors ()
 	RegionView::set_colors();
 	
 	if (gain_line) {
-		gain_line->set_line_color (audio_region()->envelope_active() ? color_map[cGainLine] : color_map[cGainLineInactive]);
+		gain_line->set_line_color (audio_region()->envelope_active() ? ARDOUR_UI::config()->canvasvar_GainLine.get() : ARDOUR_UI::config()->canvasvar_GainLineInactive.get());
 	}
 
 	for (uint32_t n=0; n < waves.size(); ++n) {
 		if (_region->muted()) {
-			waves[n]->property_wave_color() = color_map[cMutedWaveForm];
+			waves[n]->property_wave_color() = UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA);
 		} else {
-			waves[n]->property_wave_color() = color_map[cWaveForm];
+			waves[n]->property_wave_color() = ARDOUR_UI::config()->canvasvar_WaveForm.get();
 		}
 
-		waves[n]->property_clip_color() = color_map[cWaveFormClip];
-		waves[n]->property_zero_color() = color_map[cZeroLine];
+		waves[n]->property_clip_color() = ARDOUR_UI::config()->canvasvar_WaveFormClip.get();
+		waves[n]->property_zero_color() = ARDOUR_UI::config()->canvasvar_ZeroLine.get();
 	}
 }
 
@@ -777,10 +785,14 @@ AudioRegionView::create_waves ()
 		wave_caches.push_back (WaveView::create_cache ());
 
 		if (wait_for_data) {
+
 			if (audio_region()->source(n)->peaks_ready (bind (mem_fun(*this, &AudioRegionView::peaks_ready_handler), n), data_ready_connection)) {
 				create_one_wave (n, true);
 			} else {
+				// we'll get a PeaksReady signal from the source in the future
+				// and will call create_one_wave(n) then.
 			}
+
 		} else {
 			create_one_wave (n, true);
 		}
@@ -818,9 +830,9 @@ AudioRegionView::create_one_wave (uint32_t which, bool direct)
 	wave->property_height() =  (double) ht;
 	wave->property_samples_per_unit() =  samples_per_unit;
 	wave->property_amplitude_above_axis() =  _amplitude_above_axis;
-	wave->property_wave_color() = _region->muted() ? color_map[cMutedWaveForm] : color_map[cWaveForm];
-	wave->property_clip_color() = color_map[cWaveFormClip];
-	wave->property_zero_color() = color_map[cZeroLine];
+	wave->property_wave_color() = _region->muted() ? UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA) : ARDOUR_UI::config()->canvasvar_WaveForm.get();
+	wave->property_clip_color() = ARDOUR_UI::config()->canvasvar_WaveFormClip.get();
+	wave->property_zero_color() = ARDOUR_UI::config()->canvasvar_ZeroLine.get();
 	wave->property_region_start() = _region->start();
 	wave->property_rectified() = (bool) (_flags & WaveformRectified);
 	wave->property_logscaled() = (bool) (_flags & WaveformLogScaled);
@@ -859,12 +871,11 @@ AudioRegionView::create_one_wave (uint32_t which, bool direct)
 		/* all waves created, don't hook into peaks ready anymore */
 		data_ready_connection.disconnect ();		
 
-		if(0)
 		if (!zero_line) {
 			zero_line = new ArdourCanvas::SimpleLine (*group);
 			zero_line->property_x1() = (gdouble) 1.0;
 			zero_line->property_x2() = (gdouble) (_region->length() / samples_per_unit) - 1.0;
-			zero_line->property_color_rgba() = (guint) color_map[cZeroLine];
+			zero_line->property_color_rgba() = (guint) ARDOUR_UI::config()->canvasvar_ZeroLine.get();
 			manage_zero_line ();
 		}
 	}
@@ -1066,9 +1077,9 @@ AudioRegionView::add_ghost (AutomationTimeAxisView& atv)
 		wave->property_x() =  0.0;
 		wave->property_samples_per_unit() =  samples_per_unit;
 		wave->property_amplitude_above_axis() =  _amplitude_above_axis;
-		wave->property_wave_color() = color_map[cGhostTrackWave];
-		wave->property_clip_color() = color_map[cGhostTrackWaveClip];
-		wave->property_zero_color() = color_map[cGhostTrackZeroLine];
+		wave->property_wave_color() = ARDOUR_UI::config()->canvasvar_GhostTrackWave.get();
+		wave->property_clip_color() = ARDOUR_UI::config()->canvasvar_GhostTrackWaveClip.get();
+		wave->property_zero_color() = ARDOUR_UI::config()->canvasvar_GhostTrackZeroLine.get();
 		wave->property_region_start() = _region->start();
 
 		ghost->waves.push_back(wave);
@@ -1121,7 +1132,7 @@ void
 AudioRegionView::envelope_active_changed ()
 {
 	if (gain_line) {
-		gain_line->set_line_color (audio_region()->envelope_active() ? color_map[cGainLine] : color_map[cGainLineInactive]);
+		gain_line->set_line_color (audio_region()->envelope_active() ? ARDOUR_UI::config()->canvasvar_GainLine.get() : ARDOUR_UI::config()->canvasvar_GainLineInactive.get());
 	}
 }
 
@@ -1148,22 +1159,48 @@ AudioRegionView::set_waveview_data_src()
 }
 
 void
-AudioRegionView::color_handler (ColorID id, uint32_t val)
+AudioRegionView::color_handler ()
 {
-	switch (id) {
-	case cMutedWaveForm:
-	case cWaveForm:
-	case cWaveFormClip:
-	case cZeroLine:
-		set_colors ();
-		break;
+	set_colors ();
+	envelope_active_changed();
+}
 
-	case cGainLineInactive:
-	case cGainLine:
-		envelope_active_changed();
-		break;
+void
+AudioRegionView::set_frame_color ()
+{
+	if (!frame) {
+		return;
+	}
 
-	default:
-		break;
+	if (_region->opaque()) {
+		fill_opacity = 130;
+	} else {
+		fill_opacity = 0;
+	}
+
+	uint32_t r,g,b,a;
+	
+	if (_selected && should_show_selection) {
+		frame->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_WaveForm.get();
+
+		UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_FrameBase.get(), &r, &g, &b, &a);
+		for (vector<ArdourCanvas::WaveView*>::iterator w = waves.begin(); w != waves.end(); ++w) {
+			if (_region->muted()) {
+				(*w)->property_wave_color() = RGBA_TO_UINT(r, g, b, MUTED_ALPHA);
+			} else {
+				(*w)->property_wave_color() = RGBA_TO_UINT(r, g, b, fill_opacity ? fill_opacity : a);// Lets still use the theme's opacity value if Opaque is not set
+			}
+		}
+	} else {
+		UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_FrameBase.get(), &r, &g, &b, &a);
+		frame->property_fill_color_rgba() = RGBA_TO_UINT(r, g, b, fill_opacity ? fill_opacity : a);
+
+		for (vector<ArdourCanvas::WaveView*>::iterator w = waves.begin(); w != waves.end(); ++w) {
+			if (_region->muted()) {
+				(*w)->property_wave_color() = UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA);
+			} else {
+				(*w)->property_wave_color() = ARDOUR_UI::config()->canvasvar_WaveForm.get();
+			}
+		}
 	}
 }
