@@ -18,10 +18,12 @@
 */
 
 #include <libgnomecanvasmm/init.h>
+#include <libgnomecanvasmm/pixbuf.h>
 #include <jack/types.h>
 #include <gtkmm2ext/utils.h>
 
 #include <ardour/audioregion.h>
+#include <ardour/profile.h>
 
 #include "ardour_ui.h"
 #include "editor.h"
@@ -100,6 +102,9 @@ Editor::initialize_canvas ()
 	track_canvas.signal_button_press_event().connect (mem_fun (*this, &Editor::track_canvas_button_press_event));
 	track_canvas.signal_button_release_event().connect (mem_fun (*this, &Editor::track_canvas_button_release_event));
 
+	/* just scroll stuff for the timecanvas */
+	time_canvas.signal_scroll_event().connect (mem_fun (*this, &Editor::time_canvas_scroll_event));
+
 	track_canvas.set_name ("EditorMainCanvas");
 	track_canvas.add_events (Gdk::POINTER_MOTION_HINT_MASK|Gdk::SCROLL_MASK);
 	track_canvas.signal_leave_notify_event().connect (mem_fun(*this, &Editor::left_track_canvas));
@@ -121,14 +126,27 @@ Editor::initialize_canvas ()
 
 	/* stuff for the verbose canvas cursor */
 
-	Pango::FontDescription font = get_font_for_style (N_("VerboseCanvasCursor"));
+	Pango::FontDescription* font = get_font_for_style (N_("VerboseCanvasCursor"));
 
 	verbose_canvas_cursor = new ArdourCanvas::Text (*track_canvas.root());
-	verbose_canvas_cursor->property_font_desc() = font;
+	verbose_canvas_cursor->property_font_desc() = *font;
 	verbose_canvas_cursor->property_anchor() = ANCHOR_NW;
-	verbose_canvas_cursor->property_fill_color_rgba() = color_map[cVerboseCanvasCursor];
+
+	delete font;
 	
 	verbose_cursor_visible = false;
+
+	/* on the bottom, an image */
+	
+	if (Profile->get_sae()) {
+		Image img (::get_icon (X_("saelogo")));
+		logo_item = new ArdourCanvas::Pixbuf (*track_canvas.root(), 0.0, 0.0, img.get_pixbuf());
+		// logo_item->property_height_in_pixels() = true;
+		// logo_item->property_width_in_pixels() = true;
+		// logo_item->property_height_set() = true;
+		// logo_item->property_width_set() = true;
+		logo_item->show ();
+	}
 	
 	/* a group to hold time (measure) lines */
 	
@@ -143,39 +161,34 @@ Editor::initialize_canvas ()
 	
 	meter_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, 0.0);
 	tempo_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height);
-	marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 2.0);
-	range_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 3.0);
-	transport_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 4.0);
+	range_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 2.0);
+	transport_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 3.0);
+	marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 4.0);
+	cd_marker_group = new ArdourCanvas::Group (*time_canvas.root(), 0.0, timebar_height * 5.0);
 	
 	tempo_bar = new ArdourCanvas::SimpleRect (*tempo_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
-	tempo_bar->property_fill_color_rgba() = color_map[cTempoBar];
 	tempo_bar->property_outline_pixels() = 0;
 	
 	meter_bar = new ArdourCanvas::SimpleRect (*meter_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
-	meter_bar->property_fill_color_rgba() = color_map[cMeterBar];
 	meter_bar->property_outline_pixels() = 0;
 	
 	marker_bar = new ArdourCanvas::SimpleRect (*marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
-	marker_bar->property_fill_color_rgba() = color_map[cMarkerBar];
 	marker_bar->property_outline_pixels() = 0;
+
+	cd_marker_bar = new ArdourCanvas::SimpleRect (*cd_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
+	cd_marker_bar->property_outline_pixels() = 0;
 	
 	range_marker_bar = new ArdourCanvas::SimpleRect (*range_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
-	range_marker_bar->property_fill_color_rgba() = color_map[cRangeMarkerBar];
 	range_marker_bar->property_outline_pixels() = 0;
 	
 	transport_marker_bar = new ArdourCanvas::SimpleRect (*transport_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
-	transport_marker_bar->property_fill_color_rgba() = color_map[cTransportMarkerBar];
 	transport_marker_bar->property_outline_pixels() = 0;
 	
 	range_bar_drag_rect = new ArdourCanvas::SimpleRect (*range_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
-	range_bar_drag_rect->property_fill_color_rgba() = color_map[cRangeDragBarRectFill];
-	range_bar_drag_rect->property_outline_color_rgba() = color_map[cRangeDragBarRect];
 	range_bar_drag_rect->property_outline_pixels() = 0;
 	range_bar_drag_rect->hide ();
 	
 	transport_bar_drag_rect = new ArdourCanvas::SimpleRect (*transport_marker_group, 0.0, 0.0, max_canvas_coordinate, timebar_height-1.0);
-	transport_bar_drag_rect ->property_fill_color_rgba() = color_map[cTransportDragRectFill];
-	transport_bar_drag_rect->property_outline_color_rgba() = color_map[cTransportDragRect];
 	transport_bar_drag_rect->property_outline_pixels() = 0;
 	transport_bar_drag_rect->hide ();
 	
@@ -184,24 +197,17 @@ Editor::initialize_canvas ()
 
 	marker_drag_line = new ArdourCanvas::Line (*track_canvas.root());
 	marker_drag_line->property_width_pixels() = 1;
-	marker_drag_line->property_fill_color_rgba() = color_map[cMarkerDragLine];
 	marker_drag_line->property_points() = marker_drag_line_points;
 	marker_drag_line->hide();
 
 	range_marker_drag_rect = new ArdourCanvas::SimpleRect (*track_canvas.root(), 0.0, 0.0, 0.0, 0.0);
-	range_marker_drag_rect->property_fill_color_rgba() = color_map[cRangeDragRectFill];
-	range_marker_drag_rect->property_outline_color_rgba() = color_map[cRangeDragRect];
 	range_marker_drag_rect->hide ();
 	
 	transport_loop_range_rect = new ArdourCanvas::SimpleRect (*time_line_group, 0.0, 0.0, 0.0, 0.0);
-	transport_loop_range_rect->property_fill_color_rgba() = color_map[cTransportLoopRectFill];
-	transport_loop_range_rect->property_outline_color_rgba() = color_map[cTransportLoopRect];
 	transport_loop_range_rect->property_outline_pixels() = 1;
 	transport_loop_range_rect->hide();
 
 	transport_punch_range_rect = new ArdourCanvas::SimpleRect (*time_line_group, 0.0, 0.0, 0.0, 0.0);
-	transport_punch_range_rect->property_fill_color_rgba() = color_map[cTransportPunchRectFill];
-	transport_punch_range_rect->property_outline_color_rgba() = color_map[cTransportPunchRect];
 	transport_punch_range_rect->property_outline_pixels() = 0;
 	transport_punch_range_rect->hide();
 	
@@ -212,7 +218,6 @@ Editor::initialize_canvas ()
 	transport_punchin_line->property_y1() = 0.0;
 	transport_punchin_line->property_x2() = 0.0;
 	transport_punchin_line->property_y2() = 0.0;
-	transport_punchin_line->property_color_rgba() = color_map[cPunchInLine];
 	transport_punchin_line->hide ();
 	
 	transport_punchout_line  = new ArdourCanvas::SimpleLine (*time_line_group);
@@ -220,13 +225,10 @@ Editor::initialize_canvas ()
 	transport_punchout_line->property_y1() = 0.0;
 	transport_punchout_line->property_x2() = 0.0;
 	transport_punchout_line->property_y2() = 0.0;
-	transport_punchout_line->property_color_rgba() = color_map[cPunchOutLine];
 	transport_punchout_line->hide();
 	
 	// used to show zoom mode active zooming
 	zoom_rect = new ArdourCanvas::SimpleRect (*track_canvas.root(), 0.0, 0.0, 0.0, 0.0);
-	zoom_rect->property_fill_color_rgba() = color_map[cZoomRectFill];
-	zoom_rect->property_outline_color_rgba() = color_map[cZoomRect];
 	zoom_rect->property_outline_pixels() = 1;
 	zoom_rect->hide();
 	
@@ -234,59 +236,35 @@ Editor::initialize_canvas ()
 	
 	// used as rubberband rect
 	rubberband_rect = new ArdourCanvas::SimpleRect (*track_canvas.root(), 0.0, 0.0, 0.0, 0.0);
-	rubberband_rect->property_outline_color_rgba() = color_map[cRubberBandRect];
-	rubberband_rect->property_fill_color_rgba() = (guint32) color_map[cRubberBandRectFill];
 	rubberband_rect->property_outline_pixels() = 1;
 	rubberband_rect->hide();
 	
 	tempo_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_tempo_bar_event), tempo_bar));
 	meter_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_meter_bar_event), meter_bar));
 	marker_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_marker_bar_event), marker_bar));
+	cd_marker_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_cd_marker_bar_event), cd_marker_bar));
 	range_marker_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_range_marker_bar_event), range_marker_bar));
 	transport_marker_bar->signal_event().connect (bind (mem_fun (*this, &Editor::canvas_transport_marker_bar_event), transport_marker_bar));
 	
-	/* separator lines */
-	
-	tempo_line = new ArdourCanvas::SimpleLine (*tempo_group, 0, timebar_height, max_canvas_coordinate, timebar_height);
-	tempo_line->property_color_rgba() = RGBA_TO_UINT (0,0,0,255);
-
-	meter_line = new ArdourCanvas::SimpleLine (*meter_group, 0, timebar_height, max_canvas_coordinate, timebar_height);
-	meter_line->property_color_rgba() = RGBA_TO_UINT (0,0,0,255);
-
-	marker_line = new ArdourCanvas::SimpleLine (*marker_group, 0, timebar_height, max_canvas_coordinate, timebar_height);
-	marker_line->property_color_rgba() = RGBA_TO_UINT (0,0,0,255);
-	
-	range_marker_line = new ArdourCanvas::SimpleLine (*range_marker_group, 0, timebar_height, max_canvas_coordinate, timebar_height);
-	range_marker_line->property_color_rgba() = RGBA_TO_UINT (0,0,0,255);
-
-	transport_marker_line = new ArdourCanvas::SimpleLine (*transport_marker_group, 0, timebar_height, max_canvas_coordinate, timebar_height);
-	transport_marker_line->property_color_rgba() = RGBA_TO_UINT (0,0,0,255);
-
 	ZoomChanged.connect (bind (mem_fun(*this, &Editor::update_loop_range_view), false));
 	ZoomChanged.connect (bind (mem_fun(*this, &Editor::update_punch_range_view), false));
 	
 	double time_height = timebar_height * 5;
 	double time_width = FLT_MAX/frames_per_unit;
 	time_canvas.set_scroll_region(0.0, 0.0, time_width, time_height);
-	
-	if (!color_map[cEditCursor]) {
-		 warning << _("edit cursor color not defined, check your ardour.colors file!") << endmsg;
-		color_map[cEditCursor] = RGBA_TO_UINT (30,30,30,255);
-	}
 
-	if (!color_map[cPlayHead]) {
-		warning << _("playhead color not defined, check your ardour.colors file!") << endmsg;
-		color_map[cPlayHead] = RGBA_TO_UINT (0,0,0,255);
-	}
-
-	edit_cursor = new Cursor (*this, &Editor::canvas_edit_cursor_event);
-	edit_cursor->canvas_item.property_fill_color_rgba() = color_map[cEditCursor];
 	playhead_cursor = new Cursor (*this, &Editor::canvas_playhead_cursor_event);
-	playhead_cursor->canvas_item.property_fill_color_rgba() = color_map[cPlayHead];
 
 	initial_ruler_update_required = true;
 	track_canvas.signal_size_allocate().connect (mem_fun(*this, &Editor::track_canvas_allocate));
 
+	if (logo_item) {
+		logo_item->lower_to_bottom ();
+	}
+
+	ColorsChanged.connect (mem_fun (*this, &Editor::color_handler));
+	color_handler();
+
 }
 
 void
@@ -304,7 +282,6 @@ Editor::track_canvas_allocate (Gtk::Allocation alloc)
 	} 
 
 	initial_ruler_update_required = false;
-	
 	track_canvas_size_allocated ();
 }
 
@@ -334,48 +311,33 @@ Editor::track_canvas_size_allocated ()
 	}
 
 	zoom_range_clock.set ((nframes_t) floor ((canvas_width * frames_per_unit)));
-	edit_cursor->set_position (edit_cursor->current_frame);
 	playhead_cursor->set_position (playhead_cursor->current_frame);
 
 	reset_hscrollbar_stepping ();
 	reset_scrolling_region ();
 
-	if (edit_cursor) edit_cursor->set_length (canvas_height);
 	if (playhead_cursor) playhead_cursor->set_length (canvas_height);
 
-	if (marker_drag_line) {
-		marker_drag_line_points.back().set_y(canvas_height);
-		marker_drag_line->property_points() = marker_drag_line_points;
+	double y1 = vertical_adjustment.get_value ();
+ 	
+	for (MarkerSelection::iterator x = selection->markers.begin(); x != selection->markers.end(); ++x) {
+		(*x)->set_line_vpos (y1, canvas_height);
 	}
 
-	if (range_marker_drag_rect) {
-		range_marker_drag_rect->property_y1() = 0.0;
-		range_marker_drag_rect->property_y2() = canvas_height;
-	}
-
-	if (transport_loop_range_rect) {
-		transport_loop_range_rect->property_y1() = 0.0;
-		transport_loop_range_rect->property_y2() = canvas_height;
-	}
-
-	if (transport_punch_range_rect) {
-		transport_punch_range_rect->property_y1() = 0.0;
-		transport_punch_range_rect->property_y2() = canvas_height;
-	}
-
-	if (transport_punchin_line) {
-		transport_punchin_line->property_y1() = 0.0;
-		transport_punchin_line->property_y2() = canvas_height;
-	}
-
-	if (transport_punchout_line) {
-		transport_punchout_line->property_y1() = 0.0;
-		transport_punchout_line->property_y2() = canvas_height;
-	}
-		
+ 	range_marker_drag_rect->property_y1() = y1;
+ 	range_marker_drag_rect->property_y2() = y1 + canvas_height;
+ 	transport_loop_range_rect->property_y1() = y1;
+ 	transport_loop_range_rect->property_y2() = y1 + canvas_height;
+ 	transport_punch_range_rect->property_y1() = y1;
+ 	transport_punch_range_rect->property_y2() = y1 + canvas_height;
+ 	transport_punchin_line->property_y1() = y1;
+ 	transport_punchin_line->property_y2() = y1 + canvas_height;
+ 	transport_punchout_line->property_y1() = y1;
+ 	transport_punchout_line->property_y2() = y1 + canvas_height;
+	
 	update_fixed_rulers();
 	redisplay_tempo (true);
-	
+
 	Resized (); /* EMIT_SIGNAL */
 
 	return false;
@@ -395,12 +357,20 @@ Editor::reset_scrolling_region (Gtk::Allocation* alloc)
 		}
 	}
 
-	double last_canvas_unit =  last_canvas_frame / frames_per_unit;
+	double last_canvas_unit =  max ((last_canvas_frame / frames_per_unit), canvas_width);
 
-	track_canvas.set_scroll_region (0.0, 0.0, max (last_canvas_unit, canvas_width), pos);
+	track_canvas.set_scroll_region (0.0, 0.0, last_canvas_unit, pos);
 
 	// XXX what is the correct height value for the time canvas ? this overstates it
-	time_canvas.set_scroll_region ( 0.0, 0.0, max (last_canvas_unit, canvas_width), canvas_height);
+	time_canvas.set_scroll_region ( 0.0, 0.0, last_canvas_unit, canvas_height);
+
+	range_marker_drag_rect->property_y2() = canvas_height;
+	transport_loop_range_rect->property_y2() = canvas_height;
+	transport_punch_range_rect->property_y2() = canvas_height;
+	transport_punchin_line->property_y2() = canvas_height;
+	transport_punchout_line->property_y2() = canvas_height;
+
+	update_punch_range_view (true);
 
 	controls_layout.queue_resize();
 }
@@ -412,8 +382,8 @@ Editor::controls_layout_size_request (Requisition* req)
 	TreeModel::Children::iterator i;
 	double pos;
 
-        for (pos = 0, i = rows.begin(); i != rows.end(); ++i) {
-	        TimeAxisView *tv = (*i)[route_display_columns.tv];
+	for (pos = 0, i = rows.begin(); i != rows.end(); ++i) {
+		TimeAxisView *tv = (*i)[route_display_columns.tv];
 		if (tv != 0) {
 			pos += tv->effective_height;
 		}
@@ -425,12 +395,9 @@ Editor::controls_layout_size_request (Requisition* req)
 		screen = Gdk::Screen::get_default();
 	}
 
-	/* never let the width of the controls area shrink horizontally */
-
 	edit_controls_vbox.check_resize();
-
 	req->width = max (edit_controls_vbox.get_width(),  controls_layout.get_width());
-	
+
 	/* don't get too big. the fudge factors here are just guesses */
 	
 	req->width = min (req->width, screen->get_width() - 300);
@@ -439,8 +406,13 @@ Editor::controls_layout_size_request (Requisition* req)
 	/* this one is important: it determines how big the layout thinks it really is, as 
 	   opposed to what it displays on the screen
 	*/
+	
+	controls_layout.set_size (edit_controls_vbox.get_width(), (gint) pos);
+	controls_layout.set_size_request(edit_controls_vbox.get_width(), -1);
+	zoom_box.set_size_request(edit_controls_vbox.get_width(), -1);
+	time_button_frame.set_size_request(edit_controls_vbox.get_width() + edit_vscrollbar.get_width(), -1);
 
-	controls_layout.set_size (req->width, (gint) pos);
+	//cerr << "sizes = " << req->width << " " << edit_controls_vbox.get_width() << " " << controls_layout.get_width() << " " << zoom_box.get_width() << " " << time_button_frame.get_width() << endl;//DEBUG	
 }
 
 bool
@@ -482,7 +454,7 @@ Editor::drop_paths (const RefPtr<Gdk::DragContext>& context,
 	vector<ustring> paths;
 	string spath;
 	GdkEvent ev;
-	nframes_t frame;
+	nframes64_t frame;
 
 	if (convert_drop_to_paths (paths, context, x, y, data, info, time)) {
 		goto out;
@@ -510,15 +482,18 @@ Editor::drop_paths (const RefPtr<Gdk::DragContext>& context,
 
 		/* drop onto canvas background: create new tracks */
 
-		nframes_t pos = 0;
-		do_embed (paths, false, ImportAsTrack, 0, pos, false);
+		frame = 0;
+
+		do_embed (paths, Editing::ImportDistinctFiles, ImportAsTrack, frame);
 		
 	} else if ((tv = dynamic_cast<AudioTimeAxisView*>(tvp)) != 0) {
 
 		/* check that its an audio track, not a bus */
 		
 		if (tv->get_diskstream()) {
-			do_embed (paths, false, ImportToTrack, tv->audio_track(), frame, true);
+			/* select the track, then embed */
+			selection->set (tv);
+			do_embed (paths, Editing::ImportDistinctFiles, ImportToTrack, frame);
 		}
 	}
 
@@ -709,26 +684,138 @@ Editor::left_track_canvas (GdkEventCrossing *ev)
 	return FALSE;
 }
 
+void
+Editor::tie_vertical_scrolling ()
+{
+	double y1 = vertical_adjustment.get_value();
+
+	playhead_cursor->set_y_axis (y1);
+
+ 	range_marker_drag_rect->property_y1() = y1;
+ 	range_marker_drag_rect->property_y2() = y1 + canvas_height;
+ 	transport_loop_range_rect->property_y1() = y1;
+ 	transport_loop_range_rect->property_y2() = y1 + canvas_height;
+ 	transport_punch_range_rect->property_y1() = y1;
+ 	transport_punch_range_rect->property_y2() = y1 + canvas_height;
+ 	transport_punchin_line->property_y1() = y1;
+ 	transport_punchin_line->property_y2() = y1 + canvas_height;
+ 	transport_punchout_line->property_y1() = y1;
+ 	transport_punchout_line->property_y2() = y1 + canvas_height;
+
+	if (!selection->markers.empty()) {
+		for (MarkerSelection::iterator x = selection->markers.begin(); x != selection->markers.end(); ++x) {		
+			(*x)->set_line_vpos (y1, canvas_height);
+		}
+	}
+
+	if (logo_item) {
+		logo_item->property_y() = y1;
+	}
+
+	/* this will do an immediate redraw */
+
+	controls_layout.get_vadjustment()->set_value (y1);
+}
 
 void 
 Editor::canvas_horizontally_scrolled ()
 {
-	/* this is the core function that controls horizontal scrolling of the canvas. it is called
-	   whenever the horizontal_adjustment emits its "value_changed" signal. it typically executes in an
-	   idle handler, which is important because tempo_map_changed() should issue redraws immediately
-	   and not defer them to an idle handler.
-	*/
+	nframes64_t time_origin = (nframes_t) floor (horizontal_adjustment.get_value() * frames_per_unit);
 
-  	leftmost_frame = (nframes_t) floor (horizontal_adjustment.get_value() * frames_per_unit);
+	if (time_origin != leftmost_frame) {
+		canvas_scroll_to (time_origin);
+	}
+}
+
+void
+Editor::canvas_scroll_to (nframes64_t time_origin)
+{
+  	leftmost_frame = time_origin;
 	nframes_t rightmost_frame = leftmost_frame + current_page_frames ();
-	
+
 	if (rightmost_frame > last_canvas_frame) {
 		last_canvas_frame = rightmost_frame;
 		reset_scrolling_region ();
 	}
-	
+
+	if (logo_item) {
+		logo_item->property_x() = horizontal_adjustment.get_value ();
+	}
+
 	update_fixed_rulers ();
 
 	redisplay_tempo (!_dragging_hscrollbar);
 }
 
+void
+Editor::color_handler()
+{
+	playhead_cursor->canvas_item.property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_PlayHead.get();
+	verbose_canvas_cursor->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_VerboseCanvasCursor.get();
+
+	meter_bar->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_MeterBar.get();
+	meter_bar->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBarSeparator.get();
+
+	tempo_bar->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_TempoBar.get();
+	tempo_bar->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBarSeparator.get();
+
+	marker_bar->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBar.get();
+	marker_bar->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBarSeparator.get();
+
+	cd_marker_bar->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_CDMarkerBar.get();
+	cd_marker_bar->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBarSeparator.get();
+
+	range_marker_bar->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeMarkerBar.get();
+	range_marker_bar->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBarSeparator.get();
+
+	transport_marker_bar->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_TransportMarkerBar.get();
+	transport_marker_bar->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerBarSeparator.get();
+
+	range_bar_drag_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragBarRect.get();
+	range_bar_drag_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragBarRect.get();
+
+	transport_bar_drag_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_TransportDragRect.get();
+	transport_bar_drag_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_TransportDragRect.get();
+
+	marker_drag_line->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_MarkerDragLine.get();
+
+	range_marker_drag_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragRect.get();
+	range_marker_drag_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_RangeDragRect.get();
+
+	transport_loop_range_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_TransportLoopRect.get();
+	transport_loop_range_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_TransportLoopRect.get();
+
+	transport_punch_range_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_TransportPunchRect.get();
+	transport_punch_range_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_TransportPunchRect.get();
+
+	transport_punchin_line->property_color_rgba() = ARDOUR_UI::config()->canvasvar_PunchLine.get();
+	transport_punchout_line->property_color_rgba() = ARDOUR_UI::config()->canvasvar_PunchLine.get();
+
+	zoom_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_ZoomRect.get();
+	zoom_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_ZoomRect.get();
+
+	rubberband_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_RubberBandRect.get();
+	rubberband_rect->property_fill_color_rgba() = (guint32) ARDOUR_UI::config()->canvasvar_RubberBandRect.get();
+
+	location_marker_color = ARDOUR_UI::config()->canvasvar_LocationMarker.get();
+	location_range_color = ARDOUR_UI::config()->canvasvar_LocationRange.get();
+	location_cd_marker_color = ARDOUR_UI::config()->canvasvar_LocationCDMarker.get();
+	location_loop_color = ARDOUR_UI::config()->canvasvar_LocationLoop.get();
+	location_punch_color = ARDOUR_UI::config()->canvasvar_LocationPunch.get();
+
+	refresh_location_display ();
+	redisplay_tempo (true);
+
+	if (session)
+		session->tempo_map().apply_with_metrics (*this, &Editor::draw_metric_marks); // redraw metric markers
+}
+
+void
+Editor::flush_canvas ()
+{
+	if (is_mapped()) {
+		track_canvas.update_now ();
+		// gdk_window_process_updates (GTK_LAYOUT(track_canvas.gobj())->bin_window, true);
+	}
+}
+
