@@ -26,6 +26,7 @@
 #include <gtkmm2ext/window_title.h>
 
 #include <ardour/location.h>
+#include <ardour/profile.h>
 #include <pbd/memento_command.h>
 
 #include "editor.h"
@@ -77,9 +78,15 @@ Editor::add_new_location (Location *location)
 	}
 
 	if (location->is_mark()) {
-		lam->start = new Marker (*this, *marker_group, color, location->name(), Marker::Mark, location->start());
-		lam->end   = 0;
 
+		if (location->is_cd_marker() && ruler_shown[ruler_time_cd_marker]) {
+			lam->start = new Marker (*this, *cd_marker_group, color, location->name(), Marker::Mark, location->start());
+		}
+		else {
+			lam->start = new Marker (*this, *marker_group, color, location->name(), Marker::Mark, location->start());
+		}
+		lam->end   = 0;
+			
 	} else if (location->is_auto_loop()) {
 		// transport marker
 		lam->start = new Marker (*this, *transport_marker_group, color, 
@@ -95,12 +102,20 @@ Editor::add_new_location (Location *location)
 					 location->name(), Marker::PunchOut, location->end());
 		
 	} else {
-
 		// range marker
-		lam->start = new Marker (*this, *range_marker_group, color, 
-					 location->name(), Marker::Start, location->start());
-		lam->end   = new Marker (*this, *range_marker_group, color, 
-					 location->name(), Marker::End, location->end());
+		if (location->is_cd_marker() && ruler_shown[ruler_time_cd_marker]) {
+			lam->start = new Marker (*this, *cd_marker_group, color, 
+						 location->name(), Marker::Start, location->start());
+			lam->end   = new Marker (*this, *cd_marker_group, color, 
+						 location->name(), Marker::End, location->end());
+		}
+		else {
+			
+			lam->start = new Marker (*this, *range_marker_group, color, 
+						 location->name(), Marker::Start, location->start());
+			lam->end   = new Marker (*this, *range_marker_group, color, 
+						 location->name(), Marker::End, location->end());
+		}
 	}
 
 	if (location->is_hidden ()) {
@@ -121,6 +136,11 @@ Editor::add_new_location (Location *location)
 	newpair.second = lam;
 
 	location_markers.insert (newpair);
+
+	if (select_new_marker && location->is_mark()) {
+		selection->set (lam->start);
+		select_new_marker = false;
+	}
 }
 
 void
@@ -157,6 +177,9 @@ Editor::location_flags_changed (Location *location, void *src)
 		return;
 	}
 
+	// move cd markers to/from cd marker bar as appropriate
+	ensure_cd_marker_updated (lam, location);
+
 	if (location->is_cd_marker()) {
 		lam->set_color_rgba (location_cd_marker_color);
 	} else if (location->is_mark()) {
@@ -176,6 +199,52 @@ Editor::location_flags_changed (Location *location, void *src)
 	}
 }
 
+void Editor::update_cd_marker_display ()
+{
+	for (LocationMarkerMap::iterator i = location_markers.begin(); i != location_markers.end(); ++i) {
+		LocationMarkers * lam = i->second;
+		Location * location = i->first;
+
+		ensure_cd_marker_updated (lam, location);
+	}
+}
+
+void Editor::ensure_cd_marker_updated (LocationMarkers * lam, Location * location)
+{
+	if (location->is_cd_marker()
+	    && (ruler_shown[ruler_time_cd_marker] &&  lam->start->get_parent() != cd_marker_group))
+	{
+		//cerr << "reparenting non-cd marker so it can be relocated: " << location->name() << endl;
+		if (lam->start) {
+			lam->start->reparent (*cd_marker_group);
+		}
+		if (lam->end) {
+			lam->end->reparent (*cd_marker_group);
+		}
+	}
+	else if ( (!location->is_cd_marker() || !ruler_shown[ruler_time_cd_marker]) 
+		  && (lam->start->get_parent() == cd_marker_group))  
+	{
+		//cerr << "reparenting non-cd marker so it can be relocated: " << location->name() << endl;
+		if (location->is_mark()) {
+			if (lam->start) {
+				lam->start->reparent (*marker_group);
+			}
+			if (lam->end) {
+				lam->end->reparent (*marker_group);
+			}
+		}
+		else {
+			if (lam->start) {
+				lam->start->reparent (*range_marker_group);
+			}
+			if (lam->end) {
+				lam->end->reparent (*range_marker_group);
+			}
+		}
+	}
+}
+
 Editor::LocationMarkers::~LocationMarkers ()
 {
 	if (start) {
@@ -188,9 +257,9 @@ Editor::LocationMarkers::~LocationMarkers ()
 }
 
 Editor::LocationMarkers *
-Editor::find_location_markers (Location *location)
+Editor::find_location_markers (Location *location) const
 {
-	LocationMarkerMap::iterator i;
+	LocationMarkerMap::const_iterator i;
 
 	for (i = location_markers.begin(); i != location_markers.end(); ++i) {
 		if ((*i).first == location) {
@@ -202,9 +271,9 @@ Editor::find_location_markers (Location *location)
 }
 
 Location *
-Editor::find_location_from_marker (Marker *marker, bool& is_start)
+Editor::find_location_from_marker (Marker *marker, bool& is_start) const
 {
-	LocationMarkerMap::iterator i;
+	LocationMarkerMap::const_iterator i;
 
 	for (i = location_markers.begin(); i != location_markers.end(); ++i) {
 		LocationMarkers *lm = (*i).second;
@@ -223,11 +292,43 @@ Editor::find_location_from_marker (Marker *marker, bool& is_start)
 void
 Editor::refresh_location_display_internal (Locations::LocationList& locations)
 {
-	clear_marker_display ();
+	/* invalidate all */
+
+	for (LocationMarkerMap::iterator i = location_markers.begin(); i != location_markers.end(); ++i) {
+		i->second->valid = false;
+	}
 	
+	/* add new ones */
+
 	for (Locations::LocationList::iterator i = locations.begin(); i != locations.end(); ++i) {
+
+		LocationMarkerMap::iterator x;
+
+		if ((x = location_markers.find (*i)) != location_markers.end()) {
+			x->second->valid = true;
+			continue;
+		}
+
 		add_new_location (*i);
 	}
+
+	/* remove dead ones */
+
+	for (LocationMarkerMap::iterator i = location_markers.begin(); i != location_markers.end(); ) {
+
+		LocationMarkerMap::iterator tmp;
+
+		tmp = i;
+		++tmp;
+
+		if (!i->second->valid) {
+			delete i->second;
+			location_markers.erase (i);
+		} 
+
+		i = tmp;
+	}
+	
 }
 
 void
@@ -287,18 +388,28 @@ Editor::LocationMarkers::set_color_rgba (uint32_t rgba)
 }
 
 void
-Editor::mouse_add_new_marker (nframes_t where)
+Editor::mouse_add_new_marker (nframes_t where, bool is_cd)
 {
 	string markername;
+	int flags = (is_cd ? Location::IsCDMarker|Location::IsMark : Location::IsMark);
+	
 	if (session) {
 		session->locations()->next_available_name(markername,"mark");
-		Location *location = new Location (where, where, markername, Location::IsMark);
+		Location *location = new Location (where, where, markername, (Location::Flags) flags);
 		session->begin_reversible_command (_("add marker"));
                 XMLNode &before = session->locations()->get_state();
 		session->locations()->add (location, true);
                 XMLNode &after = session->locations()->get_state();
 		session->add_command (new MementoCommand<Locations>(*(session->locations()), &before, &after));
 		session->commit_reversible_command ();
+
+		/* find the marker we just added */
+
+		LocationMarkers *lam = find_location_markers (location);
+		if (lam) {
+			/* make it the selected marker */
+			selection->set (lam->start);
+		}
 	}
 }
 
@@ -313,6 +424,10 @@ Editor::remove_marker (ArdourCanvas::Item& item, GdkEvent* event)
 		/*NOTREACHED*/
 	}
 
+	if (entered_marker == marker) {
+		entered_marker = NULL;
+	}
+
 	Location* loc = find_location_from_marker (marker, is_start);
 
 	if (session && loc) {
@@ -464,17 +579,21 @@ Editor::build_marker_menu (bool start_or_end)
 	MenuList& items = markerMenu->items();
 	markerMenu->set_name ("ArdourContextMenu");
 
-	items.push_back (MenuElem (_("Locate to Mark"), mem_fun(*this, &Editor::marker_menu_set_playhead)));
-	items.push_back (MenuElem (_("Play from Mark"), mem_fun(*this, &Editor::marker_menu_play_from)));
-	items.push_back (MenuElem (_("Set Mark from Playhead"), mem_fun(*this, &Editor::marker_menu_set_from_playhead)));
+	items.push_back (MenuElem (_("Locate to here"), mem_fun(*this, &Editor::marker_menu_set_playhead)));
+	items.push_back (MenuElem (_("Play from here"), mem_fun(*this, &Editor::marker_menu_play_from)));
+	items.push_back (MenuElem (_("Move Mark to Playhead"), mem_fun(*this, &Editor::marker_menu_set_from_playhead)));
 
 	items.push_back (SeparatorElem());
 
-	items.push_back (MenuElem (_("Hide Mark"), mem_fun(*this, &Editor::marker_menu_hide)));
+	items.push_back (MenuElem (_("Hide"), mem_fun(*this, &Editor::marker_menu_hide)));
 	if (start_or_end) return;
-	items.push_back (MenuElem (_("Rename Mark"), mem_fun(*this, &Editor::marker_menu_rename)));
-	items.push_back (MenuElem (_("Remove Mark"), mem_fun(*this, &Editor::marker_menu_remove)));
+	items.push_back (MenuElem (_("Rename"), mem_fun(*this, &Editor::marker_menu_rename)));
+	items.push_back (MenuElem (_("Lock"), bind (mem_fun(*this, &Editor::marker_menu_lock), true)));
+	items.push_back (MenuElem (_("Unlock"), bind (mem_fun(*this, &Editor::marker_menu_lock), false)));
 
+	items.push_back (SeparatorElem());
+
+	items.push_back (MenuElem (_("Remove"), mem_fun(*this, &Editor::marker_menu_remove)));
 }
 
 void
@@ -492,19 +611,21 @@ Editor::build_range_marker_menu (bool loop_or_punch)
 	MenuList& items = markerMenu->items();
 	markerMenu->set_name ("ArdourContextMenu");
 
+	items.push_back (MenuElem (_("Play Range"), mem_fun(*this, &Editor::marker_menu_play_range)));
 	items.push_back (MenuElem (_("Locate to Range Mark"), mem_fun(*this, &Editor::marker_menu_set_playhead)));
 	items.push_back (MenuElem (_("Play from Range Mark"), mem_fun(*this, &Editor::marker_menu_play_from)));
 	if (!loop_or_punch) {
-		items.push_back (MenuElem (_("Play Range"), mem_fun(*this, &Editor::marker_menu_play_range)));
 		items.push_back (MenuElem (_("Loop Range"), mem_fun(*this, &Editor::marker_menu_loop_range)));
 	}
 	items.push_back (MenuElem (_("Set Range Mark from Playhead"), mem_fun(*this, &Editor::marker_menu_set_from_playhead)));
-	items.push_back (MenuElem (_("Set Range from Range Selection"), mem_fun(*this, &Editor::marker_menu_set_from_selection)));
+	if (!Profile->get_sae()) {
+		items.push_back (MenuElem (_("Set Range from Range Selection"), mem_fun(*this, &Editor::marker_menu_set_from_selection)));
+	}
 
 	items.push_back (SeparatorElem());
 
-	items.push_back (MenuElem (_("Hide Range"), mem_fun(*this, &Editor::marker_menu_hide)));
 	if (!loop_or_punch) {
+		items.push_back (MenuElem (_("Hide Range"), mem_fun(*this, &Editor::marker_menu_hide)));
 		items.push_back (MenuElem (_("Rename Range"), mem_fun(*this, &Editor::marker_menu_rename)));
 		items.push_back (MenuElem (_("Remove Range"), mem_fun(*this, &Editor::marker_menu_remove)));
 	}
@@ -513,8 +634,9 @@ Editor::build_range_marker_menu (bool loop_or_punch)
 
 	items.push_back (MenuElem (_("Separate Regions in Range"), mem_fun(*this, &Editor::marker_menu_separate_regions_using_location)));
 	items.push_back (MenuElem (_("Select All in Range"), mem_fun(*this, &Editor::marker_menu_select_all_selectables_using_range)));
-	items.push_back (MenuElem (_("Select Range"), mem_fun(*this, &Editor::marker_menu_select_using_range)));
-
+	if (!Profile->get_sae()) {
+		items.push_back (MenuElem (_("Select Range"), mem_fun(*this, &Editor::marker_menu_select_using_range)));
+	}
 }
 
 void
@@ -693,13 +815,13 @@ Editor::marker_menu_set_from_playhead ()
 	if ((l = find_location_from_marker (marker, is_start)) != 0) {
 
 		if (l->is_mark()) {
-			l->set_start (session->transport_frame ());
+			l->set_start (session->audible_frame ());
 		}
 		else {
 			if (is_start) {
-				l->set_start (session->transport_frame ());
+				l->set_start (session->audible_frame ());
 			} else {
-				l->set_end (session->transport_frame ());
+				l->set_end (session->audible_frame ());
 			}
 		}
 	}
@@ -837,6 +959,31 @@ Editor::marker_menu_remove ()
 	}
 }
 
+void
+Editor::marker_menu_lock (bool yn)
+{
+
+	Marker* marker;
+
+	if ((marker = reinterpret_cast<Marker *> (marker_menu_item->get_data ("marker"))) == 0) {
+		fatal << _("programming error: marker canvas item has no marker object pointer!") << endmsg;
+		/*NOTREACHED*/
+	}
+
+	Location* loc;
+	bool ignored;
+
+	loc = find_location_from_marker (marker, ignored);
+
+	if (!loc) return;
+
+	if (yn) {
+		loc->lock();
+	} else {
+		loc->unlock ();
+	}
+}
+
 void
 Editor::marker_menu_rename ()
 {
@@ -958,9 +1105,15 @@ Editor::update_punch_range_view (bool visibility)
 		double x1 = frame_to_pixel (tpl->start());
 		double x2 = frame_to_pixel (tpl->end());
 		
+		guint track_canvas_width,track_canvas_height;
+		track_canvas.get_size(track_canvas_width,track_canvas_height);
+		
 		transport_punch_range_rect->property_x1() = x1;
 		transport_punch_range_rect->property_x2() = x2;
 		
+		transport_punch_range_rect->property_x1() = (Config->get_punch_in() ? x1 : 0);
+		transport_punch_range_rect->property_x2() = (Config->get_punch_out() ? x2 : track_canvas_width);
+		
 		if (visibility) {
 		        transport_punch_range_rect->show();
 		}
@@ -994,3 +1147,78 @@ Editor::update_punch_range_view (bool visibility)
 // 		gnome_canvas_item_hide (transport_punchout_line);
 // 	}
 }
+
+void
+Editor::marker_selection_changed ()
+{
+	for (LocationMarkerMap::iterator i = location_markers.begin(); i != location_markers.end(); ++i) {
+		LocationMarkers* lam = i->second;
+
+		if (lam->start) {
+			lam->start->hide_line();
+		} 
+
+		if (lam->end) {
+			lam->end->hide_line();
+		}
+	}
+
+	edit_point_clock_connection_a.disconnect();
+	edit_point_clock_connection_b.disconnect();
+
+	if (selection->markers.empty()) {
+		edit_point_clock.set (0);
+		return;
+	}
+
+	for (MarkerSelection::iterator x = selection->markers.begin(); x != selection->markers.end(); ++x) {
+		(*x)->add_line (cursor_group, vertical_adjustment.get_value(), canvas_height);
+		(*x)->show_line ();
+	}
+
+	edit_point_clock.set (selection->markers.front()->position());
+
+	bool ignored;
+	Location* loc = find_location_from_marker (selection->markers.front(), ignored);
+
+	if (loc) {
+		edit_point_clock_connection_a = loc->changed.connect (mem_fun (*this, &Editor::selected_marker_moved));
+		edit_point_clock_connection_b = loc->start_changed.connect (mem_fun (*this, &Editor::selected_marker_moved));
+	}
+}
+
+void
+Editor::selected_marker_moved (Location* loc)
+{
+	edit_point_clock.set (loc->start());
+}
+
+struct SortLocationsByPosition { 
+    bool operator() (Location* a, Location* b) {
+	    return a->start() < b->start();
+    }
+};
+
+void
+Editor::goto_nth_marker (int n)
+{
+	if (!session) {
+		return;
+	}
+	const Locations::LocationList& l (session->locations()->list());
+	Locations::LocationList ordered;
+	ordered = l;
+
+	SortLocationsByPosition cmp;
+	ordered.sort (cmp);
+	
+	for (Locations::LocationList::iterator i = ordered.begin(); n >= 0 && i != ordered.end(); ++i) {
+		if ((*i)->is_mark() && !(*i)->is_hidden() && !(*i)->is_start()) {
+			if (n == 0) {
+				session->request_locate ((*i)->start(), session->transport_rolling());
+				break;
+			}
+			--n;
+		}
+	}
+}
