@@ -17,6 +17,9 @@
 
 */
 
+#include <algorithm>
+#include <stdlib.h>
+
 #include <pbd/stacktrace.h>
 
 #include <ardour/diskstream.h>
@@ -159,36 +162,47 @@ Editor::select_all_tracks ()
 	selection->set (track_views);
 }
 
-bool
+void
+Editor::set_selected_track_as_side_effect (bool force)
+{
+	if (!clicked_trackview) {
+		return;
+	}
+
+	if (!selection->tracks.empty()) {
+		if (!selection->selected (clicked_trackview)) {
+			selection->add (clicked_trackview);
+		}
+
+	} else if (force) {
+		selection->set (clicked_trackview);
+	}
+}
+
+void
 Editor::set_selected_track (TimeAxisView& view, Selection::Operation op, bool no_remove)
 {
-	bool commit = false;
 
 	switch (op) {
 	case Selection::Toggle:
 		if (selection->selected (&view)) {
 			if (!no_remove) {
 				selection->remove (&view);
-				commit = true;
 			}
 		} else {
 			selection->add (&view);
-			commit = false;
 		}
 		break;
 
 	case Selection::Add:
 		if (!selection->selected (&view)) {
 			selection->add (&view);
-			commit = true;
 		}
 		break;
 
 	case Selection::Set:
-		if (selection->selected (&view) && selection->tracks.size() == 1) {
-			/* no commit necessary */
-		} else {
-			
+		if (selection->selected (&view) && selection->tracks.size() > 1) {
+
 			/* reset track selection if there is only 1 other track
 			   selected OR if no_remove is not set (its there to 
 			   prevent deselecting a multi-track selection
@@ -198,34 +212,30 @@ Editor::set_selected_track (TimeAxisView& view, Selection::Operation op, bool no
 
 			if (selection->tracks.empty()) {
 				selection->set (&view);
-				commit = true;
 			} else if (selection->tracks.size() == 1 || !no_remove) {
 				selection->set (&view);
-				commit = true;
 			}
 		}
 		break;
 		
 	case Selection::Extend:
-		commit = extend_selection_to_track (view);
+		extend_selection_to_track (view);
 		break;
 	}
-
-	return commit;
 }
 
-bool
+void
 Editor::set_selected_track_from_click (bool press, Selection::Operation op, bool no_remove)
 {
 	if (!clicked_trackview) {
-		return false;
+		return;
 	}
 	
 	if (!press) {
-		return false;
+		return;
 	}
 
-	return set_selected_track (*clicked_trackview, op, no_remove);
+	set_selected_track (*clicked_trackview, op, no_remove);
 }
 
 bool
@@ -284,18 +294,47 @@ Editor::get_relevant_audio_tracks (set<AudioTimeAxisView*>& relevant_tracks)
 	}
 }
 
+
+/**
+ *  Call a slot for a given `basis' track and also for any track that is in the same
+ *  active edit group.
+ *  @param sl Slot to call.
+ *  @param basis Basis track.
+ */
+
 void
-Editor::mapover_audio_tracks (slot<void,AudioTimeAxisView&,uint32_t> sl)
+Editor::mapover_audio_tracks (slot<void,AudioTimeAxisView&,uint32_t> sl, TimeAxisView* basis)
 {
-	set<AudioTimeAxisView*> relevant_tracks;
-
-	get_relevant_audio_tracks (relevant_tracks);
-
-	uint32_t sz = relevant_tracks.size();
-
-	for (set<AudioTimeAxisView*>::iterator ati = relevant_tracks.begin(); ati != relevant_tracks.end(); ++ati) {
-		sl (**ati, sz);
+	AudioTimeAxisView* audio_basis = dynamic_cast<AudioTimeAxisView*> (basis);
+	if (audio_basis == 0) {
+		return;
 	}
+
+	/* work out the tracks that we will call the slot for; use
+	   a set here as it will disallow possible duplicates of the
+	   basis track */
+	set<AudioTimeAxisView*> tracks;
+
+	/* always call for the basis */
+	tracks.insert (audio_basis);
+
+	RouteGroup* group = audio_basis->route()->edit_group();
+	if (group && group->is_active()) {
+
+		/* the basis is a member of an active edit group; find other members */
+		for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
+			AudioTimeAxisView* v = dynamic_cast<AudioTimeAxisView*> (*i);
+			if (v && v->route()->edit_group() == group) {
+				tracks.insert (v);
+			}
+		}
+	}
+
+	/* call the slots */
+	uint32_t const sz = tracks.size ();
+	for (set<AudioTimeAxisView*>::iterator i = tracks.begin(); i != tracks.end(); ++i) {
+		sl (**i, sz);
+ 	}
 }
 
 void
@@ -330,7 +369,7 @@ Editor::mapped_get_equivalent_regions (RouteTimeAxisView& tv, uint32_t ignored,
 void
 Editor::get_equivalent_regions (RegionView* basis, vector<RegionView*>& equivalent_regions)
 {
-	mapover_audio_tracks (bind (mem_fun (*this, &Editor::mapped_get_equivalent_regions), basis, &equivalent_regions));
+	mapover_audio_tracks (bind (mem_fun (*this, &Editor::mapped_get_equivalent_regions), basis, &equivalent_regions), &basis->get_trackview());
 	
 	/* add clicked regionview since we skipped all other regions in the same track as the one it was in */
 	
@@ -357,7 +396,7 @@ Editor::set_selected_regionview_from_click (bool press, Selection::Operation op,
 		switch (op) {
 		case Selection::Toggle:
 			
-			if (clicked_regionview->get_selected()) {
+			if (selection->selected (clicked_regionview)) {
 				if (press) {
 
 					/* whatever was clicked was selected already; do nothing here but allow
@@ -387,11 +426,11 @@ Editor::set_selected_regionview_from_click (bool press, Selection::Operation op,
 
 				if (press) {
 
-					if (selection->selected (clicked_audio_trackview)) {
-						get_equivalent_regions (clicked_regionview, all_equivalent_regions);
-					} else {
-						all_equivalent_regions.push_back (clicked_regionview);
-					}
+                                       if (selection->selected (clicked_audio_trackview)) {
+                                               get_equivalent_regions (clicked_regionview, all_equivalent_regions);
+                                       } else {
+                                               all_equivalent_regions.push_back (clicked_regionview);
+                                       }
 
 					/* add all the equivalent regions, but only on button press */
 					
@@ -407,14 +446,9 @@ Editor::set_selected_regionview_from_click (bool press, Selection::Operation op,
 			break;
 			
 		case Selection::Set:
-			if (!clicked_regionview->get_selected()) {
-
-				if (selection->selected (clicked_audio_trackview)) {
-					get_equivalent_regions (clicked_regionview, all_equivalent_regions);
-				} else {
-					all_equivalent_regions.push_back (clicked_regionview);
-				}
+			if (!selection->selected (clicked_regionview)) {
 
+				get_equivalent_regions (clicked_regionview, all_equivalent_regions);
 				selection->set (all_equivalent_regions);
 				commit = true;
 			} else {
@@ -433,6 +467,7 @@ Editor::set_selected_regionview_from_click (bool press, Selection::Operation op,
 		list<Selectable*> results;
 		nframes_t last_frame;
 		nframes_t first_frame;
+		bool same_track = false;
 
 		/* 1. find the last selected regionview in the track that was clicked in */
 
@@ -449,63 +484,181 @@ Editor::set_selected_regionview_from_click (bool press, Selection::Operation op,
 				if ((*x)->region()->first_frame() < first_frame) {
 					first_frame = (*x)->region()->first_frame();
 				}
+
+				same_track = true;
 			}
 		}
 
-		/* 2. figure out the boundaries for our search for new objects */
+		if (same_track) {
 
-		switch (clicked_regionview->region()->coverage (first_frame, last_frame)) {
-		case OverlapNone:
-			if (last_frame < clicked_regionview->region()->first_frame()) {
-				first_frame = last_frame;
-				last_frame = clicked_regionview->region()->last_frame();
-			} else {
-				last_frame = first_frame;
-				first_frame = clicked_regionview->region()->first_frame();
+			/* 2. figure out the boundaries for our search for new objects */
+			
+			switch (clicked_regionview->region()->coverage (first_frame, last_frame)) {
+			case OverlapNone:
+				if (last_frame < clicked_regionview->region()->first_frame()) {
+					first_frame = last_frame;
+					last_frame = clicked_regionview->region()->last_frame();
+				} else {
+					last_frame = first_frame;
+					first_frame = clicked_regionview->region()->first_frame();
+				}
+				break;
+				
+			case OverlapExternal:
+				if (last_frame < clicked_regionview->region()->first_frame()) {
+					first_frame = last_frame;
+					last_frame = clicked_regionview->region()->last_frame();
+				} else {
+					last_frame = first_frame;
+					first_frame = clicked_regionview->region()->first_frame();
+				}
+				break;
+				
+			case OverlapInternal:
+				if (last_frame < clicked_regionview->region()->first_frame()) {
+					first_frame = last_frame;
+					last_frame = clicked_regionview->region()->last_frame();
+				} else {
+					last_frame = first_frame;
+					first_frame = clicked_regionview->region()->first_frame();
+				}
+				break;
+				
+			case OverlapStart:
+			case OverlapEnd:
+				/* nothing to do except add clicked region to selection, since it
+				   overlaps with the existing selection in this track.
+				*/
+				break;
 			}
-			break;
 
-		case OverlapExternal:
-			if (last_frame < clicked_regionview->region()->first_frame()) {
-				first_frame = last_frame;
-				last_frame = clicked_regionview->region()->last_frame();
-			} else {
-				last_frame = first_frame;
-				first_frame = clicked_regionview->region()->first_frame();
-			}
-			break;
+		} else {
 
-		case OverlapInternal:
-			if (last_frame < clicked_regionview->region()->first_frame()) {
-				first_frame = last_frame;
-				last_frame = clicked_regionview->region()->last_frame();
-			} else {
-				last_frame = first_frame;
-				first_frame = clicked_regionview->region()->first_frame();
-			}
-			break;
-
-		case OverlapStart:
-		case OverlapEnd:
-			/* nothing to do except add clicked region to selection, since it
-			   overlaps with the existing selection in this track.
+			/* click in a track that has no regions selected, so extend vertically
+			   to pick out all regions that are defined by the existing selection
+			   plus this one.
 			*/
-			break;
+			
+			
+			first_frame = entered_regionview->region()->position();
+			last_frame = entered_regionview->region()->last_frame();
+			
+			for (RegionSelection::iterator i = selection->regions.begin(); i != selection->regions.end(); ++i) {
+				if ((*i)->region()->position() < first_frame) {
+					first_frame = (*i)->region()->position();
+				}
+				if ((*i)->region()->last_frame() + 1 > last_frame) {
+					last_frame = (*i)->region()->last_frame();
+				}
+			}
 		}
 
-		/* 2. find all selectable objects (regionviews in this case) between that one and the end of the
-		      one that was clicked.
-		*/
+		/* 2. find all the tracks we should select in */
 
 		set<AudioTimeAxisView*> relevant_tracks;
-		
+		set<AudioTimeAxisView*> already_in_selection;
+
 		get_relevant_audio_tracks (relevant_tracks);
 
+		if (relevant_tracks.empty()) {
+
+			/* no relevant tracks -> no tracks selected .. thus .. if
+			   the regionview we're in isn't selected (i.e. we're
+			   about to extend to it), then find all tracks between
+			   the this one and any selected ones.
+			*/
+
+			if (!selection->selected (entered_regionview)) {
+
+				AudioTimeAxisView* atv = dynamic_cast<AudioTimeAxisView*> (&entered_regionview->get_time_axis_view());
+
+				if (atv) {
+
+					/* add this track to the ones we will search */
+
+					relevant_tracks.insert (atv);
+
+					/* find the track closest to this one that
+					   already a selected region.
+					*/
+
+					AudioTimeAxisView* closest = 0;
+					int distance = INT_MAX;
+					int key = atv->route()->order_key ("editor");
+
+					for (RegionSelection::iterator x = selection->regions.begin(); x != selection->regions.end(); ++x) {
+
+						AudioTimeAxisView* aatv = dynamic_cast<AudioTimeAxisView*>(&(*x)->get_time_axis_view());
+
+						if (aatv && aatv != atv) {
+
+							pair<set<AudioTimeAxisView*>::iterator,bool> result;
+
+							result = already_in_selection.insert (aatv);
+
+							if (result.second) {
+								/* newly added to already_in_selection */
+							
+
+								int d = aatv->route()->order_key ("editor");
+								
+								d -= key;
+								
+								if (abs (d) < distance) {
+									distance = abs (d);
+									closest = aatv;
+								}
+							}
+						}
+					}
+					
+					if (closest) {
+
+						/* now add all tracks between that one and this one */
+						
+						int okey = closest->route()->order_key ("editor");
+						
+						if (okey > key) {
+							swap (okey, key);
+						}
+						
+						for (TrackViewList::iterator x = track_views.begin(); x != track_views.end(); ++x) {
+							AudioTimeAxisView* aatv = dynamic_cast<AudioTimeAxisView*>(*x);
+							if (aatv && aatv != atv) {
+
+								int k = aatv->route()->order_key ("editor");
+
+								if (k >= okey && k <= key) {
+
+									/* in range but don't add it if
+									   it already has tracks selected.
+									   this avoids odd selection
+									   behaviour that feels wrong.
+									*/
+
+									if (find (already_in_selection.begin(),
+										  already_in_selection.end(),
+										  aatv) == already_in_selection.end()) {
+
+										relevant_tracks.insert (aatv);
+									}
+								}
+							}
+						}
+					}
+				}
+			}
+		}
+
+		/* 3. find all selectable objects (regionviews in this case) between that one and the end of the
+			   one that was clicked.
+		*/
+
 		for (set<AudioTimeAxisView*>::iterator t = relevant_tracks.begin(); t != relevant_tracks.end(); ++t) {
 			(*t)->get_selectables (first_frame, last_frame, -1.0, -1.0, results);
 		}
 		
-		/* 3. convert to a vector of audio regions */
+		/* 4. convert to a vector of audio regions */
 
 		vector<RegionView*> regions;
 		
@@ -527,42 +680,18 @@ Editor::set_selected_regionview_from_click (bool press, Selection::Operation op,
 	return commit;
 }
 
+
 void
 Editor::set_selected_regionview_from_region_list (boost::shared_ptr<Region> region, Selection::Operation op)
 {
 	vector<RegionView*> all_equivalent_regions;
 
-	for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
-		
-		RouteTimeAxisView* tatv;
-		
-		if ((tatv = dynamic_cast<RouteTimeAxisView*> (*i)) != 0) {
-			
-			boost::shared_ptr<Playlist> pl;
-			vector<boost::shared_ptr<Region> > results;
-			RegionView* marv;
-			boost::shared_ptr<Diskstream> ds;
-			
-			if ((ds = tatv->get_diskstream()) == 0) {
-				/* bus */
-				continue;
-			}
-			
-			if ((pl = (ds->playlist())) != 0) {
-				pl->get_region_list_equivalent_regions (region, results);
-			}
-			
-			for (vector<boost::shared_ptr<Region> >::iterator ir = results.begin(); ir != results.end(); ++ir) {
-				if ((marv = tatv->view()->find_view (*ir)) != 0) {
-					all_equivalent_regions.push_back (marv);
-				}
-			}
-			
-		}
+	get_regions_corresponding_to (region, all_equivalent_regions);
+
+	if (all_equivalent_regions.empty()) {
+		return;
 	}
-	
-	begin_reversible_command (_("set selected regions"));
-	
+
 	switch (op) {
 	case Selection::Toggle:
 		/* XXX this is not correct */
@@ -578,8 +707,6 @@ Editor::set_selected_regionview_from_region_list (boost::shared_ptr<Region> regi
 		selection->add (all_equivalent_regions);
 		break;
 	}
-
-	commit_reversible_command () ;
 }
 
 bool
@@ -631,16 +758,10 @@ Editor::track_selection_changed ()
 	}
 
 	for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
-		(*i)->set_selected (false);
-		if (mouse_mode == MouseRange) {
-			(*i)->hide_selection ();
-		}
-	}
-
-	for (TrackSelection::iterator i = selection->tracks.begin(); i != selection->tracks.end(); ++i) {
-		(*i)->set_selected (true);
-		if (mouse_mode == MouseRange) {
-			(*i)->show_selection (selection->time);
+		if (find (selection->tracks.begin(), selection->tracks.end(), *i) != selection->tracks.end()) {
+			(*i)->set_selected (true);
+		} else {
+			(*i)->set_selected (false);
 		}
 	}
 }
@@ -667,6 +788,7 @@ Editor::time_selection_changed ()
 	} else {
 		ActionManager::set_sensitive (ActionManager::time_selection_sensitive_actions, true);
 	}
+
 }
 
 void
@@ -675,6 +797,8 @@ Editor::region_selection_changed ()
 	for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
 		(*i)->set_selected_regionviews (selection->regions);
 	}
+	
+	zoomed_to_region = false;
 }
 
 void
@@ -716,7 +840,7 @@ void
 Editor::select_all (Selection::Operation op)
 {
 	list<Selectable *> touched;
-	
+
 	for (TrackViewList::iterator iter = track_views.begin(); iter != track_views.end(); ++iter) {
 		if ((*iter)->hidden()) {
 			continue;
@@ -899,7 +1023,15 @@ Editor::select_all_selectables_using_time_selection ()
 		return;
 	}
 
-	for (TrackViewList::iterator iter = selection->tracks.begin(); iter != selection->tracks.end(); ++iter) {
+	TrackSelection* ts;
+
+	if (selection->tracks.empty()) {
+		ts = &track_views;
+	} else {
+		ts = &selection->tracks;
+	}
+
+	for (TrackViewList::iterator iter = ts->begin(); iter != ts->end(); ++iter) {
 		if ((*iter)->hidden()) {
 			continue;
 		}
@@ -922,7 +1054,16 @@ Editor::select_all_selectables_using_punch()
 		return;
 	}
 
-	for (TrackViewList::iterator iter = track_views.begin(); iter != track_views.end(); ++iter) {
+
+	TrackSelection* ts;
+
+	if (selection->tracks.empty()) {
+		ts = &track_views;
+	} else {
+		ts = &selection->tracks;
+	}
+
+	for (TrackViewList::iterator iter = ts->begin(); iter != ts->end(); ++iter) {
 		if ((*iter)->hidden()) {
 			continue;
 		}
@@ -944,7 +1085,16 @@ Editor::select_all_selectables_using_loop()
 		return;
 	}
 
-	for (TrackViewList::iterator iter = track_views.begin(); iter != track_views.end(); ++iter) {
+
+	TrackSelection* ts;
+
+	if (selection->tracks.empty()) {
+		ts = &track_views;
+	} else {
+		ts = &selection->tracks;
+	}
+
+	for (TrackViewList::iterator iter = ts->begin(); iter != ts->end(); ++iter) {
 		if ((*iter)->hidden()) {
 			continue;
 		}
@@ -977,7 +1127,16 @@ Editor::select_all_selectables_using_cursor (Cursor *cursor, bool after)
 		}
 	}
 
-	for (TrackViewList::iterator iter = track_views.begin(); iter != track_views.end(); ++iter) {
+
+	TrackSelection* ts;
+
+	if (selection->tracks.empty()) {
+		ts = &track_views;
+	} else {
+		ts = &selection->tracks;
+	}
+
+	for (TrackViewList::iterator iter = ts->begin(); iter != ts->end(); ++iter) {
 		if ((*iter)->hidden()) {
 			continue;
 		}
@@ -988,28 +1147,36 @@ Editor::select_all_selectables_using_cursor (Cursor *cursor, bool after)
 }
 
 void
-Editor::select_all_selectables_between_cursors (Cursor *cursor, Cursor *other_cursor)
+Editor::select_all_selectables_using_edit (bool after)
 {
         nframes_t start;
 	nframes_t end;
 	list<Selectable *> touched;
-	bool  other_cursor_is_first = cursor->current_frame > other_cursor->current_frame;
 
-	if (cursor->current_frame == other_cursor->current_frame) {
-		return;
-	}
-
-	begin_reversible_command (_("select all between cursors"));
-	if (other_cursor_is_first) {
-		start = other_cursor->current_frame;
-		end = cursor->current_frame - 1;
-		
+	if (after) {
+		begin_reversible_command (_("select all after edit"));
+		start = get_preferred_edit_position();
+		end = session->current_end_frame();
 	} else {
-		start = cursor->current_frame;
-		end = other_cursor->current_frame - 1;
+		if ((end = get_preferred_edit_position()) > 1) {
+			begin_reversible_command (_("select all before edit"));
+			start = 0;
+			end -= 1;
+		} else {
+			return;
+		}
 	}
-	
-	for (TrackViewList::iterator iter = track_views.begin(); iter != track_views.end(); ++iter) {
+
+
+	TrackSelection* ts;
+
+	if (selection->tracks.empty()) {
+		ts = &track_views;
+	} else {
+		ts = &selection->tracks;
+	}
+
+	for (TrackViewList::iterator iter = ts->begin(); iter != ts->end(); ++iter) {
 		if ((*iter)->hidden()) {
 			continue;
 		}
@@ -1019,3 +1186,167 @@ Editor::select_all_selectables_between_cursors (Cursor *cursor, Cursor *other_cu
 	commit_reversible_command ();
 }
 
+void
+Editor::select_all_selectables_between (bool within)
+{
+        nframes64_t start;
+	nframes64_t end;
+	list<Selectable *> touched;
+
+	if (!get_edit_op_range (start, end)) {
+		return;
+	}
+
+	TrackSelection* ts;
+
+	if (selection->tracks.empty()) {
+		ts = &track_views;
+	} else {
+		ts = &selection->tracks;
+	}
+
+	for (TrackViewList::iterator iter = ts->begin(); iter != ts->end(); ++iter) {
+		if ((*iter)->hidden()) {
+			continue;
+		}
+		(*iter)->get_selectables (start, end, 0, DBL_MAX, touched);
+	}
+
+	selection->set (touched);
+}
+
+void
+Editor::select_range_between ()
+{
+        nframes64_t start;
+	nframes64_t end;
+	
+	if (!get_edit_op_range (start, end)) {
+		return;
+	}
+
+	set_mouse_mode (MouseRange);
+	selection->set ((TimeAxisView*) 0, start, end);
+}
+
+bool
+Editor::get_edit_op_range (nframes64_t& start, nframes64_t& end) const
+{
+	nframes64_t m;
+	bool ignored;
+
+	/* in range mode, use any existing selection */
+
+	if (mouse_mode == MouseRange && !selection->time.empty()) {
+		/* we know that these are ordered */
+		start = selection->time.start();
+		end = selection->time.end_frame();
+		return true;
+	}
+
+	if (!mouse_frame (m, ignored)) {
+		/* mouse is not in a canvas, try playhead+selected marker.
+		   this is probably most true when using menus.
+		 */
+
+		if (selection->markers.empty()) {
+			return false;
+		}
+
+		start = selection->markers.front()->position();
+		end = session->audible_frame();
+
+	} else {
+
+		switch (_edit_point) {
+		case EditAtPlayhead:
+			if (selection->markers.empty()) {
+				/* use mouse + playhead */
+				start = m;
+				end = session->audible_frame();
+			} else {
+				/* use playhead + selected marker */
+				start = session->audible_frame();
+				end = selection->markers.front()->position();
+			}
+			break;
+			
+		case EditAtMouse:
+			/* use mouse + selected marker */
+			if (selection->markers.empty()) {
+				start = m;
+				end = session->audible_frame();
+			} else {
+				start = selection->markers.front()->position();
+				end = m;
+			}
+			break;
+			
+		case EditAtSelectedMarker:
+			/* use mouse + selected marker */
+			if (selection->markers.empty()) {
+				
+				MessageDialog win (_("No edit range defined"),
+						   false,
+						   MESSAGE_INFO,
+						   BUTTONS_OK);
+
+				win.set_secondary_text (
+					_("the edit point is Selected Marker\nbut there is no selected marker."));
+				
+
+				win.set_default_response (RESPONSE_CLOSE);
+				win.set_position (Gtk::WIN_POS_MOUSE);
+				win.show_all();
+				
+				win.run ();
+				
+				return false; // NO RANGE
+			}
+			start = selection->markers.front()->position();
+			end = m;
+			break;
+		}
+	}
+
+	if (start == end) {
+		return false;
+	}
+
+	if (start > end) {
+		swap (start, end);
+	}
+
+	return true;
+}
+
+void
+Editor::deselect_all ()
+{
+	selection->clear ();
+}
+
+Editor::ExclusiveRegionSelection::ExclusiveRegionSelection (Editor& ed, RegionView* rv)
+	: editor (ed),
+	  regionview (rv)
+{
+
+	if (!rv || ed.current_mouse_mode() != Editing::MouseObject) {
+		return;
+	}
+	
+	if (ed.get_selection().regions.empty() && !ed.get_selection().selected (rv)) {
+		ed.get_selection().set (rv, false);
+		remove = true;
+	} else {
+		remove = false;
+	}
+}
+
+Editor::ExclusiveRegionSelection::~ExclusiveRegionSelection ()
+{
+	if (remove) {
+		editor.get_selection().remove (regionview);
+	}
+}
+
