@@ -21,6 +21,7 @@
 
 #include <sigc++/bind.h>
 #include <gtkmm/settings.h>
+#include <glibmm/ustring.h>
 
 #include <pbd/error.h>
 #include <pbd/textreceiver.h>
@@ -46,7 +47,7 @@
 #include "i18n.h"
 
 using namespace Gtk;
-using namespace GTK_ARDOUR;
+using namespace ARDOUR_COMMAND_LINE;
 using namespace ARDOUR;
 using namespace PBD;
 using namespace sigc;
@@ -56,6 +57,7 @@ TextReceiver text_receiver ("ardour");
 extern int curvetest (string);
 
 static ARDOUR_UI  *ui = 0;
+static char* localedir = LOCALEDIR;
 
 gint
 show_ui_callback (void *arg)
@@ -67,93 +69,179 @@ show_ui_callback (void *arg)
 	return FALSE;
 }
 
+#ifdef __APPLE__
+
+#include <mach-o/dyld.h>
+#include <sys/param.h>
+#include <fstream>
+
 void
-gui_jack_error ()
+fixup_bundle_environment ()
 {
-	MessageDialog win (_("Ardour could not connect to JACK."),
-		     false,
-		     Gtk::MESSAGE_INFO,
-		     (Gtk::ButtonsType)(Gtk::BUTTONS_NONE));
-win.set_secondary_text(_("There are several possible reasons:\n\
-\n\
-1) JACK is not running.\n\
-2) JACK is running as another user, perhaps root.\n\
-3) There is already another client called \"ardour\".\n\
-\n\
-Please consider the possibilities, and perhaps (re)start JACK."));
-
-	win.add_button (Stock::QUIT, RESPONSE_CLOSE);
-	win.set_default_response (RESPONSE_CLOSE);
-	
-	win.show_all ();
-	win.set_position (Gtk::WIN_POS_CENTER);
-
-	if (!no_splash) {
-		ui->hide_splash ();
+	if (!getenv ("ARDOUR_BUNDLED")) {
+		return;
 	}
 
-	/* we just don't care about the result, but we want to block */
+	char execpath[MAXPATHLEN+1];
+	uint32_t pathsz = sizeof (execpath);
 
-	win.run ();
+	_NSGetExecutablePath (execpath, &pathsz);
+
+	Glib::ustring exec_path (execpath);
+	Glib::ustring dir_path = Glib::path_get_dirname (exec_path);
+	Glib::ustring path;
+	const char *cstr = getenv ("PATH");
+
+	/* ensure that we find any bundled executables (e.g. JACK) */
+
+	path = dir_path;
+	if (cstr) {
+		path += ':';
+		path += cstr;
+	}
+	setenv ("PATH", path.c_str(), 1);
+
+	path = dir_path;
+	path += "/../Resources";
+	path += dir_path;
+	path += "/../Resources/Surfaces";
+	path += dir_path;
+	path += "/../Resources/Panners";
+
+	setenv ("ARDOUR_MODULE_PATH", path.c_str(), 1);
+
+	path = dir_path;
+	path += "/../Resources/icons:";
+	path += dir_path;
+	path += "/../Resources/pixmaps:";
+	path += dir_path;
+	path += "/../Resources/share:";
+	path += dir_path;
+	path += "/../Resources";
+
+	setenv ("ARDOUR_PATH", path.c_str(), 1);
+	setenv ("ARDOUR_CONFIG_PATH", path.c_str(), 1);
+	setenv ("ARDOUR_DATA_PATH", path.c_str(), 1);
+
+	cstr = getenv ("LADSPA_PATH");
+	if (cstr) {
+		path = cstr;
+		path += ':';
+	}
+	path = dir_path;
+	path += "/../Plugins";
+	
+	setenv ("LADSPA_PATH", path.c_str(), 1);
+
+	path = dir_path;
+	path += "/../Frameworks/clearlooks";
+
+	setenv ("GTK_PATH", path.c_str(), 1);
+
+	path = dir_path;
+	path += "/../Resources/locale";
+	
+	localedir = strdup (path.c_str());
+
+	/* write a pango.rc file and tell pango to use it */
+
+	path = dir_path;
+	path += "/../Resources/pango.rc";
+
+	std::ofstream pangorc (path.c_str());
+	if (!pangorc) {
+		error << string_compose (_("cannot open pango.rc file %1") , path) << endmsg;
+	} else {
+		pangorc << "[Pango]\nModuleFiles=";
+		Glib::ustring mpath = dir_path;
+		mpath += "/../Resources/pango.modules";
+		pangorc << mpath << endl;
+		
+		pangorc.close ();
+		setenv ("PANGO_RC_FILE", path.c_str(), 1);
+	}
+
+	// gettext charset aliases
+
+	setenv ("CHARSETALIASDIR", path.c_str(), 1);
+
+	// font config
+	
+	path = dir_path;
+	path += "/../Resources/fonts.conf";
+
+	setenv ("FONTCONFIG_FILE", path.c_str(), 1);
+
+	// GDK Pixbuf loader module file
+
+	path = dir_path;
+	path += "/../Resources/gdk-pixbuf.loaders";
+
+	setenv ("GDK_PIXBUF_MODULE_FILE", path.c_str(), 1);
+
+	if (getenv ("ARDOUR_WITH_JACK")) {
+		// JACK driver dir
+		
+		path = dir_path;
+		path += "/../Frameworks";
+		
+		setenv ("JACK_DRIVER_DIR", path.c_str(), 1);
+	}
 }
 
-static bool
-maybe_load_session ()
+#endif
+
+static void
+setup_keybindings (ARDOUR_UI* ui)
 {
-	/* If no session name is given: we're not loading a session yet, nor creating a new one */
-	if (!session_name.length()) {
-		ui->hide_splash ();
-		if (!Config->get_no_new_session_dialog()) {
-			if (!ui->new_session ()) {
-				return false;
-			}
-		}
+	Glib::ustring path;
 
-		return true;
+	if (keybindings_path.empty()) {
+		keybindings_path = "ardour";
 	}
 
-	/* Load session or start the new session dialog */
-	string name, path;
+	std::string kbpath;
+	
+	if (keybindings_path.find (".bindings") == string::npos) {
 
-	bool isnew;
-
-	if (Session::find_session (session_name, path, name, isnew)) {
-		error << string_compose(_("could not load command line session \"%1\""), session_name) << endmsg;
-		return false;
-	}
-
-	if (!new_session) {
-			
-		/* Loading a session, but the session doesn't exist */
-		if (isnew) {
-			error << string_compose (_("\n\nNo session named \"%1\" exists.\n"
-						   "To create it from the command line, start ardour as \"ardour --new %1"), path) 
-			      << endmsg;
-			return false;
+		// just a style name - allow user to
+		// specify the layout type. 
+		
+		char* layout;
+		
+		if ((layout = getenv ("ARDOUR_KEYBOARD_LAYOUT")) != 0) {
+			keybindings_path += '-';
+			keybindings_path += layout;
 		}
 
-		if (ui->load_session (path, name)) {
-			/* it failed */
-			return false;
+		keybindings_path += ".bindings";
+	} 
+
+	
+	// XXX timbyr - we need a portable test for "is-absolute" here 
+	
+	if (keybindings_path[0] != '/' && keybindings_path[0] != '.') {
+
+		/* not absolute - look in the usual places */
+		
+		path = find_config_file (keybindings_path);
+
+		if (path.empty()) {
+			warning << string_compose (_("Key bindings file \"%1\" not found. Default bindings used instead"), 
+						   keybindings_path)
+				<< endmsg;
 		}
 
 	} else {
 
-		/*  TODO: This bit of code doesn't work properly yet
-		    Glib::signal_idle().connect (bind (mem_fun (*ui, &ARDOUR_UI::cmdline_new_session), path));
-		    ui->set_will_create_new_session_automatically (true); 
-		*/
-		
-		/* Show the NSD */
-		ui->hide_splash ();
-		if (!Config->get_no_new_session_dialog()) {
-			if (!ui->new_session ()) {
-				return false;
-			}
-		}
+		// absolute path from user - use it as is
+
+		path = keybindings_path;
 	}
 
-	return true;
+	if (!path.empty()) {
+		ui->set_keybindings_path (path);
+	}
 }
 
 #ifdef VST_SUPPORT
@@ -164,17 +252,19 @@ maybe_load_session ()
 extern "C" {
 int ardour_main (int argc, char *argv[])
 #else
-int main (int argc, char *argv[])
+int main (int argc, char* argv[])
 #endif
-
 {
-	ARDOUR::AudioEngine *engine;
 	vector<Glib::ustring> null_file_list;
 
+#ifdef __APPLE__
+	fixup_bundle_environment ();
+#endif
+
         Glib::thread_init();
 	gtk_set_locale ();
 
-	(void) bindtextdomain (PACKAGE, LOCALEDIR);
+	(void) bindtextdomain (PACKAGE, localedir);
 	/* our i18n translations are all in UTF-8, so make sure
 	   that even if the user locale doesn't specify UTF-8,
 	   we use that when handling them.
@@ -235,10 +325,6 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	if (!keybindings_path.empty()) {
-		ui->set_keybindings_path (keybindings_path);
-	}
-
 	if (!no_splash) {
 		ui->show_splash ();
 		if (session_name.length()) {  
@@ -246,45 +332,13 @@ int main (int argc, char *argv[])
 		}
 	}
 
-    	try {
-		ARDOUR::init (use_vst, try_hw_optimization);
-		setup_gtk_ardour_enums ();
-		Config->set_current_owner (ConfigVariableBase::Interface);
-		ui->setup_profile ();
+	setup_keybindings (ui);
 
-		try { 
-			engine = new ARDOUR::AudioEngine (jack_client_name);
-		} catch (AudioEngine::NoBackendAvailable& err) {
-			gui_jack_error ();
-			error << string_compose (_("Could not connect to JACK server as  \"%1\""), jack_client_name) <<  endmsg;
-			return -1;
-		}
-		
-		ui->set_engine (*engine);
+	ui->run (text_receiver);
+	ui = 0;
 
-	} catch (failed_constructor& err) {
-		error << _("could not initialize Ardour.") << endmsg;
-		return -1;
-	} 
-
-	ui->start_engine ();
-
-	if (maybe_load_session ()) {
-		ui->run (text_receiver);
-		ui = 0;
-	}
-
-	delete engine;
 	ARDOUR::cleanup ();
-
-	if (ui) {
-		ui->kill();
-	}
-		
 	pthread_cancel_all ();
-
-	exit (0);
-
 	return 0;
 }
 #ifdef VST_SUPPORT
