@@ -23,6 +23,7 @@
 #include <gtkmm/accelmap.h>
 
 #include <pbd/convert.h>
+#include <pbd/stacktrace.h>
 #include <glibmm/thread.h>
 
 #include <gtkmm2ext/gtk_ui.h>
@@ -58,15 +59,21 @@ using namespace std;
 
 using PBD::atoi;
 
+
 Mixer_UI::Mixer_UI ()
 	: Window (Gtk::WINDOW_TOPLEVEL)
 {
-	_strip_width = Wide;
+	session = 0;
+	Config->get_default_narrow_ms() ? _strip_width = Narrow : _strip_width = Wide;
 	track_menu = 0;
 	mix_group_context_menu = 0;
 	no_track_list_redisplay = false;
 	in_group_row_change = false;
 	_visible = false;
+	ignore_route_reorder = false;
+	ignore_sync = false;
+
+	Route::SyncOrderKeys.connect (mem_fun (*this, &Mixer_UI::sync_order_keys));
 
  	scroller_base.add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK);
  	scroller_base.set_name ("MixerWindow");
@@ -92,6 +99,7 @@ Mixer_UI::Mixer_UI ()
 
 	track_model->signal_row_deleted().connect (mem_fun (*this, &Mixer_UI::track_list_delete));
 	track_model->signal_row_changed().connect (mem_fun (*this, &Mixer_UI::track_list_change));
+	track_model->signal_rows_reordered().connect (mem_fun (*this, &Mixer_UI::track_list_reorder));
 
 	CellRendererToggle* track_list_visible_cell = dynamic_cast<CellRendererToggle*>(track_display.get_column_cell_renderer (1));
 	track_list_visible_cell->property_activatable() = true;
@@ -236,6 +244,8 @@ Mixer_UI::show_window ()
 {
 	show_all ();
 
+	set_window_pos_and_size ();
+
 	/* now reset each strips width so the right widgets are shown */
 	MixerStrip* ms;
 
@@ -252,6 +262,8 @@ Mixer_UI::show_window ()
 bool
 Mixer_UI::hide_window (GdkEventAny *ev)
 {
+	get_window_pos_and_size ();
+
 	_visible = false;
 	return just_hide_it(ev, static_cast<Gtk::Window *>(this));
 }
@@ -273,7 +285,9 @@ Mixer_UI::add_strip (Session::RouteList& routes)
 		
 		strip = new MixerStrip (*this, *session, route);
 		strips.push_back (strip);
-		
+
+		Config->get_default_narrow_ms() ? _strip_width = Narrow : _strip_width = Wide;
+
 		if (strip->width_owner() != strip) {
 			strip->set_width (_strip_width, this);
 		}
@@ -288,11 +302,14 @@ Mixer_UI::add_strip (Session::RouteList& routes)
 		row[track_columns.visible] = strip->marked_for_display();
 		row[track_columns.route] = route;
 		row[track_columns.strip] = strip;
-		
+
+		strip->set_old_order_key (track_model->children().size() - 1);
+
 		no_track_list_redisplay = false;
 		redisplay_track_list ();
 		
 		route->name_changed.connect (bind (mem_fun(*this, &Mixer_UI::strip_name_changed), strip));
+
 		strip->GoingAway.connect (bind (mem_fun(*this, &Mixer_UI::remove_strip), strip));
 		
 		strip->signal_button_release_event().connect (bind (mem_fun(*this, &Mixer_UI::strip_button_release_event), strip));
@@ -312,6 +329,14 @@ Mixer_UI::remove_strip (MixerStrip* strip)
 		strips.erase (i);
 	}
 
+	/* Decrement old order keys for strips `above' the one that is being removed */
+	for (ri = rows.begin(); ri != rows.end(); ++ri) {
+		MixerStrip* s = (*ri)[track_columns.strip];
+		if (s->old_order_key() > strip->old_order_key()) {
+			s->set_old_order_key (s->old_order_key() - 1);
+		}
+	}
+
 	for (ri = rows.begin(); ri != rows.end(); ++ri) {
 		if ((*ri)[track_columns.strip] == strip) {
 			track_model->erase (ri);
@@ -320,6 +345,44 @@ Mixer_UI::remove_strip (MixerStrip* strip)
 	}
 }
 
+const char*
+Mixer_UI::get_order_key() 
+{
+	if (Config->get_sync_all_route_ordering()) {
+		return X_("editor");
+	} else {
+		return X_("signal");
+	}
+}
+
+
+void
+Mixer_UI::sync_order_keys ()
+{
+	vector<int> neworder;
+	TreeModel::Children rows = track_model->children();
+	TreeModel::Children::iterator ri;
+
+	if (ignore_sync || !session || (session->state_of_the_state() & Session::Loading) || rows.empty()) {
+		return;
+	}
+
+	for (ri = rows.begin(); ri != rows.end(); ++ri) {
+		neworder.push_back (0);
+	}
+
+	for (ri = rows.begin(); ri != rows.end(); ++ri) {
+		boost::shared_ptr<Route> route = (*ri)[track_columns.route];
+		MixerStrip* strip = (*ri)[track_columns.strip];
+		neworder[route->order_key (get_order_key())] = strip->old_order_key ();
+	}
+
+	ignore_route_reorder = true;
+	track_model->reorder (neworder);
+	ignore_route_reorder = false;
+}
+
+
 void
 Mixer_UI::follow_strip_selection ()
 {
@@ -341,7 +404,7 @@ Mixer_UI::strip_button_release_event (GdkEventButton *ev, MixerStrip *strip)
 		if (_selection.selected (strip->route())) {
 			_selection.remove (strip->route());
 		} else {
-			if (Keyboard::modifier_state_equals (ev->state, Keyboard::Shift)) {
+			if (Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier)) {
 				_selection.add (strip->route());
 			} else {
 				_selection.set (strip->route());
@@ -355,7 +418,6 @@ Mixer_UI::strip_button_release_event (GdkEventButton *ev, MixerStrip *strip)
 void
 Mixer_UI::connect_to_session (Session* sess)
 {
-
 	session = sess;
 
 	XMLNode* node = ARDOUR_UI::instance()->mixer_settings();
@@ -505,7 +567,7 @@ Mixer_UI::set_all_audio_visibility (int tracks, bool yn)
 			continue;
 		}
 
-		AudioTrack* at = strip->audio_track();
+		boost::shared_ptr<AudioTrack> at = strip->audio_track();
 
 		switch (tracks) {
 		case 0:
@@ -564,6 +626,13 @@ Mixer_UI::hide_all_audiotracks ()
 	set_all_audio_visibility (1, false);
 }
 
+void
+Mixer_UI::track_list_reorder (const TreeModel::Path& path, const TreeModel::iterator& iter, int* new_order)
+{
+	session->set_remote_control_ids();
+	redisplay_track_list ();
+}
+
 void
 Mixer_UI::track_list_change (const Gtk::TreeModel::Path& path,const Gtk::TreeModel::iterator& iter)
 {
@@ -598,10 +667,17 @@ Mixer_UI::redisplay_track_list ()
 		}
 
 		bool visible = (*i)[track_columns.visible];
+		
+		boost::shared_ptr<Route> route = (*i)[track_columns.route];
 
 		if (visible) {
 			strip->set_marked_for_display (true);
-			strip->route()->set_order_key (N_("signal"), order);
+
+			if (!ignore_route_reorder) {
+				strip->route()->set_order_key (get_order_key(), order);
+			} 
+
+			strip->set_old_order_key (order);
 
 			if (strip->packed()) {
 
@@ -634,12 +710,18 @@ Mixer_UI::redisplay_track_list ()
 			}
 		}
 	}
+	
+	if (Config->get_sync_all_route_ordering() && !ignore_route_reorder) {
+		ignore_sync = true;
+		Route::SyncOrderKeys (); // EMIT SIGNAL
+		ignore_sync = false;
+	}
 }
 
 struct SignalOrderRouteSorter {
     bool operator() (boost::shared_ptr<Route> a, boost::shared_ptr<Route> b) {
 	    /* use of ">" forces the correct sort order */
-	    return a->order_key ("signal") < b->order_key ("signal");
+	    return a->order_key (Mixer_UI::get_order_key()) < b->order_key (Mixer_UI::get_order_key());
     }
 };
 
@@ -1047,37 +1129,42 @@ Mixer_UI::set_strip_width (Width w)
 	}
 }
 
+void
+Mixer_UI::set_window_pos_and_size ()
+{
+	resize (m_width, m_height);
+	move (m_root_x, m_root_y);
+}
+
+	void
+Mixer_UI::get_window_pos_and_size ()
+{
+	get_position(m_root_x, m_root_y);
+	get_size(m_width, m_height);
+}
 
 int
 Mixer_UI::set_state (const XMLNode& node)
 {
 	const XMLProperty* prop;
 	XMLNode* geometry;
-	Gdk::Geometry g;
-	int x, y, xoff, yoff;
 	
 	if ((geometry = find_named_node (node, "geometry")) == 0) {
 
-		g.base_width = default_width;
-		g.base_height = default_height;
-		x = 1;
-		y = 1;
-		xoff = 0;
-		yoff = 21;
+		m_width = default_width;
+		m_height = default_height;
+		m_root_x = 1;
+		m_root_y = 1;
 
 	} else {
 
-		g.base_width = atoi(geometry->property("x_size")->value().c_str());
-		g.base_height = atoi(geometry->property("y_size")->value().c_str());
-		x = atoi(geometry->property("x_pos")->value().c_str());
-		y = atoi(geometry->property("y_pos")->value().c_str());
-		xoff = atoi(geometry->property("x_off")->value().c_str());
-		yoff = atoi(geometry->property("y_off")->value().c_str());
+		m_width = atoi(geometry->property("x_size")->value().c_str());
+		m_height = atoi(geometry->property("y_size")->value().c_str());
+		m_root_x = atoi(geometry->property("x_pos")->value().c_str());
+		m_root_y = atoi(geometry->property("y_pos")->value().c_str());
 	}
 
-	set_geometry_hints (global_vpacker, g, Gdk::HINT_BASE_SIZE);
-	set_default_size(g.base_width, g.base_height);
-	move (x, y);
+	set_window_pos_and_size ();
 
 	if ((prop = node.property ("narrow-strips"))) {
 		if (prop->value() == "yes") {
@@ -1103,25 +1190,24 @@ Mixer_UI::get_state (void)
 
 	if (is_realized()) {
 		Glib::RefPtr<Gdk::Window> win = get_window();
-		
-		int x, y, xoff, yoff, width, height;
-		win->get_root_origin(x, y);
-		win->get_position(xoff, yoff);
-		win->get_size(width, height);
+	
+		get_window_pos_and_size ();
 
 		XMLNode* geometry = new XMLNode ("geometry");
 		char buf[32];
-		snprintf(buf, sizeof(buf), "%d", width);
+		snprintf(buf, sizeof(buf), "%d", m_width);
 		geometry->add_property(X_("x_size"), string(buf));
-		snprintf(buf, sizeof(buf), "%d", height);
+		snprintf(buf, sizeof(buf), "%d", m_height);
 		geometry->add_property(X_("y_size"), string(buf));
-		snprintf(buf, sizeof(buf), "%d", x);
+		snprintf(buf, sizeof(buf), "%d", m_root_x);
 		geometry->add_property(X_("x_pos"), string(buf));
-		snprintf(buf, sizeof(buf), "%d", y);
+		snprintf(buf, sizeof(buf), "%d", m_root_y);
 		geometry->add_property(X_("y_pos"), string(buf));
-		snprintf(buf, sizeof(buf), "%d", xoff);
+		
+		// written only for compatibility, they are not used.
+		snprintf(buf, sizeof(buf), "%d", 0);
 		geometry->add_property(X_("x_off"), string(buf));
-		snprintf(buf, sizeof(buf), "%d", yoff);
+		snprintf(buf, sizeof(buf), "%d", 0);
 		geometry->add_property(X_("y_off"), string(buf));
 
 		snprintf(buf,sizeof(buf), "%d",gtk_paned_get_position (static_cast<Paned*>(&rhs_pane1)->gobj()));
