@@ -20,8 +20,11 @@
 #include "i18n.h"
 #include "new_session_dialog.h"
 
+#include <pbd/error.h>
+
 #include <ardour/recent_sessions.h>
 #include <ardour/session.h>
+#include <ardour/profile.h>
 
 #include <gtkmm/entry.h>
 #include <gtkmm/filechooserbutton.h>
@@ -35,15 +38,18 @@
 #include <gtkmm2ext/window_title.h>
 
 using namespace Gtkmm2ext;
+using namespace PBD;
 
 #include "opts.h"
+#include "utils.h"
 
 NewSessionDialog::NewSessionDialog()
 	: ArdourDialog ("session control")
 {
+	in_destructor = false;
 	session_name_label = Gtk::manage(new class Gtk::Label(_("Name :")));
 	m_name = Gtk::manage(new class Gtk::Entry());
-	m_name->set_text(GTK_ARDOUR::session_name);
+	m_name->set_text(ARDOUR_COMMAND_LINE::session_name);
 
 	chan_count_label_1 = Gtk::manage(new class Gtk::Label(_("channels")));
 	chan_count_label_2 = Gtk::manage(new class Gtk::Label(_("channels")));
@@ -305,7 +311,10 @@ NewSessionDialog::NewSessionDialog()
 	new_session_table->attach(*m_folder, 1, 2, 1, 2, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0);
 	new_session_table->attach(*session_template_label, 0, 1, 2, 3, Gtk::FILL, Gtk::FILL, 0, 0);
 	new_session_table->attach(*m_template, 1, 2, 2, 3, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0);
-	new_session_table->attach(*advanced_expander, 0, 2, 3, 4, Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 6);
+
+	if (!ARDOUR::Profile->get_sae()) {
+		new_session_table->attach(*advanced_expander, 0, 2, 3, 4, Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 6);
+	}
 
 	open_session_hbox->pack_start(*open_session_file_label, false, false, 12);
 	open_session_hbox->pack_start(*m_open_filechooser, true, true, 12);
@@ -333,16 +342,38 @@ NewSessionDialog::NewSessionDialog()
 	recent_frame->set_label_widget(*recent_sesion_label);
 	open_session_vbox->pack_start(*recent_frame, Gtk::PACK_EXPAND_WIDGET, 0);
 	open_session_vbox->pack_start(*open_session_hbox, Gtk::PACK_SHRINK, 12);
+
 	m_notebook->set_flags(Gtk::CAN_FOCUS);
 	m_notebook->set_scrollable(true);
-	m_notebook->append_page(*new_session_table, _("New Session"));
-	m_notebook->pages().back().set_tab_label_packing(false, true, Gtk::PACK_START);
-	m_notebook->append_page(*open_session_vbox, _("Open Session"));
-	m_notebook->pages().back().set_tab_label_packing(false, true, Gtk::PACK_START);
+	
 	get_vbox()->set_homogeneous(false);
 	get_vbox()->set_spacing(0);
 	get_vbox()->pack_start(*m_notebook, Gtk::PACK_SHRINK, 0);
 
+	/* 
+	   icon setting is done again in the editor (for the whole app),
+	   but its all chickens and eggs at this point.
+	*/
+
+	list<Glib::RefPtr<Gdk::Pixbuf> > window_icons;
+	Glib::RefPtr<Gdk::Pixbuf> icon;
+
+	if ((icon = ::get_icon ("ardour_icon_16px")) != 0) {
+		window_icons.push_back (icon);
+	}
+	if ((icon = ::get_icon ("ardour_icon_22px")) != 0) {
+		window_icons.push_back (icon);
+	}
+	if ((icon = ::get_icon ("ardour_icon_32px")) != 0) {
+		window_icons.push_back (icon);
+	}
+	if ((icon = ::get_icon ("ardour_icon_48px")) != 0) {
+		window_icons.push_back (icon);
+	}
+	if (!window_icons.empty()) {
+		set_icon_list (window_icons);
+	}
+
 	WindowTitle title(Glib::get_application_name());
 	title += _("Session Control");
 	set_title(title.get_string());
@@ -411,7 +442,7 @@ NewSessionDialog::NewSessionDialog()
 
 
 	set_default_response (Gtk::RESPONSE_OK);
-	if (!GTK_ARDOUR::session_name.length()) {
+	if (!ARDOUR_COMMAND_LINE::session_name.length()) {
 		set_response_sensitive (Gtk::RESPONSE_OK, false);
 		set_response_sensitive (Gtk::RESPONSE_NONE, false);
 	} else {
@@ -433,13 +464,124 @@ NewSessionDialog::NewSessionDialog()
 	m_treeview->signal_row_activated().connect (mem_fun (*this, &NewSessionDialog::recent_row_activated));
 	m_open_filechooser->signal_selection_changed ().connect (mem_fun (*this, &NewSessionDialog::file_chosen));
 	m_template->signal_selection_changed ().connect (mem_fun (*this, &NewSessionDialog::template_chosen));
-	m_name->grab_focus();
+	
+	page_set = Pages (0);
+}
+
+NewSessionDialog::~NewSessionDialog()
+{
+	in_destructor = true;
+}
+
+int
+NewSessionDialog::run ()
+{
+	if (!page_set) {
+		/* nothing to display */
+		return Gtk::RESPONSE_OK;
+	}
+
+	return ArdourDialog::run ();
 }
 
 void
-NewSessionDialog::set_session_name(const Glib::ustring& name)
+NewSessionDialog::set_have_engine (bool yn)
 {
-	m_name->set_text(name);
+	if (yn) {
+		m_notebook->remove_page (engine_control);
+		page_set = Pages (page_set & ~EnginePage);
+	} else {
+		if (!(page_set & EnginePage)) {
+			m_notebook->append_page (engine_control, _("Audio Setup"));
+			m_notebook->show_all_children();
+			page_set = Pages (page_set | EnginePage);
+		}
+	}
+}
+
+void
+NewSessionDialog::set_existing_session (bool yn)
+{
+	if (yn) {
+
+		if (page_set & NewPage) {
+			m_notebook->remove_page (*new_session_table);
+			page_set = Pages (page_set & ~NewPage);
+		}
+
+		if (page_set & OpenPage) {
+			m_notebook->remove_page (*open_session_vbox);
+			page_set = Pages (page_set & ~OpenPage);
+		}
+
+	} else {
+		if (!(page_set & NewPage)) {
+			m_notebook->append_page(*new_session_table, _("New Session"));
+			m_notebook->pages().back().set_tab_label_packing(false, true, Gtk::PACK_START);
+			page_set = Pages (page_set | NewPage);
+		}
+		if (!(page_set & OpenPage)) {
+			m_notebook->append_page(*open_session_vbox, _("Open Session"));
+			m_notebook->pages().back().set_tab_label_packing(false, true, Gtk::PACK_START);
+			page_set = Pages (page_set | OpenPage);
+		}
+
+		m_notebook->show_all_children();
+	}
+}
+
+void
+NewSessionDialog::set_session_name (const Glib::ustring& name)
+{
+	m_name->set_text (name);
+}
+
+void
+NewSessionDialog::set_session_folder(const Glib::ustring& dir)
+{
+	Glib::ustring realdir = dir;
+	char* res;
+
+	/* this little tangled mess is a result of 4 things:
+
+	    1) GtkFileChooser vomits when given a non-absolute directory
+                   argument to set_current_folder()
+            2) canonicalize_file_name() doesn't exist on OS X
+	    3) linux man page for realpath() says "do not use this function"
+	    4) canonicalize_file_name() & realpath() have entirely
+                   different semantics on OS X and Linux when given
+		   a non-existent path.
+		   
+	   as result of all this, we take two distinct pathways through the code.
+	*/
+
+
+#ifdef __APPLE__
+
+	char buf[PATH_MAX];
+
+	if((res = realpath (dir.c_str(), buf)) != 0) {
+		if (!Glib::file_test (dir, Glib::FILE_TEST_IS_DIR)) {
+			realdir = Glib::path_get_dirname (realdir);
+		}
+		m_folder->set_current_folder (realdir);
+	}
+
+	
+#else 
+	if (!Glib::file_test (dir, Glib::FILE_TEST_IS_DIR)) {
+		realdir = Glib::path_get_dirname (realdir);
+		cerr << "didn't exist, use " << realdir << endl;
+	}
+
+	if ((res = canonicalize_file_name (realdir.c_str())) != 0) {
+		cerr << "canonical, use " << res << endl;
+		m_folder->set_current_folder (res);
+		free (res);
+	}
+	
+#endif
+
 }
 
 std::string
@@ -460,36 +602,53 @@ NewSessionDialog::session_name() const
 	}	  
 	*/
 
-	if (m_notebook->get_current_page() == 0) {
+	switch (which_page()) {
+	case NewPage:
+	case EnginePage:
+		/* new or audio setup pages */
 	        return Glib::filename_from_utf8(m_name->get_text());
-	} else {
-		if (m_treeview->get_selection()->count_selected_rows() == 0) {
-		        return Glib::filename_from_utf8(str);
-		}
-		Gtk::TreeModel::iterator i = m_treeview->get_selection()->get_selected();
-		return (*i)[recent_columns.visible_name];
+	default:
+		break;
+	} 
+
+	if (m_treeview->get_selection()->count_selected_rows() == 0) {
+		return Glib::filename_from_utf8(str);
 	}
+	Gtk::TreeModel::iterator i = m_treeview->get_selection()->get_selected();
+	return (*i)[recent_columns.visible_name];
 }
 
 std::string
 NewSessionDialog::session_folder() const
 {
-        if (m_notebook->get_current_page() == 0) {
-	        return Glib::filename_from_utf8(m_folder->get_current_folder());
-	} else {
-	       
-		if (m_treeview->get_selection()->count_selected_rows() == 0) {
-		        return Glib::filename_from_utf8(m_open_filechooser->get_current_folder());
+	switch (which_page()) {
+	case NewPage:
+	        return Glib::filename_from_utf8(m_folder->get_filename());
+		
+	case EnginePage:
+		if (page_set == EnginePage) {
+			/* just engine page, nothing else : use m_folder since it should be set */
+			return Glib::filename_from_utf8(m_folder->get_filename());
 		}
-		Gtk::TreeModel::iterator i = m_treeview->get_selection()->get_selected();
-		return (*i)[recent_columns.fullpath];
+		break;
+
+	default:
+		break;
 	}
+	       
+	if (m_treeview->get_selection()->count_selected_rows() == 0) {
+		const string filename(Glib::filename_from_utf8(m_open_filechooser->get_filename()));
+		return Glib::path_get_dirname(filename);
+	}
+
+	Gtk::TreeModel::iterator i = m_treeview->get_selection()->get_selected();
+	return (*i)[recent_columns.fullpath];
 }
 
 bool
 NewSessionDialog::use_session_template() const
 {
-        if(m_template->get_filename().empty() && (m_notebook->get_current_page() == 0)) return false;
+        if(m_template->get_filename().empty() && (which_page() == NewPage)) return false;
 	return true;
 }
 
@@ -577,6 +736,56 @@ NewSessionDialog::get_current_page()
 	return m_notebook->get_current_page();
 }
 
+NewSessionDialog::Pages
+NewSessionDialog::which_page () const
+{
+	int num = m_notebook->get_current_page();
+
+	if (page_set == NewPage) {
+		return NewPage;
+
+	} else if (page_set == OpenPage) {
+		return OpenPage;
+
+	} else if (page_set == EnginePage) {
+		return EnginePage;
+
+	} else if (page_set == NewPage|OpenPage) {
+		switch (num) {
+		case 0:
+			return NewPage;
+		default:
+			return OpenPage;
+		}
+
+	} else if (page_set == NewPage|EnginePage) {
+		switch (num) {
+		case 0:
+			return NewPage;
+		default:
+			return EnginePage;
+		}
+
+	} else if (page_set == NewPage|EnginePage|OpenPage) {
+		switch (num) {
+		case 0:
+			return NewPage;
+		case 1:
+			return OpenPage;
+		default:
+			return EnginePage;
+		}
+
+	} else if (page_set == OpenPage|EnginePage) {
+		switch (num) {
+		case 0:
+			return OpenPage;
+		default:
+			return EnginePage;
+		}
+	}
+}
+
 void
 NewSessionDialog::set_current_page(int page)
 {
@@ -605,25 +814,38 @@ NewSessionDialog::on_new_session_name_entry_changed ()
 void
 NewSessionDialog::notebook_page_changed (GtkNotebookPage* np, uint pagenum)
 {
-	if (pagenum == 1) {
+	if (in_destructor) {
+		return;
+	}
+
+	switch (which_page()) {
+	case OpenPage:
 		on_new_session_page = false;
 		m_okbutton->set_label(_("Open"));
+		m_okbutton->set_image (*(manage (new Gtk::Image (Gtk::Stock::OPEN, Gtk::ICON_SIZE_BUTTON))));
 		set_response_sensitive (Gtk::RESPONSE_NONE, false);
-		m_okbutton->set_image (*(new Gtk::Image (Gtk::Stock::OPEN, Gtk::ICON_SIZE_BUTTON)));
 		if (m_treeview->get_selection()->count_selected_rows() == 0) {
 			set_response_sensitive (Gtk::RESPONSE_OK, false);
 		} else {
 			set_response_sensitive (Gtk::RESPONSE_OK, true);
 		}
-	} else {
+		break;
+
+	case EnginePage:
+		on_new_session_page = false;
+		m_okbutton->set_label(_("Open"));
+		m_okbutton->set_image (*(manage (new Gtk::Image (Gtk::Stock::OPEN, Gtk::ICON_SIZE_BUTTON))));
+		set_response_sensitive (Gtk::RESPONSE_NONE, false);
+		set_response_sensitive (Gtk::RESPONSE_OK, true);
+		break;
+
+	default:
 		on_new_session_page = true;
-		if (m_name->get_text() != "") {
-			set_response_sensitive (Gtk::RESPONSE_NONE, true);
-		}
 		m_okbutton->set_label(_("New"));
 		m_okbutton->set_image (*(new Gtk::Image (Gtk::Stock::NEW, Gtk::ICON_SIZE_BUTTON)));
 		if (m_name->get_text() == "") {
 			set_response_sensitive (Gtk::RESPONSE_OK, false);
+			m_name->grab_focus();
 		} else {
 			set_response_sensitive (Gtk::RESPONSE_OK, true);
 		}
@@ -647,11 +869,21 @@ NewSessionDialog::treeview_selection_changed ()
 void
 NewSessionDialog::file_chosen ()
 {
-	if (on_new_session_page) return;
+	switch (which_page()) {
+      case OpenPage:
+         break;
+	   case NewPage:
+	   case EnginePage:
+		   return;
+	}
 
 	m_treeview->get_selection()->unselect_all();
 
-	get_window()->set_cursor(Gdk::Cursor(Gdk::WATCH));
+	Glib::RefPtr<Gdk::Window> win (get_window());
+
+	if (win) {
+		win->set_cursor(Gdk::Cursor(Gdk::WATCH));
+	}
 
 	if (!m_open_filechooser->get_filename().empty()) {
 	        set_response_sensitive (Gtk::RESPONSE_OK, true);
@@ -728,7 +960,7 @@ NewSessionDialog::monitor_bus_button_clicked ()
 void
 NewSessionDialog::reset_template()
 {
-        m_template->set_filename("");
+        m_template->unselect_all ();
 }
 
 void
