@@ -42,6 +42,7 @@
 #include "editing.h"
 #include "option_editor.h"
 #include "midi_port_dialog.h"
+#include "gui_thread.h"
 
 #include "i18n.h"
 
@@ -55,7 +56,7 @@ using namespace std;
 static vector<string> positional_sync_strings;
 
 OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
-	: Dialog ("options editor"),
+	: ArdourDialog ("options editor", false),
 	  ui (uip),
 	  editor (ed),
 	  mixer (mixui),
@@ -63,23 +64,32 @@ OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
 	  /* Paths */
 	  path_table (11, 2),
 
-	  /* Fades */
+	  /* misc */
 
 	  short_xfade_adjustment (0, 1.0, 500.0, 5.0, 100.0),
 	  short_xfade_slider (short_xfade_adjustment),
 	  destructo_xfade_adjustment (1.0, 1.0, 500.0, 1.0, 100.0),
 	  destructo_xfade_slider (destructo_xfade_adjustment),
+	  history_depth (20, -1, 100, 1.0, 10.0),
+	  saved_history_depth (20, 0, 100, 1.0, 10.0),
+	  history_depth_spinner (history_depth),
+	  saved_history_depth_spinner (saved_history_depth),
+	  limit_history_button (_("Limit undo history")),
+	  save_history_button (_("Save undo history")),
 
 	  /* Sync */
 
 	  smpte_offset_clock (X_("smpteoffset"), false, X_("SMPTEOffsetClock"), true, true),
 	  smpte_offset_negative_button (_("SMPTE offset is negative")),
+	  synced_timecode_button (_("Timecode source is sample-clock synced")),
 
 	  /* MIDI */
 
 	  midi_port_table (4, 11),
-	  mmc_device_id_adjustment (0.0, 0.0, (double) 0x7f, 1.0, 16.0),
-	  mmc_device_id_spinner (mmc_device_id_adjustment),
+	  mmc_receive_device_id_adjustment (0.0, 0.0, (double) 0x7f, 1.0, 16.0),
+	  mmc_receive_device_id_spinner (mmc_receive_device_id_adjustment),
+	  mmc_send_device_id_adjustment (0.0, 0.0, (double) 0x7f, 1.0, 16.0),
+	  mmc_send_device_id_spinner (mmc_send_device_id_adjustment),
 	  add_midi_port_button (_("Add new MIDI port")),
 
 	  /* Click */
@@ -104,13 +114,13 @@ OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
 	session = 0;
 	
 	WindowTitle title(Glib::get_application_name());
-	title += _("Options Editor");
+	title += _("Preferences");
 	set_title(title.get_string());
 
 	set_default_size (300, 300);
-	set_wmclass (X_("ardour_option_editor"), "Ardour");
+	set_wmclass (X_("ardour_preferences"), "Ardour");
 
-	set_name ("OptionsWindow");
+	set_name ("Preferences");
 	add_events (Gdk::KEY_PRESS_MASK|Gdk::KEY_RELEASE_MASK);
 	
 	VBox *vbox = get_vbox();
@@ -127,7 +137,7 @@ OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
 
 	setup_sync_options();
 	setup_path_options();
-	setup_fade_options ();
+	setup_misc_options ();
 	setup_keyboard_options ();
 	setup_auditioner_editor ();
 
@@ -136,13 +146,16 @@ OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
 	notebook.pages().push_back (TabElem (keyboard_mouse_table, _("Kbd/Mouse")));
 	notebook.pages().push_back (TabElem (click_packer, _("Click")));
 	notebook.pages().push_back (TabElem (audition_packer, _("Audition")));
-	notebook.pages().push_back (TabElem (fade_packer, _("Layers & Fades")));
+	notebook.pages().push_back (TabElem (misc_packer, _("Misc")));
 
 	setup_midi_options ();
 	notebook.pages().push_back (TabElem (midi_packer, _("MIDI")));
 
 	set_session (0);
 	show_all_children();
+
+	Config->map_parameters (mem_fun (*this, &OptionEditor::parameter_changed));
+	Config->ParameterChanged.connect (mem_fun (*this, &OptionEditor::parameter_changed));
 }
 
 void
@@ -237,7 +250,7 @@ OptionEditor::add_session_paths ()
 }
 
 void
-OptionEditor::setup_fade_options ()
+OptionEditor::setup_misc_options ()
 {
 	Gtk::HBox* hbox;
 	
@@ -249,7 +262,7 @@ OptionEditor::setup_fade_options ()
 	hbox->set_spacing (10);
 	hbox->pack_start (*label, false, false);
 	hbox->pack_start (short_xfade_slider, true, true);
-	fade_packer.pack_start (*hbox, false, false);
+	misc_packer.pack_start (*hbox, false, false);
 
 	short_xfade_adjustment.signal_value_changed().connect (mem_fun(*this, &OptionEditor::short_xfade_adjustment_changed));
 
@@ -261,16 +274,94 @@ OptionEditor::setup_fade_options ()
 	hbox->set_spacing (10);
 	hbox->pack_start (*label, false, false);
 	hbox->pack_start (destructo_xfade_slider, true, true);
-	fade_packer.pack_start (*hbox, false, false);
+	misc_packer.pack_start (*hbox, false, false);
 	
+
 	destructo_xfade_adjustment.signal_value_changed().connect (mem_fun(*this, &OptionEditor::destructo_xfade_adjustment_changed));
 
+	hbox = manage (new HBox);
+	hbox->set_border_width (5);
+	hbox->set_spacing (10);
+	hbox->pack_start (limit_history_button, false, false);
+	misc_packer.pack_start (*hbox, false, false);
+
+	label = manage (new Label (_("History depth (commands)")));
+	label->set_name ("OptionsLabel");
+
+	hbox = manage (new HBox);
+	hbox->set_border_width (5);
+	hbox->set_spacing (10);
+	hbox->pack_start (*label, false, false);
+	hbox->pack_start (history_depth_spinner, false, false);
+	misc_packer.pack_start (*hbox, false, false);
+
+	history_depth.signal_value_changed().connect (mem_fun (*this, &OptionEditor::history_depth_changed));
+	saved_history_depth.signal_value_changed().connect (mem_fun (*this, &OptionEditor::saved_history_depth_changed));
+	save_history_button.signal_toggled().connect (mem_fun (*this, &OptionEditor::save_history_toggled));
+	limit_history_button.signal_toggled().connect (mem_fun (*this, &OptionEditor::limit_history_toggled));
+
+	hbox = manage (new HBox);
+	hbox->set_border_width (5);
+	hbox->set_spacing (10);
+	hbox->pack_start (save_history_button, false, false);
+	misc_packer.pack_start (*hbox, false, false);
+
+	label = manage (new Label (_("Saved history depth (commands)")));
+	label->set_name ("OptionsLabel");
+
+	hbox = manage (new HBox);
+	hbox->set_border_width (5);
+	hbox->set_spacing (10);
+	hbox->pack_start (*label, false, false);
+	hbox->pack_start (saved_history_depth_spinner, false, false);
+	misc_packer.pack_start (*hbox, false, false);
+	
 	short_xfade_slider.set_update_policy (UPDATE_DISCONTINUOUS);
 	destructo_xfade_slider.set_update_policy (UPDATE_DISCONTINUOUS);
 
 	destructo_xfade_adjustment.set_value (Config->get_destructive_xfade_msecs());
 
-	fade_packer.show_all ();
+	misc_packer.show_all ();
+}
+
+void
+OptionEditor::limit_history_toggled ()
+{
+	bool x = limit_history_button.get_active();
+	
+	if (!x) {
+		Config->set_history_depth (0);
+		history_depth_spinner.set_sensitive (false);
+	} else {
+		if (Config->get_history_depth() == 0) {
+			/* get back to a sane default */
+			Config->set_history_depth (20);
+		}
+		history_depth_spinner.set_sensitive (true);
+	}
+}
+
+void
+OptionEditor::save_history_toggled ()
+{
+	bool x = save_history_button.get_active();
+
+	if (x != Config->get_save_history()) {
+		Config->set_save_history (x);
+		saved_history_depth_spinner.set_sensitive (x);
+	}
+}
+
+void
+OptionEditor::history_depth_changed()
+{
+	Config->set_history_depth ((int32_t) floor (history_depth.get_value()));
+}
+
+void
+OptionEditor::saved_history_depth_changed()
+{
+	Config->set_saved_history_depth ((int32_t) floor (saved_history_depth.get_value()));
 }
 
 void
@@ -324,8 +415,10 @@ OptionEditor::setup_sync_options ()
 	hbox->pack_start (smpte_offset_negative_button, false, false);
 
 	sync_packer.pack_start (*hbox, false, false);
+	sync_packer.pack_start (synced_timecode_button, false, false);
 
 	smpte_offset_negative_button.signal_clicked().connect (mem_fun(*this, &OptionEditor::smpte_offset_negative_clicked));
+	synced_timecode_button.signal_toggled().connect (mem_fun(*this, &OptionEditor::synced_timecode_toggled));
 }
 
 void
@@ -336,6 +429,17 @@ OptionEditor::smpte_offset_negative_clicked ()
 	}
 }
 
+void
+OptionEditor::synced_timecode_toggled ()
+{
+	bool x;
+
+	if ((x = synced_timecode_button.get_active()) != Config->get_timecode_source_is_synced()) {
+		Config->set_timecode_source_is_synced (x);
+		Config->save_state();
+	}
+}
+
 void
 OptionEditor::smpte_offset_chosen()
 {
@@ -350,13 +454,18 @@ void
 OptionEditor::setup_midi_options ()
 {
 	HBox* hbox;
+	Label* label;
 
 	midi_port_table.set_row_spacings (6);
 	midi_port_table.set_col_spacings (10);
 
 	redisplay_midi_ports ();
 
-	mmc_device_id_adjustment.signal_value_changed().connect (mem_fun (*this, &OptionEditor::mmc_device_id_adjusted));
+	mmc_receive_device_id_adjustment.set_value (Config->get_mmc_receive_device_id());
+	mmc_send_device_id_adjustment.set_value (Config->get_mmc_send_device_id());
+
+	mmc_receive_device_id_adjustment.signal_value_changed().connect (mem_fun (*this, &OptionEditor::mmc_receive_device_id_adjusted));
+	mmc_send_device_id_adjustment.signal_value_changed().connect (mem_fun (*this, &OptionEditor::mmc_send_device_id_adjusted));
 
 	hbox = manage (new HBox);
 	hbox->set_border_width (6);
@@ -365,6 +474,26 @@ OptionEditor::setup_midi_options ()
 	midi_packer.pack_start (*hbox, false, false);
 	midi_packer.pack_start (add_midi_port_button, false, false);
 
+	hbox = manage (new HBox);
+	hbox->set_border_width (6);
+	hbox->set_spacing (6);
+	label = (manage (new Label (_("Inbound MMC Device ID")))); 
+	hbox->pack_start (mmc_receive_device_id_spinner, false, false);
+	hbox->pack_start (*label, false, false);
+	midi_packer.pack_start (*hbox, false, false); 
+
+	mmc_receive_device_id_spinner.set_value(Config->get_mmc_receive_device_id ());
+
+	hbox = manage (new HBox);
+	hbox->set_border_width (6);
+	hbox->set_spacing (6);
+	label = (manage (new Label (_("Outbound MMC Device ID")))); 
+	hbox->pack_start (mmc_send_device_id_spinner, false, false);
+	hbox->pack_start (*label, false, false);
+	midi_packer.pack_start (*hbox, false, false);
+
+	mmc_send_device_id_spinner.set_value(Config->get_mmc_send_device_id ());
+
 	add_midi_port_button.signal_clicked().connect (mem_fun (*this, &OptionEditor::add_midi_port));
 }
 
@@ -432,13 +561,6 @@ OptionEditor::redisplay_midi_ports ()
 	midi_port_table_widgets.push_back (vsep);
 	midi_port_table.attach (*vsep, 7, 8, 0, 8);
 	
-	label = (manage (new Label (_("MMC Device ID")))); 
-	label->show ();
-	midi_port_table_widgets.push_back (label);
-	midi_port_table.attach (*label, 9, 10, 0, 1);
-	midi_port_table_widgets.push_back (&mmc_device_id_spinner);
-	midi_port_table.attach (mmc_device_id_spinner, 9, 10, 1, 2);
-
 	for (n = 0, i = ports.begin(); i != ports.end(); ++n, ++i) {
 
 		ToggleButton* tb;
@@ -590,12 +712,15 @@ OptionEditor::add_midi_port ()
 		smod = "duplex";
 	}
 
-	MIDI::PortRequest req (X_("ardour"),
-			       dialog.port_name.get_text(),
-			       smod,
-			       MIDI::PortFactory::default_port_type());
 
-	if (MIDI::Manager::instance()->add_port (req) != 0) {
+	XMLNode node (X_("MIDI-port"));
+
+	node.add_property ("tag", dialog.port_name.get_text());
+	node.add_property ("device", X_("ardour")); // XXX this can't be right for all types
+	node.add_property ("type", MIDI::PortFactory::default_port_type());
+	node.add_property ("mode", smod);
+
+	if (MIDI::Manager::instance()->add_port (node) != 0) {
 		redisplay_midi_ports ();
 	}
 }
@@ -688,13 +813,17 @@ OptionEditor::map_port_online (MIDI::Port* port, ToggleButton* tb)
 }
 
 void
-OptionEditor::mmc_device_id_adjusted ()
+OptionEditor::mmc_receive_device_id_adjusted ()
 {
-	uint8_t id = (uint8_t) mmc_device_id_spinner.get_value();
+	uint8_t id = (uint8_t) mmc_receive_device_id_spinner.get_value();
+	Config->set_mmc_receive_device_id (id);
+}
 
-	if (id != Config->get_mmc_device_id()) {
-		Config->set_mmc_device_id (id);
-	}
+void
+OptionEditor::mmc_send_device_id_adjusted ()
+{
+	uint8_t id = (uint8_t) mmc_send_device_id_spinner.get_value();
+	Config->set_mmc_send_device_id (id);
 }
 
 void
@@ -748,10 +877,13 @@ OptionEditor::raid_path_changed ()
 void
 OptionEditor::click_browse_clicked ()
 {
-	SoundFileChooser sfdb (_("Choose Click"), session);
+	SoundFileChooser sfdb (*this, _("Choose Click"), session);
 	
-	int result = sfdb.run ();
+	sfdb.show_all ();
+	sfdb.present ();
 
+	int result = sfdb.run ();
+ 
 	if (result == Gtk::RESPONSE_OK) {
 		click_chosen(sfdb.get_filename());
 	}
@@ -767,7 +899,10 @@ OptionEditor::click_chosen (const string & path)
 void
 OptionEditor::click_emphasis_browse_clicked ()
 {
-	SoundFileChooser sfdb (_("Choose Click Emphasis"), session);
+	SoundFileChooser sfdb (*this, _("Choose Click Emphasis"), session);
+
+	sfdb.show_all ();
+	sfdb.present ();
 
 	int result = sfdb.run ();
 
@@ -943,6 +1078,23 @@ static const struct {
     const char *name;
     guint   modifier;
 } modifiers[] = {
+
+#ifdef GTKOSX 
+
+	/* Command = Mod1
+	   Option/Alt = Mod5
+	*/
+
+	{ "Shift", GDK_SHIFT_MASK },
+	{ "Command", GDK_MOD1_MASK },
+	{ "Control", GDK_CONTROL_MASK },
+	{ "Option", GDK_MOD5_MASK },
+	{ "Command-Shift", GDK_MOD1_MASK|GDK_SHIFT_MASK },
+	{ "Command-Option", GDK_MOD1_MASK|GDK_MOD5_MASK },
+	{ "Shift-Option", GDK_SHIFT_MASK|GDK_MOD5_MASK },
+	{ "Shift-Command-Option", GDK_MOD5_MASK|GDK_SHIFT_MASK|GDK_MOD1_MASK },
+
+#else
 	{ "Shift", GDK_SHIFT_MASK },
 	{ "Control", GDK_CONTROL_MASK },
 	{ "Alt (Mod1)", GDK_MOD1_MASK },
@@ -954,6 +1106,7 @@ static const struct {
 	{ "Mod3", GDK_MOD3_MASK },
 	{ "Mod4", GDK_MOD4_MASK },
 	{ "Mod5", GDK_MOD5_MASK },
+#endif
 	{ 0, 0 }
 };
 
@@ -1131,3 +1284,31 @@ OptionEditor::fixup_combo_size (Gtk::ComboBoxText& combo, vector<string>& string
 	set_size_request_to_display_given_text (combo, maxstring.c_str(), 10 + FUDGE, 10);
 }
 
+void
+OptionEditor::parameter_changed (const char* parameter_name)
+{
+	ENSURE_GUI_THREAD (bind (mem_fun (*this, &OptionEditor::parameter_changed), parameter_name));
+
+#define PARAM_IS(x) (!strcmp (parameter_name, (x)))
+	
+	if (PARAM_IS ("timecode-source-is-synced")) {
+		synced_timecode_button.set_active (Config->get_timecode_source_is_synced());
+	} else if (PARAM_IS ("history-depth")) {
+		int32_t depth = Config->get_history_depth();
+		
+		history_depth.set_value (depth);
+		history_depth_spinner.set_sensitive (depth != 0);
+		limit_history_button.set_active (depth != 0);
+
+	} else if (PARAM_IS ("saved-history-depth")) {
+
+		saved_history_depth.set_value (Config->get_saved_history_depth());
+
+	} else if (PARAM_IS ("save-history")) {
+
+		bool x = Config->get_save_history();
+
+		save_history_button.set_active (x);
+		saved_history_depth_spinner.set_sensitive (x);
+	}
+}
