@@ -65,8 +65,8 @@
 
 #include "i18n.h"
 
-#ifdef HAVE_AUDIOUNIT
-#include "au_pluginui.h"
+#ifdef HAVE_AUDIOUNITS
+class AUPluginUI;
 #endif
 
 using namespace sigc;
@@ -339,8 +339,10 @@ RedirectBox::redirect_button_release_event (GdkEventButton *ev)
 		show_redirect_menu(ev->time);
 		ret = true;
 
-	} else if (redirect && ev->button == 2 && Keyboard::modifier_state_equals (ev->state, Gdk::BUTTON2_MASK)) {
+	} else if (redirect && (ev->button == 2) && (Keyboard::no_modifier_keys_pressed (ev) && ((ev->state & Gdk::BUTTON2_MASK) == Gdk::BUTTON2_MASK))) {
 		
+		/* button2-click with no modifiers */
+
 		redirect->set_active (!redirect->active(), this);
 		ret = true;
 
@@ -395,13 +397,12 @@ RedirectBox::insert_plugin_chosen (boost::shared_ptr<Plugin> plugin)
 
 		boost::shared_ptr<Redirect> redirect (new PluginInsert (_session, plugin, _placement));
 		
-		redirect->active_changed.connect (bind (mem_fun (*this, &RedirectBox::show_redirect_active_r), boost::weak_ptr<Redirect>(redirect)));
-
 		uint32_t err_streams;
 
 		if (_route->add_redirect (redirect, this, &err_streams)) {
 			weird_plugin_dialog (*plugin, err_streams, _route);
-			// XXX SHAREDPTR delete plugin here .. do we even need to care? 
+		} else {
+			redirect->active_changed.connect (bind (mem_fun (*this, &RedirectBox::show_redirect_active_r), boost::weak_ptr<Redirect>(redirect)));
 		}
 	}
 }
@@ -414,7 +415,7 @@ RedirectBox::weird_plugin_dialog (Plugin& p, uint32_t streams, boost::shared_ptr
 
 	/* i hate this kind of code */
 
-	if (streams > p.get_info()->n_inputs) {
+	if (streams > (unsigned)p.get_info()->n_inputs) {
 		label.set_text (string_compose (_(
 "You attempted to add a plugin (%1).\n"
 "The plugin has %2 inputs\n"
@@ -426,7 +427,7 @@ RedirectBox::weird_plugin_dialog (Plugin& p, uint32_t streams, boost::shared_ptr
 					 p.name(),
 					 p.get_info()->n_inputs,
 					 streams));
-	} else if (streams < p.get_info()->n_inputs) {
+	} else if (streams < (unsigned)p.get_info()->n_inputs) {
 		label.set_text (string_compose (_(
 "You attempted to add a plugin (%1).\n"
 "The plugin has %2 inputs\n"
@@ -648,14 +649,14 @@ RedirectBox::show_redirect_active_r (Redirect* r, void *src, boost::weak_ptr<Red
 void
 RedirectBox::show_redirect_active (boost::weak_ptr<Redirect> weak_redirect)
 {
-	ENSURE_GUI_THREAD(bind (mem_fun(*this, &RedirectBox::show_redirect_active), weak_redirect));
-	
 	boost::shared_ptr<Redirect> redirect (weak_redirect.lock());
 	
 	if (!redirect) {
 		return;
 	}
 
+	ENSURE_GUI_THREAD(bind (mem_fun(*this, &RedirectBox::show_redirect_active), weak_redirect));
+	
 	Gtk::TreeModel::Children children = model->children();
 	Gtk::TreeModel::Children::iterator iter = children.begin();
 
@@ -1076,67 +1077,45 @@ RedirectBox::edit_redirect (boost::shared_ptr<Redirect> redirect)
 	} else {
 		
 		/* it's an insert */
-		
+
 		boost::shared_ptr<PluginInsert> plugin_insert;
 		boost::shared_ptr<PortInsert> port_insert;
 		
 		if ((plugin_insert = boost::dynamic_pointer_cast<PluginInsert> (insert)) != 0) {
 			
-			ARDOUR::PluginType type = plugin_insert->type();
-
-			if (type == ARDOUR::LADSPA || type == ARDOUR::VST) {
-				PluginUIWindow *plugin_ui;
+			PluginUIWindow *plugin_ui;
 			
-				if (plugin_insert->get_gui() == 0) {
-								
-					plugin_ui = new PluginUIWindow (plugin_insert);
-
-					if (_owner_is_mixer) {
-						ARDOUR_UI::instance()->the_mixer()->ensure_float (*plugin_ui);
-					} else {
-						ARDOUR_UI::instance()->the_editor().ensure_float (*plugin_ui);
-					}
-
-					WindowTitle title(Glib::get_application_name());
-					title += generate_redirect_title (plugin_insert);
-					plugin_ui->set_title (title.get_string());
-
-					plugin_insert->set_gui (plugin_ui);
-					
-					// change window title when route name is changed
-					_route->name_changed.connect (bind (mem_fun(*this, &RedirectBox::route_name_changed), plugin_ui, boost::weak_ptr<PluginInsert> (plugin_insert)));
-					
+			if (plugin_insert->get_gui() == 0) {
 				
+				plugin_ui = new PluginUIWindow (plugin_insert);
+				
+				if (_owner_is_mixer) {
+					ARDOUR_UI::instance()->the_mixer()->ensure_float (*plugin_ui);
 				} else {
-					plugin_ui = reinterpret_cast<PluginUIWindow *> (plugin_insert->get_gui());
-				}
-			
-				if (plugin_ui->is_visible()) {
-					plugin_ui->get_window()->raise ();
-				} else {
-					plugin_ui->show_all ();
-					plugin_ui->present ();
-				}
-#ifdef HAVE_AUDIOUNIT
-			} else if (type == ARDOUR::AudioUnit) {
-				AUPluginUI* plugin_ui;
-				if (plugin_insert->get_gui() == 0) {
-					plugin_ui = new AUPluginUI (plugin_insert);
-				} else {
-					plugin_ui = reinterpret_cast<AUPluginUI*> (plugin_insert->get_gui());
+					ARDOUR_UI::instance()->the_editor().ensure_float (*plugin_ui);
 				}
 				
-				if (plugin_ui->is_visible()) {
-					plugin_ui->get_window()->raise ();
-				} else {
-					plugin_ui->show_all ();
-					plugin_ui->present ();
-				}
-#endif				
+				WindowTitle title(Glib::get_application_name());
+				title += generate_redirect_title (plugin_insert);
+				plugin_ui->set_title (title.get_string());
+				
+				plugin_insert->set_gui (plugin_ui);
+				
+				// change window title when route name is changed
+				_route->name_changed.connect (bind (mem_fun(*this, &RedirectBox::route_name_changed), plugin_ui, boost::weak_ptr<PluginInsert> (plugin_insert)));
+				
+				
 			} else {
-				warning << "Unsupported plugin sent to RedirectBox::edit_redirect()" << endmsg;
-				return;
+				plugin_ui = reinterpret_cast<PluginUIWindow *> (plugin_insert->get_gui());
 			}
+			
+			if (plugin_ui->is_visible()) {
+				plugin_ui->get_window()->raise ();
+			} else {
+				plugin_ui->show_all ();
+				plugin_ui->present ();
+			}
+
 		} else if ((port_insert = boost::dynamic_pointer_cast<PortInsert> (insert)) != 0) {
 			
 			if (!_session.engine().connected()) {
@@ -1158,6 +1137,7 @@ RedirectBox::edit_redirect (boost::shared_ptr<Redirect> redirect)
 			if (io_selector->is_visible()) {
 				io_selector->get_window()->raise ();
 			} else {
+				io_selector->show_all ();
 				io_selector->present ();
 			}
 		}
