@@ -50,6 +50,7 @@
 #include <ardour/session.h>
 #include <ardour/session_playlist.h>
 #include <ardour/utils.h>
+#include <ardour/profile.h>
 
 #include "ardour_ui.h"
 #include "route_time_axis.h"
@@ -92,9 +93,12 @@ RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session& sess, boost::sh
 	  playlist_button (_("p")), 
 	  size_button (_("h")), // height
 	  automation_button (_("a")),
-	  visual_button (_("v"))
+	  visual_button (_("v")),
+	  lm (rt, sess)
 
 {
+	lm.set_no_show_all();
+	lm.setup_meters(50);
 	_has_state = true;
 	playlist_menu = 0;
 	playlist_action_menu = 0;
@@ -102,6 +106,8 @@ RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session& sess, boost::sh
 	_view = 0;
 	timestretch_rect = 0;
 	no_redraw = false;
+	destructive_track_mode_item = 0;
+	normal_track_mode_item = 0;
 
 	ignore_toggle = false;
 
@@ -144,12 +150,18 @@ RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session& sess, boost::sh
 
 		rec_enable_button->signal_button_press_event().connect (mem_fun(*this, &RouteUI::rec_enable_press), false);
 		rec_enable_button->signal_button_release_event().connect (mem_fun(*this, &RouteUI::rec_enable_release));
-		controls_table.attach (*rec_enable_button, 5, 6, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
+		controls_table.attach (*rec_enable_button, 4, 5, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
 		ARDOUR_UI::instance()->tooltips().set_tip(*rec_enable_button, _("Record"));
+
 	}
 
-	controls_table.attach (*mute_button, 6, 7, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
-	controls_table.attach (*solo_button, 7, 8, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::FILL|Gtk::EXPAND, 0, 0);
+	controls_hbox.pack_start(lm, false, false);
+	_route->meter_change.connect (mem_fun(*this, &RouteTimeAxisView::meter_changed));
+	_route->input_changed.connect (mem_fun(*this, &RouteTimeAxisView::io_changed));
+	_route->output_changed.connect (mem_fun(*this, &RouteTimeAxisView::io_changed));
+
+	controls_table.attach (*mute_button, 5, 6, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
+	controls_table.attach (*solo_button, 6, 7, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
 
 	controls_table.attach (edit_group_button, 6, 7, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
 
@@ -164,17 +176,22 @@ RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session& sess, boost::sh
 	
 	label_view ();
 
-	controls_table.attach (hide_button, 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
-	controls_table.attach (visual_button, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
-	controls_table.attach (size_button, 2, 3, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
-	controls_table.attach (automation_button, 3, 4, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
+	if (!Profile->get_sae()) {
+
+		controls_table.attach (hide_button, 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
+		controls_table.attach (visual_button, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
+		controls_table.attach (size_button, 2, 3, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
+		controls_table.attach (automation_button, 3, 4, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
+
+	} else {
+
+		controls_table.attach (automation_button, 4, 5, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
+	}
 
 	if (is_track() && track()->mode() == ARDOUR::Normal) {
 		controls_table.attach (playlist_button, 5, 6, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
 	}
 
-	/* remove focus from the buttons */
-	
 	y_position = -1;
 
 	_route->redirects_changed.connect (mem_fun(*this, &RouteTimeAxisView::redirects_changed));
@@ -251,7 +268,7 @@ RouteTimeAxisView::playlist_modified ()
 gint
 RouteTimeAxisView::edit_click (GdkEventButton *ev)
 {
-	if (Keyboard::modifier_state_equals (ev->state, Keyboard::Control)) {
+	if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
 	        _route->set_edit_group (0, this);
 		return FALSE;
 	} 
@@ -407,13 +424,15 @@ RouteTimeAxisView::build_display_menu ()
 	items.push_back (SeparatorElem());
 
 	build_remote_control_menu ();
-	items.push_back (MenuElem (_("Remote Control ID"), *remote_control_menu));
-
 	build_automation_action_menu ();
-	items.push_back (MenuElem (_("Automation"), *automation_action_menu));
+
+	if (!Profile->get_sae()) {
+		items.push_back (MenuElem (_("Remote Control ID"), *remote_control_menu));
+		items.push_back (MenuElem (_("Automation"), *automation_action_menu));
+		items.push_back (SeparatorElem());
+	}
 
 	// Hook for derived classes to add type specific stuff
-	items.push_back (SeparatorElem());
 	append_extra_display_menu_items ();
 	items.push_back (SeparatorElem());
 	
@@ -437,31 +456,31 @@ RouteTimeAxisView::build_display_menu ()
 		if (get_diskstream()->alignment_style() == CaptureTime)
 			align_capture_item->set_active();
 		
-		items.push_back (MenuElem (_("Alignment"), *alignment_menu));
+		if (!Profile->get_sae()) {
+			items.push_back (MenuElem (_("Alignment"), *alignment_menu));
+			get_diskstream()->AlignmentStyleChanged.connect (mem_fun(*this, &RouteTimeAxisView::align_style_changed));
+			
+			RadioMenuItem::Group mode_group;
+			items.push_back (RadioMenuElem (mode_group, _("Normal mode"),
+							bind (mem_fun (*this, &RouteTimeAxisView::set_track_mode), ARDOUR::Normal)));
+			normal_track_mode_item = dynamic_cast<RadioMenuItem*>(&items.back());
+			items.push_back (RadioMenuElem (mode_group, _("Tape mode"),
+							bind (mem_fun (*this, &RouteTimeAxisView::set_track_mode), ARDOUR::Destructive)));
+			destructive_track_mode_item = dynamic_cast<RadioMenuItem*>(&items.back());
+			
+			switch (track()->mode()) {
+			case ARDOUR::Destructive:
+				destructive_track_mode_item->set_active ();
+				break;
+			case ARDOUR::Normal:
+				normal_track_mode_item->set_active ();
+				break;
+			}
 
-		get_diskstream()->AlignmentStyleChanged.connect (
-			mem_fun(*this, &RouteTimeAxisView::align_style_changed));
-
-		RadioMenuItem::Group mode_group;
-		items.push_back (RadioMenuElem (mode_group, _("Normal mode"),
-						bind (mem_fun (*this, &RouteTimeAxisView::set_track_mode), ARDOUR::Normal)));
-		normal_track_mode_item = dynamic_cast<RadioMenuItem*>(&items.back());
-		items.push_back (RadioMenuElem (mode_group, _("Tape mode"),
-						bind (mem_fun (*this, &RouteTimeAxisView::set_track_mode), ARDOUR::Destructive)));
-		destructive_track_mode_item = dynamic_cast<RadioMenuItem*>(&items.back());
-				 
-		
-		switch (track()->mode()) {
-		case ARDOUR::Destructive:
-			destructive_track_mode_item->set_active ();
-			break;
-		case ARDOUR::Normal:
-			normal_track_mode_item->set_active ();
-			break;
+			items.push_back (SeparatorElem());
 		}
 	}
 
-	items.push_back (SeparatorElem());
 	items.push_back (CheckMenuElem (_("Active"), mem_fun(*this, &RouteUI::toggle_route_active)));
 	route_active_menu_item = dynamic_cast<CheckMenuItem *> (&items.back());
 	route_active_menu_item->set_active (_route->active());
@@ -496,14 +515,14 @@ RouteTimeAxisView::set_track_mode (TrackMode mode)
 		/*NOTREACHED*/
 		return;
 	}
-
-	if (item->get_active () && track()->mode() != mode) {
+	
+	if (item && other_item && item->get_active () && track()->mode() != mode) {
 		_set_track_mode (track(), mode, other_item);
 	}
 }
 
 void
-RouteTimeAxisView::_set_track_mode (Track* track, TrackMode mode, RadioMenuItem* reset_item)
+RouteTimeAxisView::_set_track_mode (boost::shared_ptr<Track> track, TrackMode mode, RadioMenuItem* reset_item)
 {
 	bool needs_bounce;
 
@@ -644,8 +663,9 @@ RouteTimeAxisView::show_selection (TimeSelection& ts)
 void
 RouteTimeAxisView::set_height (TrackHeight h)
 {
+	int gmlen = (height_to_pixels (h)) - 5;
 	bool height_changed = (height == 0) || (h != height_style);
-
+	lm.setup_meters (gmlen);
 	TimeAxisView::set_height (h);
 
 	ensure_xml_node ();
@@ -685,6 +705,7 @@ RouteTimeAxisView::set_height (TrackHeight h)
 	case Large:
 	case Larger:
 	case Normal:
+		reset_meter();
 		show_name_entry ();
 		hide_name_label ();
 
@@ -705,6 +726,7 @@ RouteTimeAxisView::set_height (TrackHeight h)
 		break;
 
 	case Smaller:
+		reset_meter();
 		show_name_entry ();
 		hide_name_label ();
 
@@ -725,6 +747,7 @@ RouteTimeAxisView::set_height (TrackHeight h)
 		break;
 
 	case Small:
+		hide_meter ();
 		hide_name_entry ();
 		show_name_label ();
 
@@ -986,7 +1009,7 @@ RouteTimeAxisView::update_diskstream_display ()
 void
 RouteTimeAxisView::selection_click (GdkEventButton* ev)
 {
-	if (Keyboard::modifier_state_equals (ev->state, (Keyboard::Shift|Keyboard::Control))) {
+	if (Keyboard::modifier_state_equals (ev->state, (Keyboard::TertiaryModifier|Keyboard::PrimaryModifier))) {
 
 		/* special case: select/deselect all tracks */
 		if (editor.get_selection().selected (this)) {
@@ -1164,6 +1187,19 @@ RouteTimeAxisView::find_next_region (nframes_t pos, RegionPoint point, int32_t d
 	return boost::shared_ptr<Region> ();
 }
 
+nframes64_t 
+RouteTimeAxisView::find_next_region_boundary (nframes64_t pos, int32_t dir)
+{
+	boost::shared_ptr<Diskstream> stream;
+	boost::shared_ptr<Playlist> playlist;
+
+	if ((stream = get_diskstream()) != 0 && (playlist = stream->playlist()) != 0) {
+		return playlist->find_next_region_boundary (pos, dir);
+	}
+
+	return -1;
+}
+
 bool
 RouteTimeAxisView::cut_copy_clear (Selection& selection, CutCopyOp op)
 {
@@ -1304,10 +1340,10 @@ RouteTimeAxisView::build_playlist_menu (Gtk::Menu * menu)
 	playlist_items.push_back (MenuElem (_("Rename"), mem_fun(*this, &RouteTimeAxisView::rename_current_playlist)));
 	playlist_items.push_back (SeparatorElem());
 
-	playlist_items.push_back (MenuElem (_("New"), mem_fun(editor, &PublicEditor::new_playlists)));
-	playlist_items.push_back (MenuElem (_("New Copy"), mem_fun(editor, &PublicEditor::copy_playlists)));
+	playlist_items.push_back (MenuElem (_("New"), bind(mem_fun(editor, &PublicEditor::new_playlists), this)));
+	playlist_items.push_back (MenuElem (_("New Copy"), bind(mem_fun(editor, &PublicEditor::copy_playlists), this)));
 	playlist_items.push_back (SeparatorElem());
-	playlist_items.push_back (MenuElem (_("Clear Current"), mem_fun(editor, &PublicEditor::clear_playlists)));
+	playlist_items.push_back (MenuElem (_("Clear Current"), bind(mem_fun(editor, &PublicEditor::clear_playlists), this)));
 	playlist_items.push_back (SeparatorElem());
 
 	playlist_items.push_back (MenuElem(_("Select from all ..."), mem_fun(*this, &RouteTimeAxisView::show_playlist_selector)));
@@ -1368,6 +1404,8 @@ RouteTimeAxisView::color_handler ()
 	if (timestretch_rect) {
 		timestretch_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_TimeStretchFill.get();
 	}
+
+	reset_meter();
 }
 
 void
@@ -1490,7 +1528,7 @@ static string
 legalize_for_xml_node (string str)
 {
 	string::size_type pos;
-	string legal_chars = "abcdefghijklmnopqrtsuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_+=:";
+	string legal_chars = "abcdefghijklmnopqrtsuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_=:";
 	string legal;
 
 	legal = str;
@@ -1774,4 +1812,51 @@ RouteTimeAxisView::update_rec_display ()
 	RouteUI::update_rec_display ();
 	name_entry.set_sensitive (!_route->record_enabled());
 }
-		
+
+void
+RouteTimeAxisView::fast_update ()
+{
+	lm.update_meters ();
+}
+
+void
+RouteTimeAxisView::hide_meter ()
+{
+	clear_meter ();
+	lm.hide_meters ();
+}
+
+void
+RouteTimeAxisView::show_meter ()
+{
+	reset_meter ();
+}
+
+void
+RouteTimeAxisView::reset_meter ()
+{
+	if (Config->get_show_track_meters()) {
+		lm.setup_meters (height-5);
+	} else {
+		hide_meter ();
+	}
+}
+
+void
+RouteTimeAxisView::clear_meter ()
+{
+	lm.clear_meters ();
+}
+
+void
+RouteTimeAxisView::meter_changed (void *src)
+{
+	ENSURE_GUI_THREAD (bind (mem_fun(*this, &RouteTimeAxisView::meter_changed), src));
+	reset_meter();
+}
+
+void
+RouteTimeAxisView::io_changed (IOChange change, void *src)
+{
+	reset_meter ();
+}
