@@ -71,7 +71,7 @@ bool TimeAxisView::need_size_info = true;
 TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisView* rent, Canvas& canvas) 
 	: AxisView (sess), 
 	  editor (ed),
-	  controls_table (2, 8)
+	  controls_table (2, 7)
 {
 	if (need_size_info) {
 		compute_controls_size_info ();
@@ -125,7 +125,7 @@ TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisVie
 	controls_table.set_col_spacings (0);
 	controls_table.set_homogeneous (true);
 
-	controls_table.attach (name_hbox, 0, 4, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
+	controls_table.attach (name_hbox, 0, 4, 0, 1, Gtk::SHRINK|Gtk::EXPAND, Gtk::SHRINK|Gtk::EXPAND);
 	controls_table.show_all ();
 	controls_table.set_no_show_all ();
 
@@ -149,7 +149,7 @@ TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisVie
 	controls_frame.set_name ("TimeAxisViewControlsBaseUnselected");
 	controls_frame.set_shadow_type (Gtk::SHADOW_ETCHED_OUT);
 
-	ColorChanged.connect (mem_fun (*this, &TimeAxisView::color_handler));
+	ColorsChanged.connect (mem_fun (*this, &TimeAxisView::color_handler));
 }
 
 TimeAxisView::~TimeAxisView()
@@ -248,14 +248,14 @@ TimeAxisView::controls_ebox_scroll (GdkEventScroll* ev)
 {
 	switch (ev->direction) {
 	case GDK_SCROLL_UP:
-		if (Keyboard::modifier_state_equals (ev->state, Keyboard::Shift)) {
+		if (Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier)) {
 			step_height (true);
 			return true;
 		}
 		break;
 		
 	case GDK_SCROLL_DOWN:
-		if (Keyboard::modifier_state_equals (ev->state, Keyboard::Shift)) {
+		if (Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier)) {
 			step_height (false);
 			return true;
 		}
@@ -365,11 +365,12 @@ TimeAxisView::set_height_pixels (uint32_t h)
 {
 	height = h;
 	controls_frame.set_size_request (-1, height + ((order == 0) ? 1 : 0));
-
+	//cerr << "TimeAxisView::set_height_pixels() called h = " << h << endl;//DEBUG
  	if (canvas_item_visible (selection_group)) {
 		/* resize the selection rect */
 		show_selection (editor.get_selection().time);
 	}
+	
 }
 
 bool
@@ -519,10 +520,7 @@ TimeAxisView::popup_display_menu (guint32 when)
 		build_display_menu ();
 	}
 
-	if (!get_selected()) {
-		editor.set_selected_track (*this, Selection::Set);
-	}
-
+	editor.set_selected_track (*this, Selection::Add);
 	display_menu->popup (1, when);	
 }
 
@@ -546,12 +544,16 @@ TimeAxisView::popup_size_menu (guint32 when)
 void
 TimeAxisView::set_selected (bool yn)
 {
-	AxisView::set_selected (yn);
+	if (yn == _selected) {
+		return;
+	}
+	
+	Selectable::set_selected (yn);
 
 	if (_selected) {
 		controls_ebox.set_name (controls_base_selected_name);
 		controls_frame.set_name (controls_base_selected_name);
-
+		
 		/* propagate any existing selection, if the mode is right */
 
 		if (editor.current_mouse_mode() == Editing::MouseRange && !editor.get_selection().time.empty()) {
@@ -572,8 +574,6 @@ TimeAxisView::set_selected (bool yn)
 		for (vector<TimeAxisView*>::iterator i = children.begin(); i != children.end(); ++i) {
 			(*i)->set_selected (false);
 		}
-
-		
 	}
 }
 
@@ -784,20 +784,20 @@ TimeAxisView::get_selection_rect (uint32_t id)
 		rect->rect->property_y1() = 0.0;
 		rect->rect->property_x2() = 0.0;
 		rect->rect->property_y2() = 0.0;
-		rect->rect->property_fill_color_rgba() = color_map[cSelectionRectFill];
-		rect->rect->property_outline_color_rgba() = color_map[cSelectionRectOutline];
+		rect->rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_SelectionRect.get();
+		rect->rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_SelectionRect.get();
 		
 		rect->start_trim = new SimpleRect (*selection_group);
 		rect->start_trim->property_x1() = 0.0;
 		rect->start_trim->property_x2() = 0.0;
-		rect->start_trim->property_fill_color_rgba() = color_map[cSelectionStartFill];
-		rect->start_trim->property_outline_color_rgba() = color_map[cSelectionStartOutline];
+		rect->start_trim->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
+		rect->start_trim->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
 		
 		rect->end_trim = new SimpleRect (*selection_group);
 		rect->end_trim->property_x1() = 0.0;
 		rect->end_trim->property_x2() = 0.0;
-		rect->end_trim->property_fill_color_rgba() = color_map[cSelectionEndFill];
-		rect->end_trim->property_outline_color_rgba() = color_map[cSelectionEndOutline];
+		rect->end_trim->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
+		rect->end_trim->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
 
 		free_selection_rects.push_front (rect);
 
@@ -1053,23 +1053,30 @@ TimeAxisView::hide_name_entry ()
 }
 
 void
-TimeAxisView::color_handler (ColorID id, uint32_t val)
+TimeAxisView::color_handler ()
 {
-	switch (id) {
-	case cSelectionRectFill:
-		break;
-	case cSelectionRectOutline:
-		break;
-	case cSelectionStartFill:
-		break;
-	case cSelectionStartOutline:
-		break;
-	case cSelectionEndFill:
-		break;
-	case cSelectionEndOutline:
-		break;
-	default:
-		break;
+	for (list<SelectionRect*>::iterator i = used_selection_rects.begin(); i != used_selection_rects.end(); ++i) {
+
+		(*i)->rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_SelectionRect.get();
+		(*i)->rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
+
+		(*i)->start_trim->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
+		(*i)->start_trim->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
+
+		(*i)->end_trim->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
+		(*i)->end_trim->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
+	}
+
+	for (list<SelectionRect*>::iterator i = free_selection_rects.begin(); i != free_selection_rects.end(); ++i) {
+
+		(*i)->rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_SelectionRect.get();
+		(*i)->rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
+
+		(*i)->start_trim->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
+		(*i)->start_trim->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
+
+		(*i)->end_trim->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
+		(*i)->end_trim->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_Selection.get();
 	}
 }
 
