@@ -20,6 +20,7 @@
 #include <unistd.h>
 #include <cerrno>
 #include <vector>
+#include <sstream>
 
 #include <glibmm/timer.h>
 #include <pbd/pthread_utils.h>
@@ -48,7 +49,8 @@ gint AudioEngine::m_meter_exit;
 static void 
 ardour_jack_error (const char* msg) 
 {
-	error << "JACK: " << msg << endmsg;
+	// throw JACK errors away - they cause visual clutter
+	// error << "JACK: " << msg << endmsg;
 }
 
 AudioEngine::AudioEngine (string client_name) 
@@ -151,7 +153,7 @@ AudioEngine::start ()
 			_has_run = true;
 			Running(); /* EMIT SIGNAL */
 		} else {
-			error << _("cannot activate JACK client") << endmsg;
+			// error << _("cannot activate JACK client") << endmsg;
 		}
 	}
 
@@ -330,6 +332,23 @@ AudioEngine::process_callback (nframes_t nframes)
 		last_monitor_check = next_processed_frames;
 	}
 
+	if (session->silent()) {
+
+		boost::shared_ptr<Ports> p = ports.reader();
+
+		for (Ports::iterator i = p->begin(); i != p->end(); ++i) {
+			
+			Port *port = (*i);
+			
+			if (port->sends_output()) {
+				Sample *buf = port->get_buffer(nframes);
+				memset (buf, 0, sizeof(Sample) * nframes);
+				// this should work but doesn't
+				//port->silence(0, nframes);  //need to implement declicking fade
+			}
+		}
+	}
+
 	_processed_frames = next_processed_frames;
 	return 0;
 }
@@ -462,6 +481,26 @@ AudioEngine::remove_session ()
 	remove_all_ports ();
 }
 
+void
+AudioEngine::port_registration_failure (const std::string& portname)
+{
+	string full_portname = jack_client_name;
+	full_portname += ':';
+	full_portname += portname;
+	
+	
+	jack_port_t* p = jack_port_by_name (_jack, full_portname.c_str());
+	string reason;
+	
+	if (p) {
+		reason = _("a port with this name already exists: check for duplicated track/bus names");
+	} else {
+		reason = _("unknown error");
+	}
+	
+	throw PortRegistrationFailure (string_compose (_("AudioEngine: cannot register port \"%1\": %2"), portname, reason).c_str());
+}	
+
 Port *
 AudioEngine::register_input_port (DataType type, const string& portname)
 {
@@ -490,7 +529,7 @@ AudioEngine::register_input_port (DataType type, const string& portname)
 		return newport;
 
 	} else {
-		throw PortRegistrationFailure();
+		port_registration_failure (portname);
 	}
 
 	return 0;
@@ -528,7 +567,7 @@ AudioEngine::register_output_port (DataType type, const string& portname)
 		return newport;
 
 	} else {
-		throw PortRegistrationFailure ();
+		port_registration_failure (portname);
 	}
 
 	return 0;
@@ -768,7 +807,7 @@ AudioEngine::halted (void *arg)
 }
 
 uint32_t
-AudioEngine::n_physical_outputs () const
+AudioEngine::n_physical_audio_outputs () const
 {
 	const char ** ports;
 	uint32_t i = 0;
@@ -777,7 +816,7 @@ AudioEngine::n_physical_outputs () const
 		return 0;
 	}
 
-	if ((ports = jack_get_ports (_jack, NULL, NULL, JackPortIsPhysical|JackPortIsInput)) == 0) {
+	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsInput)) == 0) {
 		return 0;
 	}
 
@@ -789,7 +828,7 @@ AudioEngine::n_physical_outputs () const
 }
 
 uint32_t
-AudioEngine::n_physical_inputs () const
+AudioEngine::n_physical_audio_inputs () const
 {
 	const char ** ports;
 	uint32_t i = 0;
@@ -798,7 +837,7 @@ AudioEngine::n_physical_inputs () const
 		return 0;
 	}
 	
-	if ((ports = jack_get_ports (_jack, NULL, NULL, JackPortIsPhysical|JackPortIsOutput)) == 0) {
+	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsOutput)) == 0) {
 		return 0;
 	}
 
@@ -810,7 +849,7 @@ AudioEngine::n_physical_inputs () const
 }
 
 void
-AudioEngine::get_physical_inputs (vector<string>& ins)
+AudioEngine::get_physical_audio_inputs (vector<string>& ins)
 {
 	const char ** ports;
 	uint32_t i = 0;
@@ -819,7 +858,7 @@ AudioEngine::get_physical_inputs (vector<string>& ins)
 		return;
 	}
 	
-	if ((ports = jack_get_ports (_jack, NULL, NULL, JackPortIsPhysical|JackPortIsOutput)) == 0) {
+	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsOutput)) == 0) {
 		return;
 	}
 
@@ -832,7 +871,7 @@ AudioEngine::get_physical_inputs (vector<string>& ins)
 }
 
 void
-AudioEngine::get_physical_outputs (vector<string>& outs)
+AudioEngine::get_physical_audio_outputs (vector<string>& outs)
 {
 	const char ** ports;
 	uint32_t i = 0;
@@ -841,7 +880,7 @@ AudioEngine::get_physical_outputs (vector<string>& outs)
 		return;
 	}
 	
-	if ((ports = jack_get_ports (_jack, NULL, NULL, JackPortIsPhysical|JackPortIsInput)) == 0) {
+	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsInput)) == 0) {
 		return;
 	}
 
@@ -854,7 +893,7 @@ AudioEngine::get_physical_outputs (vector<string>& outs)
 }
 
 string
-AudioEngine::get_nth_physical (uint32_t n, int flag)
+AudioEngine::get_nth_physical_audio (uint32_t n, int flag)
 {
 	const char ** ports;
 	uint32_t i;
@@ -869,7 +908,7 @@ AudioEngine::get_nth_physical (uint32_t n, int flag)
 		}
 	}
 
-	ports = jack_get_ports (_jack, NULL, NULL, JackPortIsPhysical|flag);
+	ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|flag);
 	
 	if (ports == 0) {
 		return "";
@@ -1025,19 +1064,10 @@ AudioEngine::connect_to_jack (string client_name)
 	_jack = jack_client_open (jack_client_name.c_str(), options, &status, server_name);
 	
 	if (_jack == NULL) {
-
-		if (status & JackServerFailed) {
-			error << _("Unable to connect to JACK server") << endmsg;
-		}
-		
-		error << string_compose (_("Could not connect to JACK server as  \"%1\""), jack_client_name) <<  endmsg;
+		/* just return without an error message. something else will take care of it */
 		return -1;
 	}
 
-	if (status & JackServerStarted) {
-		info << _("JACK server started") << endmsg;
-	}
-
 	if (status & JackNameNotUnique) {
 		jack_client_name = jack_get_client_name (_jack);
 	}
