@@ -20,6 +20,9 @@
 #include <unistd.h>
 #include <cstdio> /* for snprintf, grrr */
 
+#include <glib.h>  
+#include <glib/gstdio.h> /* for g_stat() */
+
 #include <pbd/failed_constructor.h>
 #include <pbd/xml++.h>
 
@@ -74,7 +77,8 @@ int
 Configuration::load_state ()
 {
 	string rcfile;
-	
+	struct stat statbuf;
+
 	/* load system configuration first */
 
 	rcfile = find_config_file ("ardour_system.rc");
@@ -83,18 +87,28 @@ Configuration::load_state ()
 
 		XMLTree tree;
 
-		cerr << string_compose (_("loading system configuration file %1"), rcfile) << endl;
+		/* stupid XML Parser hates empty files */
 		
-		if (!tree.read (rcfile.c_str())) {
-			error << string_compose(_("Ardour: cannot read system configuration file \"%1\""), rcfile) << endmsg;
+		if (g_stat (rcfile.c_str(), &statbuf)) {
 			return -1;
 		}
 
-		current_owner = ConfigVariableBase::System;
-
-		if (set_state (*tree.root())) {
-			error << string_compose(_("Ardour: system configuration file \"%1\" not loaded successfully."), rcfile) << endmsg;
-			return -1;
+		if (statbuf.st_size != 0) {
+			cerr << string_compose (_("loading system configuration file %1"), rcfile) << endl;
+			
+			if (!tree.read (rcfile.c_str())) {
+				error << string_compose(_("Ardour: cannot read system configuration file \"%1\""), rcfile) << endmsg;
+				return -1;
+			}
+			
+			current_owner = ConfigVariableBase::System;
+			
+			if (set_state (*tree.root())) {
+				error << string_compose(_("Ardour: system configuration file \"%1\" not loaded successfully."), rcfile) << endmsg;
+				return -1;
+			}
+		} else {
+			error << _("your system Ardour configuration file is empty. This probably means that there as an error installing Ardour") << endmsg;
 		}
 	}
 
@@ -106,20 +120,30 @@ Configuration::load_state ()
 	if (rcfile.length()) {
 
 		XMLTree tree;
-		
-		cerr << string_compose (_("loading user configuration file %1"), rcfile) << endl;
 
-		if (!tree.read (rcfile)) {
-			error << string_compose(_("Ardour: cannot read configuration file \"%1\""), rcfile) << endmsg;
+		/* stupid XML parser hates empty files */
+
+		if (g_stat (rcfile.c_str(), &statbuf)) {
 			return -1;
 		}
 
-		current_owner = ConfigVariableBase::Config;
-
-		if (set_state (*tree.root())) {
-			error << string_compose(_("Ardour: user configuration file \"%1\" not loaded successfully."), rcfile) << endmsg;
-			return -1;
-		}
+		if (statbuf.st_size != 0) {
+			cerr << string_compose (_("loading user configuration file %1"), rcfile) << endl;
+			
+			if (!tree.read (rcfile)) {
+				error << string_compose(_("Ardour: cannot read configuration file \"%1\""), rcfile) << endmsg;
+				return -1;
+			}
+			
+			current_owner = ConfigVariableBase::Config;
+			
+			if (set_state (*tree.root())) {
+				error << string_compose(_("Ardour: user configuration file \"%1\" not loaded successfully."), rcfile) << endmsg;
+				return -1;
+			}
+		} else {
+			warning << _("your Ardour configuration file is empty. This is not normal.") << endmsg;
+		}	
 	}
 
 	return 0;
@@ -217,10 +241,12 @@ Configuration::set_state (const XMLNode& root)
 		if (node->name() == "MIDI-port") {
 
 			try {
-				pair<string,MidiPortDescriptor*> newpair;
-				newpair.second = new MidiPortDescriptor (*node);
-				newpair.first = newpair.second->tag;
-				midi_ports.insert (newpair);
+				MIDI::Port::Descriptor desc (*node);
+				map<string,XMLNode>::iterator x;
+				if ((x = midi_ports.find (desc.tag)) != midi_ports.end()) {
+					midi_ports.erase (x);
+				}
+				midi_ports.insert (pair<string,XMLNode>(desc.tag,*node));
 			}
 
 			catch (failed_constructor& err) {
@@ -263,52 +289,6 @@ Configuration::set_variables (const XMLNode& node, ConfigVariableBase::Owner own
 
 }
 
-Configuration::MidiPortDescriptor::MidiPortDescriptor (const XMLNode& node)
-{
-	const XMLProperty *prop;
-	bool have_tag = false;
-	bool have_device = false;
-	bool have_type = false;
-	bool have_mode = false;
-
-	if ((prop = node.property ("tag")) != 0) {
-		tag = prop->value();
-		have_tag = true;
-	}
-
-	if ((prop = node.property ("device")) != 0) {
-		device = prop->value();
-		have_device = true;
-	}
-
-	if ((prop = node.property ("type")) != 0) {
-		type = prop->value();
-		have_type = true;
-	}
-
-	if ((prop = node.property ("mode")) != 0) {
-		mode = prop->value();
-		have_mode = true;
-	}
-
-	if (!have_tag || !have_device || !have_type || !have_mode) {
-		throw failed_constructor();
-	}
-}
-
-XMLNode&
-Configuration::MidiPortDescriptor::get_state()
-{
-	XMLNode* root = new XMLNode("MIDI-port");
-
-	root->add_property("tag", tag);
-	root->add_property("device", device);
-	root->add_property("type", type);
-	root->add_property("mode", mode);
-
-	return *root;
-}
-
 void
 Configuration::map_parameters (sigc::slot<void,const char*> theSlot)
 {
