@@ -32,6 +32,8 @@
 #include <xmmintrin.h>
 #endif
 
+#include <glibmm/fileutils.h>
+
 #include <lrdf.h>
 
 #include <pbd/error.h>
@@ -40,7 +42,6 @@
 #include <pbd/fpu.h>
 
 #include <midi++/port.h>
-#include <midi++/port_request.h>
 #include <midi++/manager.h>
 #include <midi++/mmc.h>
 
@@ -52,6 +53,7 @@
 #include <ardour/audiosource.h>
 #include <ardour/utils.h>
 #include <ardour/session.h>
+#include <ardour/source_factory.h>
 #include <ardour/control_protocol_manager.h>
 
 #ifdef HAVE_LIBLO
@@ -109,44 +111,24 @@ setup_osc ()
 static int 
 setup_midi ()
 {
-	std::map<string,Configuration::MidiPortDescriptor*>::iterator i;
-	int nports;
-
-	if ((nports = Config->midi_ports.size()) == 0) {
+	if (Config->midi_ports.size() == 0) {
 		warning << _("no MIDI ports specified: no MMC or MTC control possible") << endmsg;
 		return 0;
 	}
 
-	for (i = Config->midi_ports.begin(); i != Config->midi_ports.end(); ++i) {
-		Configuration::MidiPortDescriptor* port_descriptor;
-
-		port_descriptor = (*i).second;
-
-		MIDI::PortRequest request (port_descriptor->device, 
-					   port_descriptor->tag, 
-					   port_descriptor->mode, 
-					   port_descriptor->type);
-
-		if (request.status != MIDI::PortRequest::OK) {
-			error << string_compose(_("MIDI port specifications for \"%1\" are not understandable."), port_descriptor->tag) << endmsg;
-			continue;
-		}
-		
-		MIDI::Manager::instance()->add_port (request);
-
-		nports++;
+	for (std::map<string,XMLNode>::iterator i = Config->midi_ports.begin(); i != Config->midi_ports.end(); ++i) {
+		MIDI::Manager::instance()->add_port (i->second);
 	}
 
 	MIDI::Port* first;
 	const MIDI::Manager::PortMap& ports = MIDI::Manager::instance()->get_midi_ports();
-	first = ports.begin()->second;
 
-	if (nports > 1) {
+	if (ports.size() > 1) {
+
+		first = ports.begin()->second;
 
 		/* More than one port, so try using specific names for each port */
 
-		map<string,Configuration::MidiPortDescriptor *>::iterator i;
-
 		if (Config->get_mmc_port_name() != N_("default")) {
 			default_mmc_port =  MIDI::Manager::instance()->port (Config->get_mmc_port_name());
 		} 
@@ -173,7 +155,9 @@ setup_midi ()
 			default_midi_port = first;
 		}
 		
-	} else {
+	} else if (ports.size() == 1) {
+
+		first = ports.begin()->second;
 
 		/* Only one port described, so use it for both MTC and MMC */
 
@@ -205,10 +189,11 @@ void
 setup_hardware_optimization (bool try_optimization)
 {
         bool generic_mix_functions = true;
-	FPU fpu;
 
 	if (try_optimization) {
 
+		FPU fpu;
+
 #if defined (ARCH_X86) && defined (BUILD_SSE_OPTIMIZATIONS)
 		
 		if (fpu.has_sse()) {
@@ -244,6 +229,10 @@ setup_hardware_optimization (bool try_optimization)
                         info << "Apple VecLib H/W specific optimizations in use" << endmsg;
                 }
 #endif
+
+		/* consider FPU denormal handling to be "h/w optimization" */
+
+		setup_fpu ();
         }
 
         if (generic_mix_functions) {
@@ -257,8 +246,6 @@ setup_hardware_optimization (bool try_optimization)
 		info << "No H/W specific optimizations in use" << endmsg;
 	}
 
-	setup_fpu ();
-
 }
 
 int
@@ -301,6 +288,8 @@ ARDOUR::init (bool use_vst, bool try_optimization)
 
 	setup_hardware_optimization (try_optimization);
 
+	SourceFactory::init ();
+
 	/* singleton - first object is "it" */
 	new PluginManager ();
 	
@@ -362,7 +351,7 @@ ARDOUR::new_change ()
 string
 ARDOUR::get_ardour_revision ()
 {
-	return "$Rev: 2163 $";
+	return "$Rev: 2893 $";
 }
 
 string
@@ -437,7 +426,7 @@ find_file (string name, string dir, string subdir = "")
 		for (vector<string>::iterator i = split_path.begin(); i != split_path.end(); ++i) {
 			path = *i;
 			path += "/" + name;
-			if (access (path.c_str(), R_OK) == 0) {
+			if (Glib::file_test (path, Glib::FILE_TEST_EXISTS)) {
 				// cerr << "Using file " << path << " found in ARDOUR_PATH." << endl;
 				return path;
 			}
@@ -478,7 +467,7 @@ find_file (string name, string dir, string subdir = "")
 string
 ARDOUR::find_config_file (string name)
 {
-	char* envvar;
+	const char* envvar;
 
 	if ((envvar = getenv("ARDOUR_CONFIG_PATH")) == 0) {
 		envvar = CONFIG_DIR;
@@ -490,7 +479,7 @@ ARDOUR::find_config_file (string name)
 string
 ARDOUR::find_data_file (string name, string subdir)
 {
-	char* envvar;
+	const char* envvar;
 	if ((envvar = getenv("ARDOUR_DATA_PATH")) == 0) {
 		envvar = DATA_DIR;
 	}
@@ -515,6 +504,12 @@ ARDOUR::LocaleGuard::~LocaleGuard ()
 void
 ARDOUR::setup_fpu ()
 {
+	if (getenv ("ARDOUR_RUNNING_UNDER_VALGRIND")) {
+		// valgrind doesn't understand this assembler stuff
+		// September 10th, 2007
+		return;
+	}
+
 #if defined(ARCH_X86) && defined(USE_XMMINTRIN)
 
 	int MXCSR;
