@@ -113,6 +113,7 @@ IO::IO (Session& s, string name,
 	  _output_maximum (output_max)
 {
 	_panner = new Panner (name, _session);
+	_active = true;
 	_gain = 1.0;
 	_desired_gain = 1.0;
 	_input_connection = 0;
@@ -148,6 +149,7 @@ IO::IO (Session& s, const XMLNode& node, DataType dt)
 	  _gain_automation_curve (0, 0, 0) // all reset in set_state()
 {
 	_panner = 0;
+	_active = true;
 	deferred_state = 0;
 	no_panner_reset = false;
 	_desired_gain = 1.0;
@@ -179,12 +181,16 @@ IO::~IO ()
 	Glib::Mutex::Lock lm (io_lock);
 	vector<Port *>::iterator i;
 
-	for (i = _inputs.begin(); i != _inputs.end(); ++i) {
-		_session.engine().unregister_port (*i);
-	}
-
-	for (i = _outputs.begin(); i != _outputs.end(); ++i) {
-		_session.engine().unregister_port (*i);
+	{
+		BLOCK_PROCESS_CALLBACK ();
+		
+		for (i = _inputs.begin(); i != _inputs.end(); ++i) {
+			_session.engine().unregister_port (*i);
+		}
+		
+		for (i = _outputs.begin(); i != _outputs.end(); ++i) {
+			_session.engine().unregister_port (*i);
+		}
 	}
 
 	m_meter_connection.disconnect();
@@ -1084,7 +1090,7 @@ IO::ensure_inputs_locked (uint32_t n, bool clear, void* src)
 			setup_peak_meters ();
 			reset_panner ();
 			/* pass it on */
-			throw AudioEngine::PortRegistrationFailure();
+			throw;
 		}
 		
 		_inputs.push_back (input_port);
@@ -1189,7 +1195,7 @@ IO::ensure_io (uint32_t nin, uint32_t nout, bool clear, void* src)
 				setup_peak_meters ();
 				reset_panner ();
 				/* pass it on */
-				throw AudioEngine::PortRegistrationFailure();
+				throw;
 			}
 		
 			_inputs.push_back (port);
@@ -1222,7 +1228,7 @@ IO::ensure_io (uint32_t nin, uint32_t nout, bool clear, void* src)
 				setup_peak_meters ();
 				reset_panner ();
 				/* pass it on */
-				throw AudioEngine::PortRegistrationFailure ();
+				throw;
 			}
 		
 			_outputs.push_back (port);
@@ -1465,6 +1471,7 @@ IO::state (bool full_state)
 	node->add_property("name", _name);
 	id().print (buf, sizeof (buf));
 	node->add_property("id", buf);
+	node->add_property("active", _active? "yes" : "no");
 
 	str = "";
 
@@ -1617,6 +1624,10 @@ IO::set_state (const XMLNode& node)
 		/* old school automation handling */
 	}
 
+	if ((prop = node.property (X_("active"))) != 0) {
+		set_active (prop->value() == "yes");
+	}
+
 	for (iter = node.children().begin(); iter != node.children().end(); ++iter) {
 
 		if ((*iter)->name() == "Panner") {
@@ -2475,7 +2486,7 @@ IO::meter ()
 {
 	Glib::Mutex::Lock lm (io_lock); // READER: meter thread.
 	uint32_t limit = max (_ninputs, _noutputs);
-	
+
 	for (uint32_t n = 0; n < limit; ++n) {
 
 		/* XXX we should use atomic exchange here */
@@ -2497,7 +2508,6 @@ IO::meter ()
 		
 		_max_peak_power[n] = max (new_peak, _max_peak_power[n]);
 		
-		
 		if (Config->get_meter_falloff() == 0.0f || new_peak > _visible_peak_power[n]) {
 			_visible_peak_power[n] = new_peak;
 		} else {
@@ -2715,3 +2725,11 @@ IO::find_output_port_hole ()
 	
 	return n;
 }
+
+void
+IO::set_active (bool yn)
+{
+	_active = yn; 
+	 active_changed(); /* EMIT SIGNAL */
+}
+
