@@ -61,6 +61,12 @@ Region::Region (nframes_t start, nframes_t length, const string& name, layer_t l
 	_start = start; 
 	_sync_position = _start;
 	_length = length; 
+	_last_length = length; 
+	_ancestral_start = start; 
+	_ancestral_length = length; 
+	_stretch = 1.0;
+	_shift = 1.0;
+	_last_position = 0; 
 	_position = 0; 
 	_layer = layer;
 	_read_data_count = 0;
@@ -83,7 +89,13 @@ Region::Region (boost::shared_ptr<const Region> other, nframes_t offset, nframes
 		_sync_position = _start;
 	}
 	_length = length; 
+	_last_length = length; 
+	_ancestral_start = other->_ancestral_start + offset;
+	_ancestral_length = length; 
+	_stretch = 1.0;
+	_shift = 1.0;
 	_name = name;
+	_last_position = 0; 
 	_position = 0; 
 	_layer = layer; 
 	_flags = Flag (flags & ~(Locked|WholeFile|Hidden));
@@ -111,7 +123,13 @@ Region::Region (boost::shared_ptr<const Region> other)
 	_start = other->_start;
 	_sync_position = other->_sync_position;
 	_length = other->_length; 
+	_last_length = other->_length; 
+	_ancestral_start = _start; 
+	_ancestral_length = _length; 
+	_stretch = 1.0;
+	_shift = 1.0;
 	_name = other->_name;
+	_last_position = other->_position; 
 	_position = other->_position; 
 	_layer = other->_layer; 
 	_flags = Flag (other->_flags & ~Locked);
@@ -126,7 +144,9 @@ Region::Region (const XMLNode& node)
 	_start = 0; 
 	_sync_position = _start;
 	_length = 0;
+	_last_length = 0;
 	_name = X_("error: XML did not reset this");
+	_last_position = 0; 
 	_position = 0; 
 	_layer = 0;
 	_flags = Flag (0);
@@ -178,6 +198,8 @@ Region::set_length (nframes_t len, void *src)
 			return;
 		}
 		
+
+		_last_length = _length;
 		_length = len;
 
 		_flags = Region::Flag (_flags & ~WholeFile);
@@ -256,6 +278,7 @@ Region::special_set_position (nframes_t pos)
 	   a way to store its "natural" or "captured" position.
 	*/
 
+	_position = _position;
 	_position = pos;
 }
 
@@ -267,6 +290,7 @@ Region::set_position (nframes_t pos, void *src)
 	}
 
 	if (_position != pos) {
+		_last_position = _position;
 		_position = pos;
 
 		/* check that the new _position wouldn't make the current
@@ -276,6 +300,7 @@ Region::set_position (nframes_t pos, void *src)
 		*/
 
 		if (max_frames - _length < _position) {
+			_last_length = _length;
 			_length = max_frames - _position;
 		}
 	}
@@ -295,6 +320,7 @@ Region::set_position_on_top (nframes_t pos, void *src)
 	}
 
 	if (_position != pos) {
+		_last_position = _position;
 		_position = pos;
 	}
 
@@ -312,7 +338,7 @@ Region::set_position_on_top (nframes_t pos, void *src)
 }
 
 void
-Region::nudge_position (long n, void *src)
+Region::nudge_position (nframes64_t n, void *src)
 {
 	if (_flags & Locked) {
 		return;
@@ -322,6 +348,8 @@ Region::nudge_position (long n, void *src)
 		return;
 	}
 	
+	_last_position = _position;
+
 	if (n > 0) {
 		if (_position > max_frames - n) {
 			_position = max_frames;
@@ -339,6 +367,15 @@ Region::nudge_position (long n, void *src)
 	send_change (PositionChanged);
 }
 
+void
+Region::set_ancestral_data (nframes64_t s, nframes64_t l, float st, float sh)
+{
+	_ancestral_length = l;
+	_ancestral_start = s;
+	_stretch = st;
+	_shift = sh;
+}
+
 void
 Region::set_start (nframes_t pos, void *src)
 {
@@ -528,10 +565,16 @@ Region::trim_to_internal (nframes_t position, nframes_t length, void *src)
 		what_changed = Change (what_changed|StartChanged);
 	}
 	if (_length != length) {
+		if (!_frozen) {
+			_last_length = _length;
+		}
 		_length = length;
 		what_changed = Change (what_changed|LengthChanged);
 	}
 	if (_position != position) {
+		if (!_frozen) {
+			_last_position = _position;
+		}
 		_position = position;
 		what_changed = Change (what_changed|PositionChanged);
 	}
@@ -586,7 +629,7 @@ Region::set_opaque (bool yn)
 		} else {
 			_flags = Flag (_flags & ~Opaque);
 		}
-		send_change (OpacityChanged);
+			send_change (OpacityChanged);
 	}
 }
 
@@ -659,14 +702,16 @@ Region::adjust_to_sync (nframes_t pos)
 {
 	int sync_dir;
 	nframes_t offset = sync_offset (sync_dir);
+
+	// cerr << "adjusting pos = " << pos << " to sync at " << _sync_position << " offset = " << offset << " with dir = " << sync_dir << endl;
 	
 	if (sync_dir > 0) {
 		if (max_frames - pos > offset) {
-			pos += offset;
+			pos -= offset;
 		}
 	} else {
 		if (pos > offset) {
-			pos -= offset;
+			pos += offset;
 		} else {
 			pos = 0;
 		}
@@ -737,7 +782,7 @@ Region::state (bool full_state)
 {
 	XMLNode *node = new XMLNode ("Region");
 	char buf[64];
-	char* fe = NULL;
+	const char* fe = NULL;
 
 	_id.print (buf, sizeof (buf));
 	node->add_property ("id", buf);
@@ -748,6 +793,14 @@ Region::state (bool full_state)
 	node->add_property ("length", buf);
 	snprintf (buf, sizeof (buf), "%u", _position);
 	node->add_property ("position", buf);
+	snprintf (buf, sizeof (buf), "%lu", _ancestral_start);
+	node->add_property ("ancestral-start", buf);
+	snprintf (buf, sizeof (buf), "%lu", _ancestral_length);
+	node->add_property ("ancestral-length", buf);
+	snprintf (buf, sizeof (buf), "%.12g", _stretch);
+	node->add_property ("stretch", buf);
+	snprintf (buf, sizeof (buf), "%.12g", _shift);
+	node->add_property ("shift", buf);
 	
 	switch (_first_edit) {
 	case EditChangesNothing:
@@ -760,7 +813,6 @@ Region::state (bool full_state)
 		fe = X_("id");
 		break;
 	default: /* should be unreachable but makes g++ happy */
-		cerr << "Odd region property found\n";
 		fe = X_("nothing");
 		break;
 	}
@@ -815,9 +867,11 @@ Region::set_live_state (const XMLNode& node, Change& what_changed, bool send)
 		sscanf (prop->value().c_str(), "%" PRIu32, &val);
 		if (val != _length) {
 			what_changed = Change (what_changed|LengthChanged);
+			_last_length = _length;
 			_length = val;
 		}
 	} else {
+		_last_length = _length;
 		_length = 1;
 	}
 
@@ -825,9 +879,11 @@ Region::set_live_state (const XMLNode& node, Change& what_changed, bool send)
 		sscanf (prop->value().c_str(), "%" PRIu32, &val);
 		if (val != _position) {
 			what_changed = Change (what_changed|PositionChanged);
+			_last_position = _position;
 			_position = val;
 		}
 	} else {
+		_last_position = _position;
 		_position = 0;
 	}
 
@@ -854,6 +910,32 @@ Region::set_live_state (const XMLNode& node, Change& what_changed, bool send)
 
 	/* XXX FIRST EDIT !!! */
 	
+	/* these 3 properties never change as a result of any editing */
+
+	if ((prop = node.property ("ancestral-start")) != 0) {
+		_ancestral_start = atoi (prop->value());
+	} else {
+		_ancestral_start = _start;
+	}
+
+	if ((prop = node.property ("ancestral-length")) != 0) {
+		_ancestral_length = atoi (prop->value());
+	} else {
+		_ancestral_length = _length;
+	}
+
+	if ((prop = node.property ("stretch")) != 0) {
+		_stretch = atof (prop->value());
+	} else {
+		_stretch = 1.0;
+	}
+
+	if ((prop = node.property ("shift")) != 0) {
+		_shift = atof (prop->value());
+	} else {
+		_shift = 1.0;
+	}
+
 	/* note: derived classes set flags */
 
 	if (_extra_xml) {
@@ -906,6 +988,8 @@ void
 Region::freeze ()
 {
 	_frozen++;
+	_last_length = _length;
+	_last_position = _position;
 }
 
 void
