@@ -27,6 +27,7 @@
 #include <cerrno>
 #include <unistd.h>
 #include <limits.h>
+#include <sys/time.h>
 
 #include <sigc++/bind.h>
 #include <sigc++/retype.h>
@@ -94,6 +95,8 @@ const char* Session::dead_sound_dir_name = X_("dead_sounds");
 const char* Session::interchange_dir_name = X_("interchange");
 const char* Session::export_dir_name = X_("export");
 
+bool Session::_disable_all_loaded_plugins = false;
+
 Session::compute_peak_t			Session::compute_peak 		= 0;
 Session::find_peaks_t			Session::find_peaks 		= 0;
 Session::apply_gain_to_buffer_t		Session::apply_gain_to_buffer 	= 0;
@@ -258,9 +261,9 @@ Session::find_session (string str, string& path, string& snapshot, bool& isnew)
 }
 
 Session::Session (AudioEngine &eng,
-		  string fullpath,
-		  string snapshot_name,
-		  string* mix_template)
+		  const string& fullpath,
+		  const string& snapshot_name,
+		  string mix_template)
 
 	: _engine (eng),
 	  _mmc_port (default_mmc_port),
@@ -282,15 +285,14 @@ Session::Session (AudioEngine &eng,
 
 	cerr << "Loading session " << fullpath << " using snapshot " << snapshot_name << " (1)" << endl;
 
-	n_physical_outputs = _engine.n_physical_outputs();
-	n_physical_inputs =  _engine.n_physical_inputs();
+	n_physical_audio_outputs = _engine.n_physical_audio_outputs();
+	n_physical_audio_inputs =  _engine.n_physical_audio_inputs();
 
 	first_stage_init (fullpath, snapshot_name);
 	
 	new_session = !g_file_test (_path.c_str(), GFileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR));
 	if (new_session) {
 		if (create (new_session, mix_template, compute_initial_length())) {
-			cerr << "create failed\n";
 			destroy ();
 			throw failed_constructor ();
 		}
@@ -344,15 +346,15 @@ Session::Session (AudioEngine &eng,
 
 	cerr << "Loading session " << fullpath << " using snapshot " << snapshot_name << " (2)" << endl;
 
-	n_physical_outputs = _engine.n_physical_outputs();
-	n_physical_inputs = _engine.n_physical_inputs();
+	n_physical_audio_outputs = _engine.n_physical_audio_outputs();
+	n_physical_audio_inputs = _engine.n_physical_audio_inputs();
 
-	if (n_physical_inputs) {
-		n_physical_inputs = max (requested_physical_in, n_physical_inputs);
+	if (n_physical_audio_inputs) {
+		n_physical_audio_inputs = max (requested_physical_in, n_physical_audio_inputs);
 	}
 
-	if (n_physical_outputs) {
-		n_physical_outputs = max (requested_physical_out, n_physical_outputs);
+	if (n_physical_audio_outputs) {
+		n_physical_audio_outputs = max (requested_physical_out, n_physical_audio_outputs);
 	}
 
 	first_stage_init (fullpath, snapshot_name);
@@ -360,7 +362,7 @@ Session::Session (AudioEngine &eng,
 	new_session = !g_file_test (_path.c_str(), GFileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR));
 
 	if (new_session) {
-		if (create (new_session, 0, initial_length)) {
+		if (create (new_session, string(), initial_length)) {
 			destroy ();
 			throw failed_constructor ();
 		}
@@ -390,7 +392,7 @@ Session::Session (AudioEngine &eng,
 		}
 		
 		if (!rl.empty()) {
-			add_routes (rl);
+			add_routes (rl, false);
 		}
 		
 	}
@@ -403,17 +405,12 @@ Session::Session (AudioEngine &eng,
 		throw failed_constructor ();
 	}
 	
-	store_recent_sessions(_name, _path);
+	store_recent_sessions (_name, _path);
 	
-	bool was_dirty = dirty ();
-
 	_state_of_the_state = StateOfTheState (_state_of_the_state & ~Dirty);
 
-	Config->ParameterChanged.connect (mem_fun (*this, &Session::config_changed));
 
-	if (was_dirty) {
-		DirtyChanged (); /* EMIT SIGNAL */
-	}
+	Config->ParameterChanged.connect (mem_fun (*this, &Session::config_changed));
 }
 
 Session::~Session ()
@@ -431,6 +428,7 @@ Session::destroy ()
 	remove_pending_capture_state ();
 
 	_state_of_the_state = StateOfTheState (CannotSave|Deletion);
+
 	_engine.remove_session ();
 
 	GoingAway (); /* EMIT SIGNAL */
@@ -707,7 +705,7 @@ Session::when_engine_running ()
 			
 			/* default state for Click */
 
-			first_physical_output = _engine.get_nth_physical_output (0);
+			first_physical_output = _engine.get_nth_physical_audio_output (0);
 			
 			if (first_physical_output.length()) {
 				if (_click_io->add_output_port (first_physical_output, this)) {
@@ -735,33 +733,33 @@ Session::when_engine_running ()
 
 	/* ONE: MONO */
 
-	for (uint32_t np = 0; np < n_physical_outputs; ++np) {
+	for (uint32_t np = 0; np < n_physical_audio_outputs; ++np) {
 		char buf[32];
 		snprintf (buf, sizeof (buf), _("out %" PRIu32), np+1);
 
 		Connection* c = new OutputConnection (buf, true);
 
 		c->add_port ();
-		c->add_connection (0, _engine.get_nth_physical_output (np));
+		c->add_connection (0, _engine.get_nth_physical_audio_output (np));
 
 		add_connection (c);
 	}
 
-	for (uint32_t np = 0; np < n_physical_inputs; ++np) {
+	for (uint32_t np = 0; np < n_physical_audio_inputs; ++np) {
 		char buf[32];
 		snprintf (buf, sizeof (buf), _("in %" PRIu32), np+1);
 
 		Connection* c = new InputConnection (buf, true);
 
 		c->add_port ();
-		c->add_connection (0, _engine.get_nth_physical_input (np));
+		c->add_connection (0, _engine.get_nth_physical_audio_input (np));
 
 		add_connection (c);
 	}
 
 	/* TWO: STEREO */
 
-	for (uint32_t np = 0; np < n_physical_outputs; np +=2) {
+	for (uint32_t np = 0; np < n_physical_audio_outputs; np +=2) {
 		char buf[32];
 		snprintf (buf, sizeof (buf), _("out %" PRIu32 "+%" PRIu32), np+1, np+2);
 
@@ -769,13 +767,13 @@ Session::when_engine_running ()
 
 		c->add_port ();
 		c->add_port ();
-		c->add_connection (0, _engine.get_nth_physical_output (np));
-		c->add_connection (1, _engine.get_nth_physical_output (np+1));
+		c->add_connection (0, _engine.get_nth_physical_audio_output (np));
+		c->add_connection (1, _engine.get_nth_physical_audio_output (np+1));
 
 		add_connection (c);
 	}
 
-	for (uint32_t np = 0; np < n_physical_inputs; np +=2) {
+	for (uint32_t np = 0; np < n_physical_audio_inputs; np +=2) {
 		char buf[32];
 		snprintf (buf, sizeof (buf), _("in %" PRIu32 "+%" PRIu32), np+1, np+2);
 
@@ -783,8 +781,8 @@ Session::when_engine_running ()
 
 		c->add_port ();
 		c->add_port ();
-		c->add_connection (0, _engine.get_nth_physical_input (np));
-		c->add_connection (1, _engine.get_nth_physical_input (np+1));
+		c->add_connection (0, _engine.get_nth_physical_audio_input (np));
+		c->add_connection (1, _engine.get_nth_physical_audio_input (np+1));
 
 		add_connection (c);
 	}
@@ -817,7 +815,7 @@ Session::when_engine_running ()
 			}
 			n = 0;
 			while ((int) _master_out->n_outputs() < _master_out->output_maximum()) {
-				if (_master_out->add_output_port (_engine.get_nth_physical_output (n), this)) {
+				if (_master_out->add_output_port (_engine.get_nth_physical_audio_output (n), this)) {
 					error << _("cannot setup master outputs")
 					      << endmsg;
 					break;
@@ -869,6 +867,7 @@ Session::when_engine_running ()
 	
 	_state_of_the_state = StateOfTheState (_state_of_the_state & ~(CannotSave|Dirty));
 
+
 	/* hook us up to the engine */
 
 	_engine.set_session (this);
@@ -879,9 +878,6 @@ Session::when_engine_running ()
 	osc->set_session (*this);
 #endif
     
-	_state_of_the_state = Clean;
-
-	DirtyChanged (); /* EMIT SIGNAL */
 }
 
 void
@@ -893,6 +889,7 @@ Session::hookup_io ()
 
 	_state_of_the_state = StateOfTheState (_state_of_the_state | InitialConnecting);
 
+
 	if (auditioner == 0) {
 		
 		/* we delay creating the auditioner till now because
@@ -926,7 +923,7 @@ Session::hookup_io ()
 		}
 		n = 0;
 		while ((int) _control_out->n_outputs() < _control_out->output_maximum()) {
-			if (_control_out->add_output_port (_engine.get_nth_physical_output (n), this)) {
+			if (_control_out->add_output_port (_engine.get_nth_physical_audio_output (n), this)) {
 				error << _("cannot set up master outputs")
 				      << endmsg;
 				break;
@@ -962,6 +959,7 @@ Session::hookup_io ()
 
 	_state_of_the_state = StateOfTheState (_state_of_the_state & ~InitialConnecting);
 
+
 	/* now handle the whole enchilada as if it was one
 	   graph reorder event.
 	*/
@@ -986,8 +984,15 @@ Session::playlist_length_changed ()
 }
 
 void
-Session::diskstream_playlist_changed (boost::shared_ptr<Diskstream> dstream)
+Session::diskstream_playlist_changed (boost::weak_ptr<Diskstream> wptr)
 {
+	boost::shared_ptr<Diskstream> dstream = wptr.lock();
+	
+	if (!dstream) {
+		return;
+
+	}
+
 	boost::shared_ptr<Playlist> playlist;
 
 	if ((playlist = dstream->playlist()) != 0) {
@@ -1252,7 +1257,7 @@ Session::disable_record (bool rt_context, bool force)
 
 	if ((rs = (RecordState) g_atomic_int_get (&_record_status)) != Disabled) {
 
-		if (!Config->get_latched_record_enable () || force) {
+		if ((!Config->get_latched_record_enable () && !play_loop) || force) {
 			g_atomic_int_set (&_record_status, Disabled);
 		} else {
 			if (rs == Recording) {
@@ -1283,15 +1288,18 @@ Session::disable_record (bool rt_context, bool force)
 void
 Session::step_back_from_record ()
 {
-	g_atomic_int_set (&_record_status, Enabled);
+	/* XXX really atomic compare+swap here */
+	if (g_atomic_int_get (&_record_status) == Recording) {
+		g_atomic_int_set (&_record_status, Enabled);
 
-	if (Config->get_monitoring_model() == HardwareMonitoring) {
-		boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
-		
-		for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
-		        if (Config->get_auto_input() && (*i)->record_enabled ()) {
-			        //cerr << "switching from input" << __FILE__ << __LINE__ << endl << endl;
-		                (*i)->monitor_input (false);   
+		if (Config->get_monitoring_model() == HardwareMonitoring) {
+			boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+			
+			for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
+				if (Config->get_auto_input() && (*i)->record_enabled ()) {
+					//cerr << "switching from input" << __FILE__ << __LINE__ << endl << endl;
+					(*i)->monitor_input (false);   
+				}
 			}
 		}
 	}
@@ -1392,7 +1400,9 @@ Session::set_frame_rate (nframes_t frames_per_second)
 
 	sync_time_vars();
 
-	Route::set_automation_interval ((jack_nframes_t) ceil ((double) frames_per_second * 0.25));
+	IO::set_automation_interval ((jack_nframes_t) ceil ((double) frames_per_second * (0.001 * Config->get_automation_interval())));
+
+	clear_clicks ();
 
 	// XXX we need some equivalent to this, somehow
 	// SndFileSource::setup_standard_crossfades (frames_per_second);
@@ -1677,8 +1687,8 @@ Session::new_audio_track (int input_channels, int output_channels, TrackMode mod
 	uint32_t nphysical_in;
 	uint32_t nphysical_out;
 
-	_engine.get_physical_outputs (physoutputs);
-	_engine.get_physical_inputs (physinputs);
+	_engine.get_physical_audio_outputs (physoutputs);
+	_engine.get_physical_audio_inputs (physinputs);
 	control_id = ntracks() + nbusses() + 1;
 
 	while (how_many) {
@@ -1702,13 +1712,13 @@ Session::new_audio_track (int input_channels, int output_channels, TrackMode mod
 		} while (track_id < (UINT_MAX-1));
 
 		if (Config->get_input_auto_connect() & AutoConnectPhysical) {
-			nphysical_in = min (n_physical_inputs, (uint32_t) physinputs.size());
+			nphysical_in = min (n_physical_audio_inputs, (uint32_t) physinputs.size());
 		} else {
 			nphysical_in = 0;
 		}
 		
 		if (Config->get_output_auto_connect() & AutoConnectPhysical) {
-			nphysical_out = min (n_physical_outputs, (uint32_t) physinputs.size());
+			nphysical_out = min (n_physical_audio_outputs, (uint32_t) physinputs.size());
 		} else {
 			nphysical_out = 0;
 		}
@@ -1724,7 +1734,7 @@ Session::new_audio_track (int input_channels, int output_channels, TrackMode mod
 				      << endmsg;
 				goto failed;
 			}
-
+	
 			if (nphysical_in) {
 				for (uint32_t x = 0; x < track->n_inputs() && x < nphysical_in; ++x) {
 					
@@ -1767,6 +1777,7 @@ Session::new_audio_track (int input_channels, int output_channels, TrackMode mod
 
 			new_routes.push_back (track);
 			ret.push_back (track);
+
 		}
 
 		catch (failed_constructor &err) {
@@ -1809,8 +1820,7 @@ Session::new_audio_track (int input_channels, int output_channels, TrackMode mod
 
   failed:
 	if (!new_routes.empty()) {
-		add_routes (new_routes, false);
-		save_state (_current_snapshot_name);
+		add_routes (new_routes, true);
 	}
 
 	return ret;
@@ -1864,8 +1874,8 @@ Session::new_audio_route (int input_channels, int output_channels, uint32_t how_
 	vector<string> physinputs;
 	vector<string> physoutputs;
 
-	_engine.get_physical_outputs (physoutputs);
-	_engine.get_physical_inputs (physinputs);
+	_engine.get_physical_audio_outputs (physoutputs);
+	_engine.get_physical_audio_inputs (physinputs);
 	control_id = ntracks() + nbusses() + 1;
 
 	while (how_many) {
@@ -1891,12 +1901,12 @@ Session::new_audio_route (int input_channels, int output_channels, uint32_t how_
 				goto failure;
 			}
 			
-			for (uint32_t x = 0; n_physical_inputs && x < bus->n_inputs(); ++x) {
+			for (uint32_t x = 0; n_physical_audio_inputs && x < bus->n_inputs(); ++x) {
 				
 				port = "";
 
 				if (Config->get_input_auto_connect() & AutoConnectPhysical) {
-						port = physinputs[((n+x)%n_physical_inputs)];
+						port = physinputs[((n+x)%n_physical_audio_inputs)];
 				} 
 				
 				if (port.length() && bus->connect_input (bus->input (x), port, this)) {
@@ -1904,12 +1914,12 @@ Session::new_audio_route (int input_channels, int output_channels, uint32_t how_
 				}
 			}
 			
-			for (uint32_t x = 0; n_physical_outputs && x < bus->n_outputs(); ++x) {
+			for (uint32_t x = 0; n_physical_audio_outputs && x < bus->n_outputs(); ++x) {
 				
 				port = "";
 				
 				if (Config->get_output_auto_connect() & AutoConnectPhysical) {
-					port = physoutputs[((n+x)%n_physical_outputs)];
+					port = physoutputs[((n+x)%n_physical_audio_outputs)];
 				} else if (Config->get_output_auto_connect() & AutoConnectMaster) {
 					if (_master_out) {
 						port = _master_out->input (x%_master_out->n_inputs())->name();
@@ -1944,8 +1954,7 @@ Session::new_audio_route (int input_channels, int output_channels, uint32_t how_
 
   failure:
 	if (!ret.empty()) {
-		add_routes (ret, false);
-		save_state (_current_snapshot_name);
+		add_routes (ret, true);
 	}
 
 	return ret;
@@ -2019,12 +2028,12 @@ Session::add_diskstream (boost::shared_ptr<Diskstream> dstream)
 		/* writer goes out of scope, copies ds back to main */
 	} 
 
-	dstream->PlaylistChanged.connect (sigc::bind (mem_fun (*this, &Session::diskstream_playlist_changed), dstream));
+	dstream->PlaylistChanged.connect (sigc::bind (mem_fun (*this, &Session::diskstream_playlist_changed), 
+						      boost::weak_ptr<Diskstream> (dstream)));
 	/* this will connect to future changes, and check the current length */
 	diskstream_playlist_changed (dstream);
 
 	dstream->prepare ();
-
 }
 
 void
@@ -2077,17 +2086,20 @@ Session::remove_route (shared_ptr<Route> route)
 			boost::shared_ptr<DiskstreamList> d = dsl.get_copy();
 			d->remove (ds);
 		}
+
+		diskstreams.flush ();
 	}
 
 	find_current_end ();
 	
+	// We need to disconnect the routes inputs and outputs 
+
+	route->disconnect_inputs (0);
+	route->disconnect_outputs (0);
+	
 	update_latency_compensation (false, false);
 	set_dirty();
 
-	// We need to disconnect the routes inputs and outputs 
-	route->disconnect_inputs(NULL);
-	route->disconnect_outputs(NULL);
-	
 	/* get rid of it from the dead wood collection in the route list manager */
 
 	/* XXX i think this is unsafe as it currently stands, but i am not sure. (pd, october 2nd, 2006) */
@@ -2559,7 +2571,9 @@ Session::add_region (boost::shared_ptr<Region> region)
 	{ 
 		Glib::Mutex::Lock lm (region_lock);
 
-		if ((ar = boost::dynamic_pointer_cast<AudioRegion> (region)) != 0) {
+		if (region == 0) {
+			error << _("Session::add_region() ignored a null region. Warning: you might have lost a region.") << endmsg;
+		} else if ((ar = boost::dynamic_pointer_cast<AudioRegion> (region)) != 0) {
 
 			AudioRegionList::iterator x;
 
@@ -2857,7 +2871,6 @@ Session::source_by_path_and_channel (const Glib::ustring& path, uint16_t chn)
 	Glib::Mutex::Lock lm (audio_source_lock);
 
 	for (AudioSourceList::iterator i = audio_sources.begin(); i != audio_sources.end(); ++i) {
-		cerr << "comparing " << path << " with " << i->second->name() << endl;
 		boost::shared_ptr<AudioFileSource> afs = boost::dynamic_pointer_cast<AudioFileSource>(i->second);
 
 		if (afs && afs->path() == path && chn == afs->channel()) {
@@ -2868,13 +2881,13 @@ Session::source_by_path_and_channel (const Glib::ustring& path, uint16_t chn)
 	return boost::shared_ptr<Source>();
 }
 
-string
-Session::peak_path_from_audio_path (string audio_path) const
+Glib::ustring
+Session::peak_path (Glib::ustring base) const
 {
-	string res;
-
+	Glib::ustring res;
+	
 	res = peak_dir ();
-	res += PBD::basename_nosuffix (audio_path);
+	res += base;
 	res += ".peak";
 
 	return res;
@@ -3285,7 +3298,7 @@ Session::remove_empty_sounds ()
 {
 	PathScanner scanner;
 
-	vector<string *>* possible_audiofiles = scanner (sound_dir(), "\\.(wav|aiff|caf|w64)$", false, true);
+	vector<string *>* possible_audiofiles = scanner (sound_dir(), "\\.(wav|aiff|caf|w64|L|R)$", false, true);
 	
 	Glib::Mutex::Lock lm (audio_source_lock);
 	
@@ -3311,12 +3324,12 @@ Session::remove_empty_sounds ()
 			continue;
 		}
 			
-		if (AudioFileSource::is_empty (*this, *(*i))) {
+		if (AudioFileSource::is_empty (*this, **i)) {
 
 			unlink ((*i)->c_str());
 			
-			string peak_path = peak_path_from_audio_path (**i);
-			unlink (peak_path.c_str());
+			Glib::ustring peakpath = peak_path (PBD::basename_nosuffix (**i));
+			unlink (peakpath.c_str());
 		}
 
 		delete* i;
@@ -3517,6 +3530,10 @@ Session::available_capture_duration ()
 		sample_bytes_on_disk = 3.0;
 		break;
 
+	case FormatInt16:
+		sample_bytes_on_disk = 2.0;
+		break;
+
 	default: 
 		/* impossible, but keep some gcc versions happy */
 		fatal << string_compose (_("programming error: %1"),
@@ -4034,3 +4051,19 @@ Session::compute_initial_length ()
 	return _engine.frame_rate() * 60 * 5;
 }
 
+void
+Session::sync_order_keys ()
+{
+	if (!Config->get_sync_all_route_ordering()) {
+		/* leave order keys as they are */
+		return;
+	}
+
+	boost::shared_ptr<RouteList> r = routes.reader ();
+
+	for (RouteList::iterator i = r->begin(); i != r->end(); ++i) {
+		(*i)->sync_order_keys ();
+	}
+
+	Route::SyncOrderKeys (); // EMIT SIGNAL
+}
