@@ -44,12 +44,12 @@
 #endif
 
 #include <glibmm.h>
+#include <glibmm/thread.h>
 
 #include <midi++/mmc.h>
 #include <midi++/port.h>
-#include <pbd/error.h>
 
-#include <glibmm/thread.h>
+#include <pbd/error.h>
 #include <pbd/pathscanner.h>
 #include <pbd/pthread_utils.h>
 #include <pbd/strsplit.h>
@@ -117,6 +117,9 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 		_path += '/';
 	}
 
+	set_history_depth (Config->get_history_depth());
+	
+
 	/* these two are just provisional settings. set_state()
 	   will likely override them.
 	*/
@@ -135,6 +138,7 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 	transport_sub_state = 0;
 	_transport_frame = 0;
 	last_stop_frame = 0;
+	_requested_return_frame = -1;
 	end_location = new Location (0, 0, _("end"), Location::Flags ((Location::IsMark|Location::IsEnd)));
 	start_location = new Location (0, 0, _("start"), Location::Flags ((Location::IsMark|Location::IsStart)));
 	_end_location_is_free = true;
@@ -159,8 +163,10 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 	_worst_output_latency = 0;
 	_worst_input_latency = 0;
 	_worst_track_latency = 0;
-	_state_of_the_state = StateOfTheState(CannotSave|InitialConnecting|Loading|Deletion);
+	_state_of_the_state = StateOfTheState(CannotSave|InitialConnecting|Loading);
+
 	_slave = 0;
+	_silent = false;
 	butler_mixdown_buffer = 0;
 	butler_gain_buffer = 0;
 	mmc = 0;
@@ -282,6 +288,7 @@ Session::second_stage_init (bool new_session)
 
 	// set_state() will call setup_raid_path(), but if it's a new session we need
 	// to call setup_raid_path() here.
+
 	if (state_tree) {
 		if (set_state (*state_tree->root())) {
 			return -1;
@@ -302,7 +309,7 @@ Session::second_stage_init (bool new_session)
 
 	_state_of_the_state = StateOfTheState (_state_of_the_state|CannotSave|Loading);
 
- 	// set_auto_input (true);
+
 	_locations.changed.connect (mem_fun (this, &Session::locations_changed));
 	_locations.added.connect (mem_fun (this, &Session::locations_added));
 	setup_click_sounds (0);
@@ -318,11 +325,10 @@ Session::second_stage_init (bool new_session)
 	}
 
 	/* handle this one in a different way than all others, so that its clear what happened */
-	
+
 	catch (AudioEngine::PortRegistrationFailure& err) {
-		error << _("Unable to create all required ports")
-		      << endmsg;
-		return -1;
+		destroy ();
+		throw;
 	}
 
 	catch (...) {
@@ -341,6 +347,17 @@ Session::second_stage_init (bool new_session)
 	} else {
 		_end_location_is_free = false;
 	}
+
+	_state_of_the_state = Clean;
+
+	
+	DirtyChanged (); /* EMIT SIGNAL */
+
+	if (state_was_pending) {
+		save_state (_current_snapshot_name);
+		remove_pending_capture_state ();
+		state_was_pending = false;
+	}
 	
 	return 0;
 }
@@ -452,7 +469,7 @@ Session::setup_raid_path (string path)
 }
 
 int
-Session::create (bool& new_session, string* mix_template, nframes_t initial_length)
+Session::create (bool& new_session, const string& mix_template, nframes_t initial_length)
 {
 	string dir;
 
@@ -497,8 +514,8 @@ Session::create (bool& new_session, string* mix_template, nframes_t initial_leng
 
 	/* check new_session so we don't overwrite an existing one */
 
-	if (mix_template) {
-		std::string in_path = *mix_template;
+	if (!mix_template.empty()) {
+		std::string in_path = mix_template;
 
 		ifstream in(in_path.c_str());
 
@@ -542,9 +559,8 @@ Session::create (bool& new_session, string* mix_template, nframes_t initial_leng
 
 	_state_of_the_state = Clean;
 
-	if (save_state (_current_snapshot_name)) {
-		return -1;
-	}
+
+	save_state ("");
 
 	return 0;
 }
@@ -712,6 +728,7 @@ Session::save_state (string snapshot_name, bool pending)
 		bool was_dirty = dirty();
 
 		_state_of_the_state = StateOfTheState (_state_of_the_state & ~Dirty);
+
 		
 		if (was_dirty) {
 			DirtyChanged (); /* EMIT SIGNAL */
@@ -814,13 +831,17 @@ Session::load_state (string snapshot_name)
 		backup_path += "-1";
 		backup_path += _statefile_suffix;
 
-		info << string_compose (_("Copying old session file %1 to %2\nUse %2 with Ardour versions before 2.0 from now on"),
-					xmlpath, backup_path) 
-		     << endmsg;
+		/* don't make another copy if it already exists */
 
-		copy_file (xmlpath, backup_path);
-
-		/* if it fails, don't worry. right? */
+		if (!Glib::file_test (backup_path, Glib::FILE_TEST_EXISTS)) {
+			info << string_compose (_("Copying old session file %1 to %2\nUse %2 with Ardour versions before 2.0 from now on"),
+						xmlpath, backup_path) 
+			     << endmsg;
+			
+			copy_file (xmlpath, backup_path);
+			
+			/* if it fails, don't worry. right? */
+		}
 	}
 
 	return 0;
@@ -895,7 +916,7 @@ Session::state(bool full_state)
 	// store libardour version, just in case
 	char buf[16];
 	snprintf(buf, sizeof(buf)-1, "%d.%d.%d", 
-		 libardour_major_version, libardour_minor_version, libardour_micro_version);
+		 libardour2_major_version, libardour2_minor_version, libardour2_micro_version);
 	node->add_property("version", string(buf));
 		
 	/* store configuration settings */
@@ -1121,6 +1142,7 @@ Session::set_state (const XMLNode& node)
 	int ret = -1;
 
 	_state_of_the_state = StateOfTheState (_state_of_the_state|CannotSave);
+
 	
 	if (node.name() != X_("Session")){
 		fatal << _("programming error: Session: incorrect XML node sent to set_state()") << endmsg;
@@ -1306,14 +1328,6 @@ Session::set_state (const XMLNode& node)
 
 	StateReady (); /* EMIT SIGNAL */
 
-	_state_of_the_state = Clean;
-
-	if (state_was_pending) {
-		save_state (_current_snapshot_name);
-		remove_pending_capture_state ();
-		state_was_pending = false;
-	}
-
 	return 0;
 
   out:
@@ -1343,7 +1357,7 @@ Session::load_routes (const XMLNode& node)
 		new_routes.push_back (route);
 	}
 
-	add_routes (new_routes);
+	add_routes (new_routes, false);
 
 	return 0;
 }
@@ -1377,7 +1391,14 @@ Session::load_regions (const XMLNode& node)
 
 	for (niter = nlist.begin(); niter != nlist.end(); ++niter) {
 		if ((region = XMLRegionFactory (**niter, false)) == 0) {
-			error << _("Session: cannot create Region from XML description.") << endmsg;
+			error << _("Session: cannot create Region from XML description.");
+			const XMLProperty *name = (**niter).property("name");
+
+			if (name) {
+				error << " " << string_compose (_("Can not load state for region '%1'"), name->value());
+			}
+
+			error << endmsg;
 		}
 	}
 
@@ -1391,6 +1412,7 @@ Session::XMLRegionFactory (const XMLNode& node, bool full)
 	boost::shared_ptr<Source> source;
 	boost::shared_ptr<AudioSource> as;
 	SourceList sources;
+	SourceList master_sources;
 	uint32_t nchans = 1;
 	char buf[128];
 	
@@ -1451,7 +1473,27 @@ Session::XMLRegionFactory (const XMLNode& node, bool full)
 			sources.push_back (as);
 		}
 	}
-	
+
+	for (uint32_t n=0; n < nchans; ++n) {
+		snprintf (buf, sizeof(buf), X_("master-source-%d"), n);
+		if ((prop = node.property (buf)) != 0) {
+		
+			PBD::ID id2 (prop->value());
+			
+			if ((source = source_by_id (id2)) == 0) {
+				error << string_compose(_("Session: XMLNode describing a AudioRegion references an unknown source id =%1"), id2) << endmsg;
+				return boost::shared_ptr<AudioRegion>();
+			}
+			
+			as = boost::dynamic_pointer_cast<AudioSource>(source);
+			if (!as) {
+				error << string_compose(_("Session: XMLNode describing a AudioRegion references a non-audio source id =%1"), id2) << endmsg;
+				return boost::shared_ptr<AudioRegion>();
+			}
+			master_sources.push_back (as);
+		} 
+	}
+
 	try {
 		boost::shared_ptr<AudioRegion> region (boost::dynamic_pointer_cast<AudioRegion> (RegionFactory::create (sources, node)));
 
@@ -1466,6 +1508,13 @@ Session::XMLRegionFactory (const XMLNode& node, bool full)
 			}
 		}
 
+		if (!master_sources.empty()) {
+			if (master_sources.size() != nchans) {
+				error << _("Session: XMLNode describing an AudioRegion is missing some master sources; ignored") << endmsg;
+			} else {
+				region->set_master_sources (master_sources);
+			}
+		}
 
 		return region;
 						       
@@ -2263,6 +2312,10 @@ Session::commit_reversible_command (Command *cmd)
 		current_trans->add_command (cmd);
 	}
 
+	if (current_trans->empty()) {
+		return;
+	}
+
 	gettimeofday (&now, 0);
 	current_trans->set_timestamp (now);
 
@@ -2591,6 +2644,7 @@ Session::cleanup_sources (Session::cleanup_report& rep)
 	int ret = -1;
 		
 	_state_of_the_state = (StateOfTheState) (_state_of_the_state | InCleanup);
+
 	
 	/* step 1: consider deleting all unused playlists */
 
@@ -2830,6 +2884,7 @@ Session::cleanup_sources (Session::cleanup_report& rep)
 
   out:
 	_state_of_the_state = (StateOfTheState) (_state_of_the_state & ~InCleanup);
+
 	return ret;
 }
 
@@ -2901,6 +2956,7 @@ Session::set_dirty ()
 
 	_state_of_the_state = StateOfTheState (_state_of_the_state | Dirty);
 
+
 	if (!was_dirty) {
 		DirtyChanged(); /* EMIT SIGNAL */
 	}
@@ -2914,6 +2970,7 @@ Session::set_clean ()
 	
 	_state_of_the_state = Clean;
 
+
 	if (was_dirty) {
 		DirtyChanged(); /* EMIT SIGNAL */
 	}
@@ -2923,6 +2980,7 @@ void
 Session::set_deletion_in_progress ()
 {
 	_state_of_the_state = StateOfTheState (_state_of_the_state | Deletion);
+
 }
 
 void
@@ -2975,7 +3033,6 @@ Session::add_instant_xml (XMLNode& node, const std::string& dir)
 	Config->add_instant_xml (node, get_user_ardour_path());
 }
 
-
 int 
 Session::save_history (string snapshot_name)
 {
@@ -2983,8 +3040,6 @@ Session::save_history (string snapshot_name)
     string xml_path;
     string bak_path;
 
-    tree.set_root (&_history.get_state (Config->get_saved_history_depth()));
-
     if (snapshot_name.empty()) {
 	snapshot_name = _current_snapshot_name;
     }
@@ -2993,13 +3048,17 @@ Session::save_history (string snapshot_name)
 
     bak_path = xml_path + ".bak";
 
-    if ((access (xml_path.c_str(), F_OK) == 0) &&
-        (rename (xml_path.c_str(), bak_path.c_str())))
-    {
+    if (Glib::file_test (xml_path, Glib::FILE_TEST_EXISTS) && ::rename (xml_path.c_str(), bak_path.c_str())) {
         error << _("could not backup old history file, current history not saved.") << endmsg;
         return -1;
     }
 
+    if (!Config->get_save_history() || Config->get_saved_history_depth() < 0) {
+	    return 0;
+    }
+
+    tree.set_root (&_history.get_state (Config->get_saved_history_depth()));
+
     if (!tree.write (xml_path))
     {
         error << string_compose (_("history could not be saved to %1"), xml_path) << endmsg;
@@ -3037,8 +3096,7 @@ Session::restore_history (string snapshot_name)
     xmlpath = _path + snapshot_name + ".history";
     cerr << string_compose(_("Loading history from '%1'."), xmlpath) << endmsg;
 
-    if (access (xmlpath.c_str(), F_OK)) {
-	    info << string_compose (_("%1: no history file \"%2\" for this session."), _name, xmlpath) << endmsg;
+    if (!Glib::file_test (xmlpath, Glib::FILE_TEST_EXISTS)) {
 	    return 1;
     }
 
@@ -3157,21 +3215,19 @@ Session::config_changed (const char* parameter_name)
 
 	} else if (PARAM_IS ("use-video-sync")) {
 
-		if (transport_stopped()) {
-			if (Config->get_use_video_sync()) {
-				waiting_for_sync_offset = true;
-			}
-		}
+		waiting_for_sync_offset = Config->get_use_video_sync();
 
 	} else if (PARAM_IS ("mmc-control")) {
 
 		poke_midi_thread ();
 
-	} else if (PARAM_IS ("mmc-device-id")) {
+	} else if (PARAM_IS ("mmc-device-id") || PARAM_IS ("mmc-receive-device-id")) {
 
-		if (mmc) {
-			mmc->set_device_id (Config->get_mmc_device_id());
-		}
+		set_mmc_receive_device_id (Config->get_mmc_receive_device_id());
+
+	} else if (PARAM_IS ("mmc-send-device-id")) {
+
+		set_mmc_send_device_id (Config->get_mmc_send_device_id());
 
 	} else if (PARAM_IS ("midi-control")) {
 		
@@ -3282,6 +3338,12 @@ Session::config_changed (const char* parameter_name)
 		set_slave_source (Config->get_slave_source());
 	} else if (PARAM_IS ("remote-model")) {
 		set_remote_control_ids ();
+	} else if (PARAM_IS ("denormal-model")) {
+		setup_fpu ();
+	} else if (PARAM_IS ("history-depth")) {
+		set_history_depth (Config->get_history_depth());
+	} else if (PARAM_IS ("sync-all-route-ordering")) {
+		sync_order_keys ();
 	}
 
 	set_dirty ();
@@ -3289,3 +3351,9 @@ Session::config_changed (const char* parameter_name)
 #undef PARAM_IS
 
 }
+
+void
+Session::set_history_depth (uint32_t d)
+{
+	_history.set_depth (d);
+}
